@@ -26,7 +26,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.util.Arrays;
-import java.util.HashMap;
+import java.util.Collections;
 import java.util.Map;
 
 import org.apache.hadoop.fs.FileSystem;
@@ -178,6 +178,24 @@ public class Connection implements Closeable
 		
 		setLocalConfigs();
 	}
+
+	/**
+	 * Sets a boolean flag indicating if runtime statistics should be gathered
+	 * Same behavior as in "MLContext.setStatistics()"
+	 *
+	 * @param stats boolean value with true indicating statistics should be gathered
+	 */
+	public void setStatistics(boolean stats) { DMLScript.STATISTICS = stats; }
+
+	/**
+	 * Sets a boolean flag indicating if memory profiling statistics should be
+	 * gathered. The option is false by default.
+	 * @param stats boolean value with true indicating memory statistics should be gathered
+	 */
+	public void gatherMemStats(boolean stats) {
+		DMLScript.STATISTICS = stats || DMLScript.STATISTICS;
+		DMLScript.JMLC_MEM_STATISTICS = stats;
+	}
 	
 	/**
 	 * Prepares (precompiles) a script and registers input and output variables.
@@ -186,11 +204,8 @@ public class Connection implements Closeable
 	 * @param inputs string array of input variables to register
 	 * @param outputs string array of output variables to register
 	 * @return PreparedScript object representing the precompiled script
-	 * @throws DMLException if DMLException occurs
 	 */
-	public PreparedScript prepareScript( String script, String[] inputs, String[] outputs) 
-		throws DMLException 
-	{
+	public PreparedScript prepareScript( String script, String[] inputs, String[] outputs) {
 		return prepareScript(script, inputs, outputs, false);
 	}
 	
@@ -202,12 +217,9 @@ public class Connection implements Closeable
 	 * @param outputs string array of output variables to register
 	 * @param parsePyDML {@code true} if PyDML, {@code false} if DML
 	 * @return PreparedScript object representing the precompiled script
-	 * @throws DMLException if DMLException occurs
 	 */
-	public PreparedScript prepareScript( String script, String[] inputs, String[] outputs, boolean parsePyDML) 
-		throws DMLException 
-	{
-		return prepareScript(script, new HashMap<String,String>(), inputs, outputs, parsePyDML);
+	public PreparedScript prepareScript( String script, String[] inputs, String[] outputs, boolean parsePyDML) {
+		return prepareScript(script, Collections.emptyMap(), inputs, outputs, parsePyDML);
 	}
 	
 	/**
@@ -219,11 +231,23 @@ public class Connection implements Closeable
 	 * @param outputs string array of output variables to register
 	 * @param parsePyDML {@code true} if PyDML, {@code false} if DML
 	 * @return PreparedScript object representing the precompiled script
-	 * @throws DMLException if DMLException occurs
 	 */
-	public PreparedScript prepareScript( String script, Map<String, String> args, String[] inputs, String[] outputs, boolean parsePyDML) 
-		throws DMLException 
-	{
+	public PreparedScript prepareScript( String script, Map<String, String> args, String[] inputs, String[] outputs, boolean parsePyDML) {
+		return prepareScript(script, Collections.emptyMap(), args, inputs, outputs, parsePyDML);
+	}
+	
+	/**
+	 * Prepares (precompiles) a script, sets input parameter values, and registers input and output variables.
+	 * 
+	 * @param script string representing of the DML or PyDML script
+	 * @param nsscripts map (name, script) of the DML or PyDML namespace scripts
+	 * @param args map of input parameters ($) and their values
+	 * @param inputs string array of input variables to register
+	 * @param outputs string array of output variables to register
+	 * @param parsePyDML {@code true} if PyDML, {@code false} if DML
+	 * @return PreparedScript object representing the precompiled script
+	 */
+	public PreparedScript prepareScript(String script, Map<String,String> nsscripts, Map<String, String> args, String[] inputs, String[] outputs, boolean parsePyDML) {
 		DMLScript.SCRIPT_TYPE = parsePyDML ? ScriptType.PYDML : ScriptType.DML;
 		
 		//check for valid names of passed arguments
@@ -244,7 +268,8 @@ public class Connection implements Closeable
 		Program rtprog = null;
 		try {
 			//parsing
-			ParserWrapper parser = ParserFactory.createParser(parsePyDML ? ScriptType.PYDML : ScriptType.DML);
+			ParserWrapper parser = ParserFactory.createParser(
+				parsePyDML ? ScriptType.PYDML : ScriptType.DML, nsscripts);
 			DMLProgram prog = parser.parse(null, script, args);
 			
 			//language validate
@@ -360,10 +385,10 @@ public class Connection implements Closeable
 					jmtd.getInt(DataExpression.ROWBLOCKCOUNTPARAM) : -1;
 			int bclen = jmtd.containsKey(DataExpression.COLUMNBLOCKCOUNTPARAM)?
 					jmtd.getInt(DataExpression.COLUMNBLOCKCOUNTPARAM) : -1;
-			long nnz = jmtd.containsKey(DataExpression.READNUMNONZEROPARAM)?
-					jmtd.getLong(DataExpression.READNUMNONZEROPARAM) : -1;
+			long nnz = jmtd.containsKey(DataExpression.READNNZPARAM)?
+					jmtd.getLong(DataExpression.READNNZPARAM) : -1;
 			String format = jmtd.getString(DataExpression.FORMAT_TYPE);
-			InputInfo iinfo = InputInfo.stringExternalToInputInfo(format);			
+			InputInfo iinfo = InputInfo.stringExternalToInputInfo(format);
 		
 			//read matrix file
 			return readDoubleMatrix(fname, iinfo, rows, cols, brlen, bclen, nnz);
@@ -430,7 +455,9 @@ public class Connection implements Closeable
 	 * @throws IOException if IOException occurs
 	 */
 	public double[][] convertToDoubleMatrix(String input, int rows, int cols) throws IOException {
-		return convertToDoubleMatrix(IOUtilFunctions.toInputStream(input), rows, cols);
+		try( InputStream is = IOUtilFunctions.toInputStream(input) ) {
+			return convertToDoubleMatrix(is, rows, cols);
+		}
 	}
 	
 	/**
@@ -476,7 +503,9 @@ public class Connection implements Closeable
 	 * @throws IOException if IOException occurs
 	 */
 	public MatrixBlock convertToMatrix(String input, String meta) throws IOException {
-		return convertToMatrix(IOUtilFunctions.toInputStream(input), meta);
+		try( InputStream is = IOUtilFunctions.toInputStream(input) ) {
+			return convertToMatrix(is, meta);
+		}
 	}
 	
 	/**
@@ -518,7 +547,9 @@ public class Connection implements Closeable
 	 * @throws IOException if IOException occurs
 	 */
 	public MatrixBlock convertToMatrix(String input, int rows, int cols) throws IOException {
-		return convertToMatrix(IOUtilFunctions.toInputStream(input), rows, cols);
+		try( InputStream is = IOUtilFunctions.toInputStream(input) ) {
+			return convertToMatrix(is, rows, cols);
+		}
 	}
 	
 	/**
@@ -664,7 +695,9 @@ public class Connection implements Closeable
 	 * @throws IOException if IOException occurs
 	 */
 	public String[][] convertToStringFrame(String input, int rows, int cols) throws IOException {
-		return convertToStringFrame(IOUtilFunctions.toInputStream(input), rows, cols);
+		try( InputStream is = IOUtilFunctions.toInputStream(input) ) {
+			return convertToStringFrame(is, rows, cols);
+		}
 	}
 	
 	/**
@@ -710,7 +743,9 @@ public class Connection implements Closeable
 	 * @throws IOException if IOException occurs
 	 */
 	public FrameBlock convertToFrame(String input, String meta) throws IOException {
-		return convertToFrame(IOUtilFunctions.toInputStream(input), meta);
+		try( InputStream is = IOUtilFunctions.toInputStream(input) ) {
+			return convertToFrame(is, meta);
+		}
 	}
 	
 	/**
@@ -752,7 +787,9 @@ public class Connection implements Closeable
 	 * @throws IOException if IOException occurs
 	 */
 	public FrameBlock convertToFrame(String input, int rows, int cols) throws IOException {
-		return convertToFrame(IOUtilFunctions.toInputStream(input), rows, cols);
+		try( InputStream is = IOUtilFunctions.toInputStream(input) ) {
+			return convertToFrame(is, rows, cols);
+		}
 	}
 	
 	/**
