@@ -26,8 +26,7 @@ import java.util.Set;
 
 import org.apache.commons.lang3.StringUtils;
 import org.apache.sysml.api.DMLScript;
-import org.apache.sysml.api.DMLScript.DMLOptions;
-import org.apache.sysml.api.DMLScript.EvictionPolicy;
+import org.apache.sysml.api.DMLOptions;
 import org.apache.sysml.api.ScriptExecutorUtils;
 import org.apache.sysml.api.jmlc.JMLCUtils;
 import org.apache.sysml.api.mlcontext.MLContext.ExecutionType;
@@ -37,8 +36,6 @@ import org.apache.sysml.conf.ConfigurationManager;
 import org.apache.sysml.conf.DMLConfig;
 import org.apache.sysml.hops.HopsException;
 import org.apache.sysml.hops.OptimizerUtils;
-import org.apache.sysml.hops.OptimizerUtils.OptimizationLevel;
-import org.apache.sysml.hops.globalopt.GlobalOptimizerWrapper;
 import org.apache.sysml.hops.rewrite.ProgramRewriter;
 import org.apache.sysml.hops.rewrite.RewriteRemovePersistentReadWrite;
 import org.apache.sysml.lops.LopsException;
@@ -53,7 +50,6 @@ import org.apache.sysml.runtime.controlprogram.LocalVariableMap;
 import org.apache.sysml.runtime.controlprogram.Program;
 import org.apache.sysml.runtime.controlprogram.context.ExecutionContext;
 import org.apache.sysml.runtime.controlprogram.context.ExecutionContextFactory;
-import org.apache.sysml.runtime.instructions.gpu.context.GPUContextPool;
 import org.apache.sysml.utils.Explain;
 import org.apache.sysml.utils.Explain.ExplainCounts;
 import org.apache.sysml.utils.Explain.ExplainType;
@@ -95,22 +91,7 @@ import org.apache.sysml.utils.Statistics;
  * </ol>
  * <p>
  * Modifications to these steps can be accomplished by subclassing
- * ScriptExecutor. For example, the following code will turn off the global data
- * flow optimization check by subclassing ScriptExecutor and overriding the
- * globalDataFlowOptimization method.
- * </p>
- *
- * <code>ScriptExecutor scriptExecutor = new ScriptExecutor() {
- * <br>&nbsp;&nbsp;// turn off global data flow optimization check
- * <br>&nbsp;&nbsp;@Override
- * <br>&nbsp;&nbsp;protected void globalDataFlowOptimization() {
- * <br>&nbsp;&nbsp;&nbsp;&nbsp;return;
- * <br>&nbsp;&nbsp;}
- * <br>};
- * <br>ml.execute(script, scriptExecutor);</code>
- * <p>
- *
- * For more information, please see the {@link #execute} method.
+ * ScriptExecutor. For more information, please see the {@link #execute} method.
  */
 public class ScriptExecutor {
 
@@ -263,16 +244,7 @@ public class ScriptExecutor {
 			throw new RuntimeException(ex);
 		}
 
-		// set the GPUs to use for this process (a range, all GPUs, comma separated list or a specific GPU)
-		GPUContextPool.AVAILABLE_GPUS = config.getTextValue(DMLConfig.AVAILABLE_GPUS);
-
-		String evictionPolicy = config.getTextValue(DMLConfig.GPU_EVICTION_POLICY).toUpperCase();
-		try {
-			DMLScript.GPU_EVICTION_POLICY = EvictionPolicy.valueOf(evictionPolicy);
-		} 
-		catch(IllegalArgumentException e) {
-			throw new RuntimeException("Unsupported eviction policy:" + evictionPolicy);
-		}
+		DMLScript.setGlobalFlags(config);
 	}
 	
 
@@ -309,7 +281,6 @@ public class ScriptExecutor {
 	 * <li>{@link #constructLops()}</li>
 	 * <li>{@link #generateRuntimeProgram()}</li>
 	 * <li>{@link #showExplanation()}</li>
-	 * <li>{@link #globalDataFlowOptimization()}</li>
 	 * <li>{@link #countCompiledMRJobsAndSparkInstructions()}</li>
 	 * <li>{@link #initializeCachingAndScratchSpace()}</li>
 	 * <li>{@link #cleanupRuntimeProgram()}</li>
@@ -337,7 +308,6 @@ public class ScriptExecutor {
 		constructLops();
 		generateRuntimeProgram();
 		showExplanation();
-		globalDataFlowOptimization();
 		countCompiledMRJobsAndSparkInstructions();
 		initializeCachingAndScratchSpace();
 		cleanupRuntimeProgram();
@@ -476,23 +446,6 @@ public class ScriptExecutor {
 		}
 	}
 
-	/**
-	 * Optimize the program.
-	 */
-	protected void globalDataFlowOptimization() {
-		if (OptimizerUtils.isOptLevel(OptimizationLevel.O4_GLOBAL_TIME_MEMORY)) {
-			try {
-				runtimeProgram = GlobalOptimizerWrapper.optimizeProgram(dmlProgram, runtimeProgram);
-			} catch (DMLRuntimeException e) {
-				throw new MLContextException("Exception occurred during global data flow optimization", e);
-			} catch (HopsException e) {
-				throw new MLContextException("Exception occurred during global data flow optimization", e);
-			} catch (LopsException e) {
-				throw new MLContextException("Exception occurred during global data flow optimization", e);
-			}
-		}
-	}
-
 	/**
 	 * Parse the script into an ANTLR parse tree, and convert this parse tree
 	 * into a SystemML program. Parsing includes lexical/syntactic analysis.
@@ -568,11 +521,7 @@ public class ScriptExecutor {
 	protected void validateScript() {
 		try {
 			dmlTranslator.validateParseTree(dmlProgram);
-		} catch (LanguageException e) {
-			throw new MLContextException("Exception occurred while validating script", e);
-		} catch (ParseException e) {
-			throw new MLContextException("Exception occurred while validating script", e);
-		} catch (IOException e) {
+		} catch (LanguageException | ParseException e) {
 			throw new MLContextException("Exception occurred while validating script", e);
 		}
 	}
