@@ -29,9 +29,9 @@ import org.apache.commons.lang.math.IntRange;
 import org.apache.sysml.hops.OptimizerUtils;
 import org.apache.sysml.lops.Lop;
 import org.apache.sysml.parser.Expression.DataType;
-import org.apache.sysml.runtime.DMLRuntimeException;
 import org.apache.sysml.runtime.controlprogram.LocalVariableMap;
 import org.apache.sysml.runtime.controlprogram.caching.MatrixObject;
+import org.apache.sysml.runtime.controlprogram.caching.CacheableData.CacheStatus;
 import org.apache.sysml.runtime.instructions.Instruction;
 import org.apache.sysml.runtime.instructions.MRJobInstruction;
 import org.apache.sysml.runtime.instructions.cp.BreakPointInstruction;
@@ -278,7 +278,7 @@ public class DMLDebuggerFunctions {
 			System.out.println("Symbol table for current frame is empty");
 	}
 	
-	public void print(LocalVariableMap variables, String varname, String displayFunction, int rowIndex, int colIndex) throws DMLRuntimeException {
+	public void print(LocalVariableMap variables, String varname, String displayFunction, int rowIndex, int colIndex) {
 		if (varname == null) {
 			System.err.println("No matrix variable name entered.");
 			return;
@@ -292,7 +292,7 @@ public class DMLDebuggerFunctions {
 						
 						try {
 							mo = (MatrixObject) variables.get(varname);
-							if (mo.getStatusAsString().equals("EMPTY") && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
+							if (mo.getStatus()==CacheStatus.EMPTY && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
 								//TODO @jlugoma Need to add functionality to bring and display a block. 
 								System.err.println("ERROR: Matrix dimensions are too large to fit in main memory.");
 								return;
@@ -357,9 +357,8 @@ public class DMLDebuggerFunctions {
 	 * Print DML matrix variable in current frame (if existing)
 	 * @param variables Current frame variables
  	 * @param varname Variable name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public void printMatrixVariable(LocalVariableMap variables, String varname) throws DMLRuntimeException {
+	public void printMatrixVariable(LocalVariableMap variables, String varname) {
 		if (varname == null) {
 			System.err.println("No matrix variable name entered.");
 			return;
@@ -369,7 +368,7 @@ public class DMLDebuggerFunctions {
 				if (variables.get(varname).getDataType() == DataType.MATRIX) {
 					try {
 						MatrixObject mo = (MatrixObject) variables.get(varname);
-						if (mo.getStatusAsString().equals("EMPTY") && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
+						if (mo.getStatus()==CacheStatus.EMPTY && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
 							//TODO @jlugoma Need to add functionality to bring and display a block. 
 							System.err.println("ERROR: DML matrix/vector dimensions are too large to fit in main memory.");
 							return;
@@ -419,7 +418,7 @@ public class DMLDebuggerFunctions {
 					double cellValue;
 					try {
 						MatrixObject mo = (MatrixObject) variables.get(varname);
-						if (mo.getStatusAsString().equals("EMPTY") && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
+						if (mo.getStatus()==CacheStatus.EMPTY && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
 							//TODO @jlugoma Need to add functionality to bring and display a block. 
 							System.err.println("ERROR: DML matrix/vector dimensions are too large to fit in main memory.");
 							return;
@@ -466,14 +465,16 @@ public class DMLDebuggerFunctions {
 					double updatedCellValue;
 					try {
 						MatrixObject mo = (MatrixObject) variables.get(varname);
-						if (mo.getStatusAsString().equals("EMPTY") && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
+						if (mo.getStatus()==CacheStatus.EMPTY && (OptimizerUtils.estimateSizeExactSparsity(mo.getNumRows(), mo.getNumColumns(), mo.getSparsity()) > OptimizerUtils.getLocalMemBudget())) {
 							//TODO @jlugoma Need to add functionality to bring and display a block. 
 							System.err.println("ERROR: DML matrix/vector dimensions are too large to fit in main memory.");
 							return;
-						}						
-						MatrixBlock mb = mo.acquireModify();
+						}
+						MatrixBlock mb = mo.acquireRead();
+						mo.release();
 						mb.setValue(rowIndex, columnIndex, value);
 						updatedCellValue = mb.getValue(rowIndex, columnIndex);
+						mo.acquireModify(mb);
 						mo.release();
 					} catch (Exception e) {
 						System.err.println("Error processing DML matrix variable "+varname+". Certain matrix operations are disabled due to memory constraints or read-only restrictions.");
