@@ -22,7 +22,6 @@ package org.apache.sysml.hops;
 import java.util.ArrayList;
 
 import org.apache.sysml.api.DMLScript;
-import org.apache.sysml.hops.Hop.MultiThreadedHop;
 import org.apache.sysml.lops.Aggregate;
 import org.apache.sysml.lops.Aggregate.OperationTypes;
 import org.apache.sysml.lops.CombineUnary;
@@ -33,7 +32,6 @@ import org.apache.sysml.lops.Data;
 import org.apache.sysml.lops.Group;
 import org.apache.sysml.lops.Lop;
 import org.apache.sysml.lops.LopProperties.ExecType;
-import org.apache.sysml.lops.LopsException;
 import org.apache.sysml.lops.PartialAggregate;
 import org.apache.sysml.lops.PartialAggregate.CorrectionLocationType;
 import org.apache.sysml.lops.PickByCount;
@@ -49,13 +47,10 @@ import org.apache.sysml.runtime.matrix.MatrixCharacteristics;
  * 		Semantic: given a value, perform the operation (independent of other values)
  */
 
-public class UnaryOp extends Hop implements MultiThreadedHop
+public class UnaryOp extends MultiThreadedHop
 {
 	private OpOp1 _op = null;
 	
-	private int _maxNumThreads = -1; //-1 for unlimited
-	
-	
 	private UnaryOp() {
 		//default constructor for clone
 	}
@@ -72,7 +67,7 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	}
 
 	@Override
-	public void checkArity() throws HopsException {
+	public void checkArity() {
 		HopsException.check(_input.size() == 1, this, "should have arity 1 but has arity %d", _input.size());
 	}
 
@@ -88,16 +83,6 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 		return s;
 	}
 
-	@Override
-	public void setMaxNumThreads( int k ) {
-		_maxNumThreads = k;
-	}
-	
-	@Override
-	public int getMaxNumThreads() {
-		return _maxNumThreads;
-	}
-	
 	@Override
 	public boolean isGPUEnabled() {
 		if(!DMLScript.USE_ACCELERATOR)
@@ -126,8 +111,7 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	
 	@Override
 	public Lop constructLops()
-		throws HopsException, LopsException 
-	{		
+	{
 		//reuse existing lop
 		if( getLops() != null )
 			return getLops();
@@ -179,9 +163,10 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 				}
 				else //default unary 
 				{
-					int k = isCumulativeUnaryOperation() ? OptimizerUtils.getConstrainedNumThreads( _maxNumThreads ) : 1;
-					Unary unary1 = new Unary(input.constructLops(), HopsOpOp1LopsU.get(_op), 
-							                 getDataType(), getValueType(), et, k);
+					int k = isCumulativeUnaryOperation() || isExpensiveUnaryOperation() ?
+						OptimizerUtils.getConstrainedNumThreads( _maxNumThreads ) : 1;
+					Unary unary1 = new Unary(input.constructLops(),
+						HopsOpOp1LopsU.get(_op), getDataType(), getValueType(), et, k);
 					setOutputDimensions(unary1);
 					setLineNumbers(unary1);
 					setLops(unary1);
@@ -201,7 +186,6 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	
 
 	private Lop constructLopsMedian() 
-		throws HopsException, LopsException 
 	{
 		ExecType et = optFindExecType();
 
@@ -269,7 +253,6 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	}
 	
 	private Lop constructLopsIQM() 
-		throws HopsException, LopsException
 	{
 
 		ExecType et = optFindExecType();
@@ -380,11 +363,8 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	 * fit into the map task memory budget) or by creating custom job types.
 	 * 
 	 * @return low-level operator
-	 * @throws HopsException if HopsException occurs
-	 * @throws LopsException if LopsException occurs
 	 */
 	private Lop constructLopsMRCumulativeUnary() 
-		throws HopsException, LopsException 
 	{
 		Hop input = getInput().get(0);
 		long rlen = input.getDim1();
@@ -460,7 +440,6 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	}
 
 	private Lop constructLopsSparkCumulativeUnary() 
-		throws HopsException, LopsException 
 	{
 		Hop input = getInput().get(0);
 		long rlen = input.getDim1();
@@ -477,8 +456,7 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 		
 		//recursive preaggregation until aggregates fit into CP memory budget
 		while( ((2*OptimizerUtils.estimateSize(TEMP.getOutputParameters().getNumRows(), clen) + OptimizerUtils.estimateSize(1, clen)) 
-				 > OptimizerUtils.getLocalMemBudget()
-			   && TEMP.getOutputParameters().getNumRows()>1) || force )
+			> OptimizerUtils.getLocalMemBudget() && TEMP.getOutputParameters().getNumRows()>1) || force )
 		{
 			DATA.add(TEMP);
 	
@@ -489,7 +467,7 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 			preagg.getOutputParameters().setDimensions(rlenAgg, clen, brlen, bclen, -1);
 			setLineNumbers(preagg);
 			
-			TEMP = preagg;	
+			TEMP = preagg;
 			level++;
 			force = false; //in case of unknowns, generate one level
 		}
@@ -518,19 +496,20 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 		return TEMP;
 	}
 
-	private OperationTypes getCumulativeAggType()
-	{
+	private OperationTypes getCumulativeAggType() {
 		switch( _op ) {
-			case CUMSUM: 	return OperationTypes.KahanSum;
-			case CUMPROD: 	return OperationTypes.Product;
-			case CUMMIN: 	return OperationTypes.Min;
-			case CUMMAX: 	return OperationTypes.Max;
-			default: 		return null;
+			case CUMSUM:     return OperationTypes.KahanSum;
+			case CUMPROD:    return OperationTypes.Product;
+			case CUMSUMPROD: return OperationTypes.SumProduct;
+			case CUMMIN:     return OperationTypes.Min;
+			case CUMMAX:     return OperationTypes.Max;
+			default:         return null;
 		}
 	}
 
 	private double getCumulativeInitValue() {
 		switch( _op ) {
+			case CUMSUMPROD: 
 			case CUMSUM:  return 0;
 			case CUMPROD: return 1;
 			case CUMMIN:  return Double.POSITIVE_INFINITY;
@@ -601,6 +580,8 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 			{
 				ret = new long[]{mc.getRows(), mc.getCols(), mc.getNonZeros()};
 			}
+			else if( _op==OpOp1.CUMSUMPROD )
+				ret = new long[]{mc.getRows(), 1, -1};
 			else 
 				ret = new long[]{mc.getRows(), mc.getCols(), -1};
 		}
@@ -620,34 +601,39 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 		return ( _op == OpOp1.INVERSE );
 	}
 
-	public boolean isCumulativeUnaryOperation() 
-	{
-		return (   _op == OpOp1.CUMSUM 
-				|| _op == OpOp1.CUMPROD
-				|| _op == OpOp1.CUMMIN
-				|| _op == OpOp1.CUMMAX  );
+	public boolean isCumulativeUnaryOperation()  {
+		return (_op == OpOp1.CUMSUM 
+			|| _op == OpOp1.CUMPROD
+			|| _op == OpOp1.CUMMIN
+			|| _op == OpOp1.CUMMAX
+			|| _op == OpOp1.CUMSUMPROD);
 	}
 
 	public boolean isCastUnaryOperation() {
-		return (   _op == OpOp1.CAST_AS_MATRIX
-				|| _op == OpOp1.CAST_AS_SCALAR
-				|| _op == OpOp1.CAST_AS_FRAME
-				|| _op == OpOp1.CAST_AS_BOOLEAN
-				|| _op == OpOp1.CAST_AS_DOUBLE
-				|| _op == OpOp1.CAST_AS_INT    );
+		return (_op == OpOp1.CAST_AS_MATRIX
+			|| _op == OpOp1.CAST_AS_SCALAR
+			|| _op == OpOp1.CAST_AS_FRAME
+			|| _op == OpOp1.CAST_AS_BOOLEAN
+			|| _op == OpOp1.CAST_AS_DOUBLE
+			|| _op == OpOp1.CAST_AS_INT);
+	}
+	
+	public boolean isExpensiveUnaryOperation()  {
+		return (_op == OpOp1.EXP 
+			|| _op == OpOp1.LOG
+			|| _op == OpOp1.SIGMOID);
 	}
 	
 	@Override
 	protected ExecType optFindExecType() 
-		throws HopsException 
-	{		
+	{
 		checkAndSetForcedPlatform();
 	
 		ExecType REMOTE = OptimizerUtils.isSparkExecutionMode() ? ExecType.SPARK : ExecType.MR;
 		
-		if( _etypeForced != null ) 			
+		if( _etypeForced != null )
 		{
-			_etype = _etypeForced;		
+			_etype = _etypeForced;
 		}
 		else 
 		{
@@ -688,8 +674,9 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 		setRequiresRecompileIfNecessary();
 		
 		//ensure cp exec type for single-node operations
-		if( _op == OpOp1.PRINT || _op == OpOp1.ASSERT || _op == OpOp1.STOP 
-			|| _op == OpOp1.INVERSE || _op == OpOp1.EIGEN || _op == OpOp1.CHOLESKY || _op == OpOp1.SVD)
+		if( _op == OpOp1.PRINT || _op == OpOp1.ASSERT || _op == OpOp1.STOP
+			|| _op == OpOp1.INVERSE || _op == OpOp1.EIGEN || _op == OpOp1.CHOLESKY || _op == OpOp1.SVD
+			|| getInput().get(0).getDataType() == DataType.LIST )
 		{
 			_etype = ExecType.CP;
 		}
@@ -700,22 +687,31 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	@Override
 	public void refreshSizeInformation()
 	{
-		if ( getDataType() == DataType.SCALAR ) 
-		{
+		Hop input = getInput().get(0);
+		if ( getDataType() == DataType.SCALAR )  {
 			//do nothing always known
 		}
+		else if( (_op == OpOp1.CAST_AS_MATRIX || _op == OpOp1.CAST_AS_FRAME
+			|| _op == OpOp1.CAST_AS_SCALAR) && input.getDataType()==DataType.LIST ){
+			//handle two cases of list of scalars or list of single matrix
+			setDim1( input.getLength() > 1 ? input.getLength() : -1 );
+			setDim2( input.getLength() > 1 ? 1 : -1 );
+		}
 		else if( (_op == OpOp1.CAST_AS_MATRIX || _op == OpOp1.CAST_AS_FRAME)
-			&& getInput().get(0).getDataType()==DataType.SCALAR )
+			&& input.getDataType()==DataType.SCALAR )
 		{
 			//prevent propagating 0 from scalar (which would be interpreted as unknown)
 			setDim1( 1 );
 			setDim2( 1 );
 		}
+		else if ( _op==OpOp1.CUMSUMPROD ) {
+			setDim1(input.getDim1());
+			setDim2(1);
+		}
 		else //general case
 		{
 			// If output is a Matrix then this operation is of type (B = op(A))
 			// Dimensions of B are same as that of A, and sparsity may/maynot change
-			Hop input = getInput().get(0);
 			setDim1( input.getDim1() );
 			setDim2( input.getDim2() );
 			// cosh(0)=cos(0)=1, acos(0)=1.5707963267948966
@@ -732,7 +728,7 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 	@Override
 	public Object clone() throws CloneNotSupportedException 
 	{
-		UnaryOp ret = new UnaryOp();	
+		UnaryOp ret = new UnaryOp();
 		
 		//copy generic attributes
 		ret.clone(this, false);
@@ -760,7 +756,7 @@ public class UnaryOp extends Hop implements MultiThreadedHop
 		if( _op == OpOp1.PRINT )
 			return false;
 		
-		UnaryOp that2 = (UnaryOp)that;		
+		UnaryOp that2 = (UnaryOp)that;
 		return (   _op == that2._op
 				&& getInput().get(0) == that2.getInput().get(0));
 	}
