@@ -48,7 +48,6 @@ import org.apache.sysml.parser.FunctionStatement;
 import org.apache.sysml.parser.FunctionStatementBlock;
 import org.apache.sysml.parser.IfStatement;
 import org.apache.sysml.parser.IfStatementBlock;
-import org.apache.sysml.parser.ParseException;
 import org.apache.sysml.parser.StatementBlock;
 import org.apache.sysml.parser.WhileStatement;
 import org.apache.sysml.parser.WhileStatementBlock;
@@ -97,11 +96,12 @@ public class InterProceduralAnalysis
 	protected static final boolean PROPAGATE_SCALAR_LITERALS      = true; //propagate and replace scalar literals into functions
 	protected static final boolean APPLY_STATIC_REWRITES          = true; //apply static hop dag and statement block rewrites
 	protected static final int     INLINING_MAX_NUM_OPS           = 10;    //inline single-statement functions w/ #ops <= threshold, other than dataops and literals
+	protected static final boolean ELIMINATE_DEAD_CODE            = true; //remove dead code (e.g., assigments) not used later on
 	
 	static {
 		// for internal debugging only
 		if( LDEBUG ) {
-			Logger.getLogger("org.apache.sysml.hops.ipa.InterProceduralAnalysis")
+			Logger.getLogger("org.apache.sysml.hops.ipa")
 				.setLevel((Level) Level.DEBUG);
 		}
 	}
@@ -136,8 +136,12 @@ public class InterProceduralAnalysis
 		_passes.add(new IPAPassRemoveUnnecessaryCheckpoints());
 		_passes.add(new IPAPassRemoveConstantBinaryOps());
 		_passes.add(new IPAPassPropagateReplaceLiterals());
-		_passes.add(new IPAPassApplyStaticHopRewrites());
 		_passes.add(new IPAPassInlineFunctions());
+		_passes.add(new IPAPassEliminateDeadCode());
+		//note: apply rewrites last because statement block rewrites
+		//might merge relevant statement blocks in special cases, which 
+		//would require an update of the function call graph
+		_passes.add(new IPAPassApplyStaticHopRewrites());
 	}
 	
 	public InterProceduralAnalysis(StatementBlock sb) {
@@ -153,9 +157,8 @@ public class InterProceduralAnalysis
 	/**
 	 * Main interface to perform IPA over a given DML program.
 	 * 
-	 * @throws HopsException in case of compilation errors
 	 */
-	public void analyzeProgram() throws HopsException {
+	public void analyzeProgram() {
 		analyzeProgram(1); //single run
 	}
 	
@@ -163,11 +166,8 @@ public class InterProceduralAnalysis
 	 * Main interface to perform IPA over a given DML program.
 	 * 
 	 * @param repetitions number of IPA rounds 
-	 * @throws HopsException in case of compilation errors
 	 */
-	public void analyzeProgram(int repetitions) 
-		throws HopsException
-	{
+	public void analyzeProgram(int repetitions) {
 		//sanity check for valid number of repetitions
 		if( repetitions <= 0 )
 			throw new HopsException("Invalid number of IPA repetitions: " + repetitions);
@@ -186,9 +186,14 @@ public class InterProceduralAnalysis
 			//step 1: intra- and inter-procedural 
 			if( INTRA_PROCEDURAL_ANALYSIS ) {
 				//get unary dimension-preserving non-candidate functions
-				for( String tmp : fcallSizes.getInvalidFunctions() )
-					if( isUnarySizePreservingFunction(_prog.getFunctionStatementBlock(tmp)) )
+				//note: we have to guard against recursive functions because these might
+				//be seen at top-level as a unary size-preserving function but internally
+				//called with different sizes (e.g., common block-recursive cholesky)
+				for( String tmp : fcallSizes.getInvalidFunctions() ) {
+					if( !_fgraph.isRecursiveFunction(tmp)
+						&& isUnarySizePreservingFunction(_prog.getFunctionStatementBlock(tmp)) )
 						fcallSizes.addDimsPreservingFunction(tmp);
+				}
 				if( LOG.isDebugEnabled() )
 					LOG.debug("IPA: Extended FunctionCallSummary: \n" + fcallSizes);
 				
@@ -221,9 +226,7 @@ public class InterProceduralAnalysis
 			rmFuns.rewriteProgram(_prog, graph2, null);
 	}
 	
-	public Set<String> analyzeSubProgram() 
-		throws HopsException, ParseException
-	{
+	public Set<String> analyzeSubProgram() {
 		DMLTranslator.resetHopsDAGVisitStatus(_sb);
 		
 		//get function call size infos to obtain candidates for statistics propagation
@@ -239,9 +242,7 @@ public class InterProceduralAnalysis
 		return fcallSizes.getValidFunctions();
 	}
 	
-	private boolean isUnarySizePreservingFunction(FunctionStatementBlock fsb) 
-		throws HopsException 
-	{
+	private boolean isUnarySizePreservingFunction(FunctionStatementBlock fsb) {
 		FunctionStatement fstmt = (FunctionStatement) fsb.getStatement(0);
 		
 		//check unary functions over matrices
@@ -282,7 +283,6 @@ public class InterProceduralAnalysis
 	//////	
 
 	private void propagateStatisticsAcrossBlock( StatementBlock sb, LocalVariableMap callVars, FunctionCallSizeInfo fcallSizes, Set<String> fnStack )
-		throws HopsException
 	{
 		if (sb instanceof FunctionStatementBlock)
 		{
@@ -316,7 +316,7 @@ public class InterProceduralAnalysis
 			IfStatementBlock isb = (IfStatementBlock) sb;
 			IfStatement istmt = (IfStatement)isb.getStatement(0);
 			//old stats into predicate
-			propagateStatisticsAcrossPredicateDAG(isb.getPredicateHops(), callVars);			
+			propagateStatisticsAcrossPredicateDAG(isb.getPredicateHops(), callVars);
 			//check and propagate stats into body
 			LocalVariableMap oldCallVars = (LocalVariableMap) callVars.clone();
 			LocalVariableMap callVarsElse = (LocalVariableMap) callVars.clone();
@@ -380,11 +380,8 @@ public class InterProceduralAnalysis
 	 *
 	 * @param roots  List of HOPs.
 	 * @param vars  Map of variables eligible for propagation.
-	 * @throws HopsException  If a HopsException occurs.
 	 */
-	private static void propagateScalarsAcrossDAG(ArrayList<Hop> roots, LocalVariableMap vars)
-		throws HopsException
-	{
+	private static void propagateScalarsAcrossDAG(ArrayList<Hop> roots, LocalVariableMap vars) {
 		for (Hop hop : roots) {
 			try {
 				Recompiler.rReplaceLiterals(hop, vars, true);
@@ -394,9 +391,7 @@ public class InterProceduralAnalysis
 		}
 	}
 
-	private static void propagateStatisticsAcrossPredicateDAG( Hop root, LocalVariableMap vars ) 
-		throws HopsException
-	{
+	private static void propagateStatisticsAcrossPredicateDAG( Hop root, LocalVariableMap vars ) {
 		if( root == null )
 			return;
 		
@@ -417,11 +412,8 @@ public class InterProceduralAnalysis
 	 *
 	 * @param roots  List of HOP DAG root nodes.
 	 * @param vars  Map of variables eligible for propagation.
-	 * @throws HopsException  If a HopsException occurs.
 	 */
-	private static void propagateStatisticsAcrossDAG( ArrayList<Hop> roots, LocalVariableMap vars )
-		throws HopsException
-	{
+	private static void propagateStatisticsAcrossDAG( ArrayList<Hop> roots, LocalVariableMap vars ) {
 		if( roots == null )
 			return;
 		
@@ -452,11 +444,8 @@ public class InterProceduralAnalysis
 	 * @param callVars  Calling program's map of variables eligible for propagation.
 	 * @param fcallSizes function call summary
 	 * @param fnStack  Function stack to determine current scope.
-	 * @throws HopsException  If a HopsException occurs.
 	 */
-	private void propagateStatisticsIntoFunctions(DMLProgram prog, ArrayList<Hop> roots, LocalVariableMap callVars, FunctionCallSizeInfo fcallSizes, Set<String> fnStack)
-			throws HopsException
-	{
+	private void propagateStatisticsIntoFunctions(DMLProgram prog, ArrayList<Hop> roots, LocalVariableMap callVars, FunctionCallSizeInfo fcallSizes, Set<String> fnStack) {
 		for( Hop root : roots )
 			propagateStatisticsIntoFunctions(prog, root, callVars, fcallSizes, fnStack);
 	}
@@ -470,10 +459,8 @@ public class InterProceduralAnalysis
 	 * @param callVars  Calling program's map of variables eligible for propagation.
 	 * @param fcallSizes function call summary
 	 * @param fnStack  Function stack to determine current scope.
-	 * @throws HopsException  If a HopsException occurs.
 	 */
 	private void propagateStatisticsIntoFunctions(DMLProgram prog, Hop hop, LocalVariableMap callVars, FunctionCallSizeInfo fcallSizes, Set<String> fnStack ) 
-		throws HopsException
 	{
 		if( hop.isVisited() )
 			return;
@@ -492,8 +479,8 @@ public class InterProceduralAnalysis
 				FunctionStatementBlock fsb = prog.getFunctionStatementBlock(fop.getFunctionNamespace(), fop.getFunctionName());
 				FunctionStatement fstmt = (FunctionStatement)fsb.getStatement(0);
 				
-				if(  fcallSizes.isValidFunction(fkey) && 
-				    !fnStack.contains(fkey)  ) //prevent recursion	
+				if( fcallSizes.isValidFunction(fkey) && 
+					!fnStack.contains(fkey)  ) //prevent recursion
 				{
 					//maintain function call stack
 					fnStack.add(fkey);
@@ -506,7 +493,7 @@ public class InterProceduralAnalysis
 					propagateStatisticsAcrossBlock(fsb, tmpVars, fcallSizes, fnStack);
 					
 					//extract vars from symbol table, re-map and refresh main program
-					extractFunctionCallReturnStatistics(fstmt, fop, tmpVars, callVars, true);		
+					extractFunctionCallReturnStatistics(fstmt, fop, tmpVars, callVars, true);
 					
 					//maintain function call stack
 					fnStack.remove(fkey);
@@ -535,28 +522,29 @@ public class InterProceduralAnalysis
 	}
 	
 	private static void populateLocalVariableMapForFunctionCall( FunctionStatement fstmt, FunctionOp fop, LocalVariableMap callvars, LocalVariableMap vars, FunctionCallSizeInfo fcallSizes ) 
-		throws HopsException
 	{
-		ArrayList<DataIdentifier> inputVars = fstmt.getInputParams();
+		//note: due to arbitrary binding sequences of named function arguments,
+		//we cannot use the sequence as defined in the function signature
+		String[] funArgNames = fop.getInputVariableNames();
 		ArrayList<Hop> inputOps = fop.getInput();
 		String fkey = fop.getFunctionKey();
 		
-		for( int i=0; i<inputVars.size(); i++ )
+		for( int i=0; i<funArgNames.length; i++ )
 		{
 			//create mapping between input hops and vars
-			DataIdentifier dat = inputVars.get(i);
+			DataIdentifier dat = fstmt.getInputParam(funArgNames[i]);
 			Hop input = inputOps.get(i);
 			
 			if( input.getDataType()==DataType.MATRIX )
 			{
 				//propagate matrix characteristics
 				MatrixObject mo = new MatrixObject(ValueType.DOUBLE, null);
-				MatrixCharacteristics mc = new MatrixCharacteristics( input.getDim1(), input.getDim2(), 
+				MatrixCharacteristics mc = new MatrixCharacteristics( input.getDim1(), input.getDim2(),
 					ConfigurationManager.getBlocksize(), ConfigurationManager.getBlocksize(),
 					fcallSizes.isSafeNnz(fkey, i)?input.getNnz():-1 );
 				MetaDataFormat meta = new MetaDataFormat(mc,null,null);
-				mo.setMetaData(meta);	
-				vars.put(dat.getName(), mo);	
+				mo.setMetaData(meta);
+				vars.put(dat.getName(), mo);
 			}
 			else if( input.getDataType()==DataType.SCALAR )
 			{
@@ -570,7 +558,7 @@ public class InterProceduralAnalysis
 				//and input scalar is existing variable in symbol table
 				else if( PROPAGATE_SCALAR_VARS_INTO_FUN 
 					&& fcallSizes.getFunctionCallCount(fkey) == 1
-					&& input instanceof DataOp  ) 
+					&& input instanceof DataOp )
 				{
 					Data scalar = callvars.get(input.getName()); 
 					if( scalar != null && scalar instanceof ScalarObject ) {
@@ -592,19 +580,19 @@ public class InterProceduralAnalysis
 	 * @param callVars  Calling program's map of variables.
 	 * @param overwrite  Whether or not to overwrite variables in the
 	 *                      calling program's variable map.
-	 * @throws HopsException  If a HopsException occurs.
 	 */
-	private static void extractFunctionCallReturnStatistics( FunctionStatement fstmt, FunctionOp fop, LocalVariableMap tmpVars, LocalVariableMap callVars, boolean overwrite ) 
-		throws HopsException
-	{
+	private static void extractFunctionCallReturnStatistics( FunctionStatement fstmt, FunctionOp fop, LocalVariableMap tmpVars, LocalVariableMap callVars, boolean overwrite ) {
 		ArrayList<DataIdentifier> foutputOps = fstmt.getOutputParams();
 		String[] outputVars = fop.getOutputVariableNames();
 		String fkey = fop.getFunctionKey();
 		
 		try
 		{
-			for( int i=0; i<foutputOps.size(); i++ )
-			{
+			for( int i=0; i<foutputOps.size(); i++ ) {
+				//robustness for unbound outputs
+				if( outputVars.length <= i )
+					break;
+				
 				DataIdentifier di = foutputOps.get(i);
 				String fvarname = di.getName(); //name in function signature
 				String pvarname = outputVars[i]; //name in calling program
@@ -622,71 +610,55 @@ public class InterProceduralAnalysis
 					}
 				}
 				// Update or add to the calling program's variable map.
-				if( di.getDataType()==DataType.MATRIX && tmpVars.keySet().contains(fvarname) )
-				{
+				if( di.getDataType()==DataType.MATRIX && tmpVars.keySet().contains(fvarname) ) {
 					MatrixObject moIn = (MatrixObject) tmpVars.get(fvarname);
-					
-					if( !callVars.keySet().contains(pvarname) || overwrite ) //not existing so far
-					{
-						MatrixObject moOut = createOutputMatrix(moIn.getNumRows(), moIn.getNumColumns(), moIn.getNnz());	
+					if( !callVars.keySet().contains(pvarname) || overwrite ) { //not existing so far
+						MatrixObject moOut = createOutputMatrix(moIn.getNumRows(), moIn.getNumColumns(), moIn.getNnz());
 						callVars.put(pvarname, moOut);
 					}
-					else //already existing: take largest   
-					{
+					else { //already existing: take largest
 						Data dat = callVars.get(pvarname);
-						if( dat instanceof MatrixObject )
+						if( !(dat instanceof MatrixObject) )
+							continue;
+						MatrixObject moOut = (MatrixObject)dat;
+						MatrixCharacteristics mc = moOut.getMatrixCharacteristics();
+						if( OptimizerUtils.estimateSizeExactSparsity(mc.getRows(), mc.getCols(), (mc.getNonZeros()>0)?
+							OptimizerUtils.getSparsity(mc):1.0) 
+							< OptimizerUtils.estimateSize(moIn.getNumRows(), moIn.getNumColumns()) )
 						{
-							MatrixObject moOut = (MatrixObject)dat;
-							MatrixCharacteristics mc = moOut.getMatrixCharacteristics();
-							if( OptimizerUtils.estimateSizeExactSparsity(mc.getRows(), mc.getCols(), (mc.getNonZeros()>0)?
-								OptimizerUtils.getSparsity(mc):1.0) 
-								< OptimizerUtils.estimateSize(moIn.getNumRows(), moIn.getNumColumns()) )
-							{
-								//update statistics if necessary
-								mc.setDimension(moIn.getNumRows(), moIn.getNumColumns());
-								mc.setNonZeros(moIn.getNnz());
-							}
+							//update statistics if necessary
+							mc.setDimension(moIn.getNumRows(), moIn.getNumColumns());
+							mc.setNonZeros(moIn.getNnz());
 						}
-						
 					}
 				}
 			}
 		}
-		catch( Exception ex )
-		{
+		catch( Exception ex ) {
 			throw new HopsException( "Failed to extract output statistics of function "+fkey+".", ex);
 		}
 	}
 	
-	private static void extractFunctionCallUnknownReturnStatistics( FunctionStatement fstmt, FunctionOp fop, LocalVariableMap callVars ) 
-		throws HopsException
-	{
+	private static void extractFunctionCallUnknownReturnStatistics(FunctionStatement fstmt, FunctionOp fop, LocalVariableMap callVars) {
 		ArrayList<DataIdentifier> foutputOps = fstmt.getOutputParams();
 		String[] outputVars = fop.getOutputVariableNames();
 		String fkey = fop.getFunctionKey();
-		
-		try
-		{
-			for( int i=0; i<foutputOps.size(); i++ )
-			{
+		try {
+			//robustness for subset of bound output variables
+			int olen = Math.min(foutputOps.size(), outputVars.length);
+			for( int i=0; i<olen; i++ ) {
 				DataIdentifier di = foutputOps.get(i);
 				String pvarname = outputVars[i]; //name in calling program
-				
 				if( di.getDataType()==DataType.MATRIX )
-				{
-					MatrixObject moOut = createOutputMatrix(-1, -1, -1);
-					callVars.put(pvarname, moOut);
-				}
+					callVars.put(pvarname, createOutputMatrix(-1, -1, -1));
 			}
 		}
-		catch( Exception ex )
-		{
+		catch( Exception ex ) {
 			throw new HopsException( "Failed to extract output statistics of function "+fkey+".", ex);
 		}
 	}
 	
 	private static void extractFunctionCallEquivalentReturnStatistics( FunctionStatement fstmt, FunctionOp fop, LocalVariableMap callVars ) 
-		throws HopsException
 	{
 		try {
 			Hop input = fop.getInput().get(0);
@@ -700,7 +672,6 @@ public class InterProceduralAnalysis
 	}
 	
 	private static void extractExternalFunctionCallReturnStatistics( ExternalFunctionStatement fstmt, FunctionOp fop, LocalVariableMap callVars ) 
-		throws HopsException
 	{
 		String className = fstmt.getOtherParams().get(ExternalFunctionStatement.CLASS_NAME);
 
