@@ -28,8 +28,6 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.wink.json4j.JSONObject;
@@ -46,12 +44,13 @@ import org.apache.sysml.hops.Hop.DataGenMethod;
 import org.apache.sysml.hops.Hop.DataOpTypes;
 import org.apache.sysml.hops.Hop.FileFormatTypes;
 import org.apache.sysml.hops.Hop.OpOp1;
+import org.apache.sysml.hops.Hop.ReOrgOp;
 import org.apache.sysml.hops.HopsException;
 import org.apache.sysml.hops.IndexingOp;
 import org.apache.sysml.hops.LiteralOp;
 import org.apache.sysml.hops.MemoTable;
+import org.apache.sysml.hops.MultiThreadedHop;
 import org.apache.sysml.hops.OptimizerUtils;
-import org.apache.sysml.hops.ReorgOp;
 import org.apache.sysml.hops.UnaryOp;
 import org.apache.sysml.hops.codegen.SpoofCompiler;
 import org.apache.sysml.hops.rewrite.HopRewriteUtils;
@@ -60,7 +59,6 @@ import org.apache.sysml.lops.CSVReBlock;
 import org.apache.sysml.lops.DataGen;
 import org.apache.sysml.lops.Lop;
 import org.apache.sysml.lops.LopProperties.ExecType;
-import org.apache.sysml.lops.LopsException;
 import org.apache.sysml.lops.ReBlock;
 import org.apache.sysml.lops.compile.Dag;
 import org.apache.sysml.parser.DMLProgram;
@@ -84,7 +82,7 @@ import org.apache.sysml.runtime.controlprogram.caching.CacheableData;
 import org.apache.sysml.runtime.controlprogram.caching.FrameObject;
 import org.apache.sysml.runtime.controlprogram.caching.MatrixObject;
 import org.apache.sysml.runtime.controlprogram.context.ExecutionContext;
-import org.apache.sysml.runtime.controlprogram.parfor.ProgramConverter;
+import org.apache.sysml.runtime.util.ProgramConverter;
 import org.apache.sysml.runtime.controlprogram.parfor.opt.OptTreeConverter;
 import org.apache.sysml.runtime.controlprogram.parfor.stat.InfrastructureAnalyzer;
 import org.apache.sysml.runtime.instructions.Instruction;
@@ -120,8 +118,6 @@ import org.apache.sysml.utils.JSONHelper;
  */
 public class Recompiler 
 {
-	private static final Log LOG = LogFactory.getLog(Recompiler.class.getName());
-	
 	//Max threshold for in-memory reblock of text input [in bytes]
 	//reason: single-threaded text read at 20MB/s, 1GB input -> 50s (should exploit parallelism)
 	//note that we scale this threshold up by the degree of available parallelism
@@ -153,7 +149,6 @@ public class Recompiler
 	
 	public static ArrayList<Instruction> recompileHopsDag( StatementBlock sb, ArrayList<Hop> hops, 
 			LocalVariableMap vars, RecompileStatus status, boolean inplace, boolean replaceLit, long tid ) 
-		throws DMLRuntimeException, HopsException, LopsException
 	{
 		ArrayList<Instruction> newInst = null;
 
@@ -180,7 +175,6 @@ public class Recompiler
 	
 	public static ArrayList<Instruction> recompileHopsDag( Hop hop, LocalVariableMap vars, 
 			RecompileStatus status, boolean inplace, boolean replaceLit, long tid ) 
-		throws DMLRuntimeException, HopsException, LopsException
 	{
 		ArrayList<Instruction> newInst = null;
 
@@ -202,7 +196,6 @@ public class Recompiler
 	}
 	
 	public static ArrayList<Instruction> recompileHopsDag2Forced( StatementBlock sb, ArrayList<Hop> hops, long tid, ExecType et )
-		throws DMLRuntimeException, HopsException, LopsException
 	{
 		ArrayList<Instruction> newInst = null;
 		
@@ -225,7 +218,6 @@ public class Recompiler
 	}
 	
 	public static ArrayList<Instruction> recompileHopsDag2Forced( Hop hop, long tid, ExecType et ) 
-		throws DMLRuntimeException, HopsException, LopsException
 	{
 		ArrayList<Instruction> newInst = null;
 
@@ -248,7 +240,6 @@ public class Recompiler
 	}
 
 	public static ArrayList<Instruction> recompileHopsDagInstructions( StatementBlock sb, ArrayList<Hop> hops ) 
-		throws DMLRuntimeException, HopsException, LopsException
 	{
 		ArrayList<Instruction> newInst = null;
 
@@ -267,7 +258,6 @@ public class Recompiler
 	}
 
 	public static ArrayList<Instruction> recompileHopsDagInstructions( Hop hop )
-		throws DMLRuntimeException, HopsException, LopsException
 	{
 		ArrayList<Instruction> newInst = null;
 
@@ -301,26 +291,30 @@ public class Recompiler
 	 * @param et given execution type
 	 * @param tid thread id, 0 for main or before worker creation
 	 * @return modified list of instructions
-	 * @throws HopsException if hop compile error
-	 * @throws LopsException if lop compile error
-	 * @throws DMLRuntimeException if runtime error on literal replacement
 	 */
 	private static ArrayList<Instruction> recompile(StatementBlock sb, ArrayList<Hop> hops, LocalVariableMap vars, RecompileStatus status,
 		boolean inplace, boolean replaceLit, boolean updateStats, boolean forceEt, boolean pred, ExecType et, long tid ) 
-			throws HopsException, LopsException, DMLRuntimeException
 	{
+		boolean codegen = ConfigurationManager.isCodegenEnabled()
+			&& !(forceEt && et == null ) //not on reset
+			&& SpoofCompiler.RECOMPILE_CODEGEN;
+		
 		// prepare hops dag for recompile
 		if( !inplace ){ 
 			// deep copy hop dag (for non-reversable rewrites)
 			hops = deepCopyHopsDag(hops);
 		}
-		else {
+		else if( !codegen ) {
 			// clear existing lops
 			Hop.resetVisitStatus(hops);
 			for( Hop hopRoot : hops )
 				rClearLops( hopRoot );
 		}
 		
+		// get max parallelism constraint, see below
+		Hop.resetVisitStatus(hops);
+		int maxK = rGetMaxParallelism(hops);
+		
 		// replace scalar reads with literals 
 		if( !inplace && replaceLit ) {
 			Hop.resetVisitStatus(hops);
@@ -366,9 +360,7 @@ public class Recompiler
 		}
 		
 		// codegen if enabled
-		if( ConfigurationManager.isCodegenEnabled()
-			&& !(forceEt && et == null ) //not on reset
-			&& SpoofCompiler.RECOMPILE_CODEGEN ) {
+		if( codegen ) {
 			//create deep copy for in-place
 			if( inplace )
 				hops = deepCopyHopsDag(hops);
@@ -377,6 +369,11 @@ public class Recompiler
 				(status==null || !status.isInitialCodegen()));
 		}
 		
+		// set max parallelism constraint to ensure compilation 
+		// incl rewrites does not lose these hop-lop constraints
+		Hop.resetVisitStatus(hops);
+		rSetMaxParallelism(hops, maxK);
+		
 		// construct lops
 		Dag<Lop> dag = new Dag<>();
 		for( Hop hopRoot : hops ){
@@ -400,44 +397,29 @@ public class Recompiler
 		return newInst;
 	}
 	
-	private static void logExplainDAG(StatementBlock sb, ArrayList<Hop> hops, ArrayList<Instruction> inst)
-		throws DMLRuntimeException
-	{
+	private static void logExplainDAG(StatementBlock sb, ArrayList<Hop> hops, ArrayList<Instruction> inst) {
 		if( DMLScript.EXPLAIN == ExplainType.RECOMPILE_HOPS ) {
-			LOG.info("EXPLAIN RECOMPILE \nGENERIC (lines "+sb.getBeginLine()+"-"+sb.getEndLine()+"):\n" +
-			Explain.explainHops(hops, 1));
+			System.out.println("EXPLAIN RECOMPILE \nGENERIC (lines "+sb.getBeginLine()+"-"+sb.getEndLine()+"):\n" +
+				Explain.explainHops(hops, 1));
 		}
 		if( DMLScript.EXPLAIN == ExplainType.RECOMPILE_RUNTIME ) {
-			LOG.info("EXPLAIN RECOMPILE \nGENERIC (lines "+sb.getBeginLine()+"-"+sb.getEndLine()+"):\n" +
-			Explain.explain(inst, 1));
+			System.out.println("EXPLAIN RECOMPILE \nGENERIC (lines "+sb.getBeginLine()+"-"+sb.getEndLine()+"):\n" +
+				Explain.explain(inst, 1));
 		}
 	}
 	
-	private static void logExplainPred(Hop hops, ArrayList<Instruction> inst)
-		throws DMLRuntimeException
-	{
+	private static void logExplainPred(Hop hops, ArrayList<Instruction> inst) {
 		if( DMLScript.EXPLAIN == ExplainType.RECOMPILE_HOPS )
-			LOG.info("EXPLAIN RECOMPILE \nPRED (line "+hops.getBeginLine()+"):\n" + Explain.explain(hops,1));
+			System.out.println("EXPLAIN RECOMPILE \nPRED (line "+hops.getBeginLine()+"):\n" + Explain.explain(hops,1));
 		if( DMLScript.EXPLAIN == ExplainType.RECOMPILE_RUNTIME )
-			LOG.info("EXPLAIN RECOMPILE \nPRED (line "+hops.getBeginLine()+"):\n" + Explain.explain(inst,1));
+			System.out.println("EXPLAIN RECOMPILE \nPRED (line "+hops.getBeginLine()+"):\n" + Explain.explain(inst,1));
 	}
 
-	public static void recompileProgramBlockHierarchy( ArrayList<ProgramBlock> pbs, LocalVariableMap vars, long tid, ResetType resetRecompile ) 
-		throws DMLRuntimeException
-	{
-		try 
-		{
-			RecompileStatus status = new RecompileStatus();
-			
-			synchronized( pbs )
-			{
-				for( ProgramBlock pb : pbs )
-					rRecompileProgramBlock(pb, vars, status, tid, resetRecompile);
-			}
-		}
-		catch(Exception ex)
-		{
-			throw new DMLRuntimeException("Unable to recompile program block hierarchy.", ex);
+	public static void recompileProgramBlockHierarchy( ArrayList<ProgramBlock> pbs, LocalVariableMap vars, long tid, ResetType resetRecompile ) {
+		RecompileStatus status = new RecompileStatus();
+		synchronized( pbs ) {
+			for( ProgramBlock pb : pbs )
+				rRecompileProgramBlock(pb, vars, status, tid, resetRecompile);
 		}
 	}
 	
@@ -450,22 +432,11 @@ public class Recompiler
 	 * @param tid thread id
 	 * @param fnStack function stack
 	 * @param et execution type
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void recompileProgramBlockHierarchy2Forced( ArrayList<ProgramBlock> pbs, long tid, HashSet<String> fnStack, ExecType et ) 
-		throws DMLRuntimeException
-	{
-		try 
-		{
-			synchronized( pbs )
-			{
-				for( ProgramBlock pb : pbs )
-					rRecompileProgramBlock2Forced(pb, tid, fnStack, et);
-			}
-		}
-		catch(Exception ex)
-		{
-			throw new DMLRuntimeException("Unable to recompile program block hierarchy to CP.", ex);
+	public static void recompileProgramBlockHierarchy2Forced( ArrayList<ProgramBlock> pbs, long tid, HashSet<String> fnStack, ExecType et ) {
+		synchronized( pbs ) {
+			for( ProgramBlock pb : pbs )
+				rRecompileProgramBlock2Forced(pb, tid, fnStack, et);
 		}
 	}
 	
@@ -476,13 +447,10 @@ public class Recompiler
 	 * and better performance for recompiling individual program blocks.  
 	 * 
 	 * @param pb program block
-	 * @throws HopsException if HopsException occurs
-	 * @throws LopsException if LopsException occurs
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 * @throws IOException if IOException occurs
 	 */
 	public static void recompileProgramBlockInstructions(ProgramBlock pb) 
-		throws HopsException, LopsException, DMLRuntimeException, IOException
+		throws IOException
 	{
 		if( pb instanceof WhileProgramBlock )
 		{
@@ -547,10 +515,8 @@ public class Recompiler
 	 * 
 	 * @param hops list of high-level operators
 	 * @return list of high-level operators
-	 * @throws HopsException if HopsException occurs
 	 */
 	public static ArrayList<Hop> deepCopyHopsDag( List<Hop> hops )
-		throws HopsException 
 	{
 		ArrayList<Hop> ret = new ArrayList<>(hops.size());
 		
@@ -573,11 +539,8 @@ public class Recompiler
 	 * 
 	 * @param hops high-level operator
 	 * @return high-level operator
-	 * @throws HopsException if HopsException occurs
 	 */
-	public static Hop deepCopyHopsDag( Hop hops )
-		throws HopsException 
-	{
+	public static Hop deepCopyHopsDag( Hop hops ) {
 		Hop ret = null;
 		
 		try {
@@ -628,8 +591,7 @@ public class Recompiler
 		//update function names
 		if( hop instanceof FunctionOp && ((FunctionOp)hop).getFunctionType() != FunctionType.MULTIRETURN_BUILTIN) {
 			FunctionOp fop = (FunctionOp) hop;
-			fop.setFunctionName( fop.getFunctionName() +
-					             ProgramConverter.CP_CHILD_THREAD + pid);
+			fop.setFunctionName( fop.getFunctionName() + Lop.CP_CHILD_THREAD + pid);
 		}
 		
 		if( hop.getInput() != null )
@@ -646,7 +608,6 @@ public class Recompiler
 	
 	private static void rRecompileProgramBlock( ProgramBlock pb, LocalVariableMap vars, 
 		RecompileStatus status, long tid, ResetType resetRecompile ) 
-		throws HopsException, DMLRuntimeException, LopsException, IOException
 	{
 		if (pb instanceof WhileProgramBlock)
 		{
@@ -968,7 +929,6 @@ public class Recompiler
 	//helper functions for predicate recompile
 	
 	private static void recompileIfPredicate( IfProgramBlock ipb, IfStatementBlock isb, LocalVariableMap vars, RecompileStatus status, long tid, ResetType resetRecompile ) 
-		throws DMLRuntimeException, HopsException, LopsException, IOException
 	{
 		if( isb == null )
 			return;
@@ -986,9 +946,7 @@ public class Recompiler
 		}
 	}
 	
-	private static void recompileWhilePredicate( WhileProgramBlock wpb, WhileStatementBlock wsb, LocalVariableMap vars, RecompileStatus status, long tid, ResetType resetRecompile ) 
-		throws DMLRuntimeException, HopsException, LopsException, IOException
-	{
+	private static void recompileWhilePredicate( WhileProgramBlock wpb, WhileStatementBlock wsb, LocalVariableMap vars, RecompileStatus status, long tid, ResetType resetRecompile ) {
 		if( wsb == null )
 			return;
 		
@@ -1005,9 +963,7 @@ public class Recompiler
 		}
 	}
 	
-	private static void recompileForPredicates( ForProgramBlock fpb, ForStatementBlock fsb, LocalVariableMap vars, RecompileStatus status, long tid, ResetType resetRecompile ) 
-		throws DMLRuntimeException, HopsException, LopsException, IOException
-	{
+	private static void recompileForPredicates( ForProgramBlock fpb, ForStatementBlock fsb, LocalVariableMap vars, RecompileStatus status, long tid, ResetType resetRecompile ) {
 		if( fsb != null )
 		{
 			Hop fromHops = fsb.getFromHops();
@@ -1059,9 +1015,7 @@ public class Recompiler
 		}
 	}
 	
-	private static void rRecompileProgramBlock2Forced( ProgramBlock pb, long tid, HashSet<String> fnStack, ExecType et ) 
-		throws HopsException, DMLRuntimeException, LopsException, IOException
-	{
+	private static void rRecompileProgramBlock2Forced( ProgramBlock pb, long tid, HashSet<String> fnStack, ExecType et ) {
 		if (pb instanceof WhileProgramBlock)
 		{
 			WhileProgramBlock pbTmp = (WhileProgramBlock)pb;
@@ -1326,7 +1280,6 @@ public class Recompiler
 	}
 	
 	public static void rUpdateStatistics( Hop hop, LocalVariableMap vars ) 
-		throws DMLRuntimeException
 	{
 		if( hop.isVisited() )
 			return;
@@ -1336,17 +1289,12 @@ public class Recompiler
 			for( Hop c : hop.getInput() )
 				rUpdateStatistics(c, vars);	
 		
-		boolean updatedSizeExpr = false;
-		
 		//update statistics for transient reads according to current statistics
 		//(with awareness not to override persistent reads to an existing name)
-		if(     hop instanceof DataOp 
-			&& ((DataOp)hop).getDataOpType() != DataOpTypes.PERSISTENTREAD )
-		{
+		if( HopRewriteUtils.isData(hop, DataOpTypes.TRANSIENTREAD) ) {
 			DataOp d = (DataOp) hop;
 			String varName = d.getName();
-			if( vars.keySet().contains( varName ) )
-			{
+			if( vars.keySet().contains( varName ) ) {
 				Data dat = vars.get(varName);
 				if( dat instanceof MatrixObject ) {
 					MatrixObject mo = (MatrixObject) dat;
@@ -1362,10 +1310,9 @@ public class Recompiler
 			}
 		}
 		//special case for persistent reads with unknown size (read-after-write)
-		else if( hop instanceof DataOp 
-				&& ((DataOp)hop).getDataOpType() == DataOpTypes.PERSISTENTREAD
-				&& !hop.dimsKnown() && ((DataOp)hop).getInputFormatType()!=FileFormatTypes.CSV
-				&& !ConfigurationManager.getCompilerConfigFlag(ConfigType.IGNORE_READ_WRITE_METADATA) )
+		else if( HopRewriteUtils.isData(hop, DataOpTypes.PERSISTENTREAD)
+			&& !hop.dimsKnown() && ((DataOp)hop).getInputFormatType()!=FileFormatTypes.CSV
+			&& !ConfigurationManager.getCompilerConfigFlag(ConfigType.IGNORE_READ_WRITE_METADATA) )
 		{
 			//update hop with read meta data
 			DataOp dop = (DataOp) hop; 
@@ -1386,7 +1333,8 @@ public class Recompiler
 				HashMap<Long, Long> memo = new HashMap<>();
 				d.refreshRowsParameterInformation(d.getInput().get(ix1), vars, memo);
 				d.refreshColsParameterInformation(d.getInput().get(ix2), vars, memo);
-				updatedSizeExpr = initUnknown & d.dimsKnown();
+				if( !(initUnknown & d.dimsKnown()) )
+					d.refreshSizeInformation();
 			} 
 			else if ( d.getOp() == DataGenMethod.SEQ ) 
 			{
@@ -1409,47 +1357,40 @@ public class Recompiler
 					d.setDim2( 1 );
 					d.setIncrementValue( incr );
 				}
-				updatedSizeExpr = initUnknown & d.dimsKnown();
+				if( !(initUnknown & d.dimsKnown()) )
+					d.refreshSizeInformation();
 			}
 			else {
 				throw new DMLRuntimeException("Unexpected data generation method: " + d.getOp());
 			}
 		}
 		//update size expression for reshape according to symbol table entries
-		else if (    hop instanceof ReorgOp 
-				 && ((ReorgOp)(hop)).getOp()==Hop.ReOrgOp.RESHAPE )
-		{
-			ReorgOp d = (ReorgOp) hop;
-			boolean initUnknown = !d.dimsKnown();
-			HashMap<Long, Long> memo = new HashMap<>();
-			d.refreshRowsParameterInformation(d.getInput().get(1), vars, memo);
-			d.refreshColsParameterInformation(d.getInput().get(2), vars, memo);
-			updatedSizeExpr = initUnknown & d.dimsKnown();
+		else if( HopRewriteUtils.isReorg(hop, ReOrgOp.RESHAPE) ) {
+			hop.refreshSizeInformation(); //update incl reset
+			if( !hop.dimsKnown() ) {
+				HashMap<Long, Long> memo = new HashMap<>();
+				hop.refreshRowsParameterInformation(hop.getInput().get(1), vars, memo);
+				hop.refreshColsParameterInformation(hop.getInput().get(2), vars, memo);
+			}
 		}
 		//update size expression for indexing according to symbol table entries
-		else if( hop instanceof IndexingOp )
-		{
-			IndexingOp iop = (IndexingOp)hop;
-			Hop input2 = iop.getInput().get(1); //inpRowL
-			Hop input3 = iop.getInput().get(2); //inpRowU
-			Hop input4 = iop.getInput().get(3); //inpColL
-			Hop input5 = iop.getInput().get(4); //inpColU
-			boolean initUnknown = !iop.dimsKnown();
-			HashMap<Long, Double> memo = new HashMap<>();
-			double rl = iop.computeBoundsInformation(input2, vars, memo);
-			double ru = iop.computeBoundsInformation(input3, vars, memo);
-			double cl = iop.computeBoundsInformation(input4, vars, memo);
-			double cu = iop.computeBoundsInformation(input5, vars, memo);
-			if( rl!=Double.MAX_VALUE && ru!=Double.MAX_VALUE )
-				iop.setDim1( (long)(ru-rl+1) );
-			if( cl!=Double.MAX_VALUE && cu!=Double.MAX_VALUE )
-				iop.setDim2( (long)(cu-cl+1) );
-			updatedSizeExpr = initUnknown & iop.dimsKnown();
+		else if( hop instanceof IndexingOp && hop.getDataType()!=DataType.LIST ) {
+			hop.refreshSizeInformation(); //update, incl reset
+			if( !hop.dimsKnown() ) {
+				HashMap<Long, Double> memo = new HashMap<>();
+				double rl = hop.computeBoundsInformation(hop.getInput().get(1), vars, memo);
+				double ru = hop.computeBoundsInformation(hop.getInput().get(2), vars, memo);
+				double cl = hop.computeBoundsInformation(hop.getInput().get(3), vars, memo);
+				double cu = hop.computeBoundsInformation(hop.getInput().get(4), vars, memo);
+				if( rl!=Double.MAX_VALUE && ru!=Double.MAX_VALUE )
+					hop.setDim1( (long)(ru-rl+1) );
+				if( cl!=Double.MAX_VALUE && cu!=Double.MAX_VALUE )
+					hop.setDim2( (long)(cu-cl+1) );
+			}
 		}
-		
+		else {
 		//propagate statistics along inner nodes of DAG,
 		//without overwriting inferred size expressions
-		if( !updatedSizeExpr ) {
 			hop.refreshSizeInformation();
 		}
 		
@@ -1463,30 +1404,58 @@ public class Recompiler
 	 * @param vars local variable map
 	 * @param scalarsOnly if true, replace only scalar variables but no matrix operations;
 	 *            if false, apply full literal replacement
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void rReplaceLiterals( Hop hop, LocalVariableMap vars, boolean scalarsOnly ) 
-		throws DMLRuntimeException
+	public static void rReplaceLiterals( Hop hop, LocalVariableMap vars, boolean scalarsOnly )
 	{
 		//public interface 
 		LiteralReplacement.rReplaceLiterals(hop, vars, scalarsOnly);
 	}
 	
-	public static void rSetExecType( Hop hop, ExecType etype )
-	{
+	public static void rSetExecType( Hop hop, ExecType etype ) {
 		if( hop.isVisited() )
 			return;
-		
 		//update function names
 		hop.setForcedExecType(etype);
-		
 		if( hop.getInput() != null )
 			for( Hop c : hop.getInput() )
 				rSetExecType(c, etype);
-		
 		hop.setVisited();
 	}
 	
+	public static int rGetMaxParallelism(List<Hop> hops) {
+		int ret = -1;
+		for( Hop c : hops )
+			ret = Math.max(ret, rGetMaxParallelism(c));
+		return ret;
+	}
+	
+	public static int rGetMaxParallelism(Hop hop) {
+		if( hop.isVisited() )
+			return -1;
+		//recursively process children and
+		int ret = rGetMaxParallelism(hop.getInput());
+		//obtain max num thread constraints
+		if( hop instanceof MultiThreadedHop )
+			ret = Math.max(ret, ((MultiThreadedHop)hop).getMaxNumThreads());
+		hop.setVisited();
+		return ret;
+	}
+	
+	public static void rSetMaxParallelism(List<Hop> hops, int k) {
+		for( Hop c : hops )
+			rSetMaxParallelism(c, k);
+	}
+	
+	public static void rSetMaxParallelism(Hop hop, int k) {
+		if( hop.isVisited() )
+			return;
+		//recursively process children
+		rSetMaxParallelism(hop.getInput(), k);
+		//set max num thread constraint
+		if( hop instanceof MultiThreadedHop )
+			((MultiThreadedHop)hop).setMaxNumThreads(k);
+		hop.setVisited();
+	}
 
 	/**
 	 * Returns true iff (1) all instruction are reblock instructions and (2) all
@@ -1496,11 +1465,10 @@ public class Recompiler
 	 * @param inputs the inputs
 	 * @return return true if and only if all instructions are reblock instructions and all
 	 * individual reblock oeprations fir in the current memory budget.
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 * @throws IOException if IOException occurs
 	 */
 	public static boolean checkCPReblock(MRJobInstruction inst, MatrixObject[] inputs) 
-		throws DMLRuntimeException, IOException 
+		throws IOException 
 	{
 		boolean ret = true;
 		
@@ -1622,10 +1590,8 @@ public class Recompiler
 	 * @param ec execution context
 	 * @param varin variable
 	 * @return true if CP reblock?
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
 	public static boolean checkCPReblock(ExecutionContext ec, String varin) 
-		throws DMLRuntimeException
 	{
 		CacheableData<?> obj = ec.getCacheableData(varin);
 		MatrixCharacteristics mc = ec.getMatrixCharacteristics(varin);
@@ -1682,7 +1648,6 @@ public class Recompiler
 	}
 	
 	public static boolean checkCPDataGen( MRJobInstruction inst, String updatedRandInst ) 
-		throws DMLRuntimeException 
 	{
 		boolean ret = true;
 		
@@ -1748,9 +1713,7 @@ public class Recompiler
 		return ret;
 	}
 	
-	public static void executeInMemoryMatrixReblock(ExecutionContext ec, String varin, String varout) 
-		throws DMLRuntimeException
-	{
+	public static void executeInMemoryMatrixReblock(ExecutionContext ec, String varin, String varout) {
 		MatrixObject in = ec.getMatrixObject(varin);
 		MatrixObject out = ec.getMatrixObject(varout);
 
@@ -1761,11 +1724,10 @@ public class Recompiler
 		//set output (incl update matrix characteristics)
 		out.acquireModify( mb );
 		out.release();
-		in.release();				
+		in.release();
 	}
 	
 	public static void executeInMemoryFrameReblock(ExecutionContext ec, String varin, String varout) 
-		throws DMLRuntimeException
 	{
 		FrameObject in = ec.getFrameObject(varin);
 		FrameObject out = ec.getFrameObject(varout);
@@ -1777,11 +1739,10 @@ public class Recompiler
 		//set output (incl update matrix characteristics)
 		out.acquireModify( fb );
 		out.release();
-		in.release();				
+		in.release();
 	}
 	
 	private static void tryReadMetaDataFileMatrixCharacteristics( DataOp dop )
-		throws DMLRuntimeException
 	{
 		try
 		{
@@ -1808,8 +1769,7 @@ public class Recompiler
 				}
 			}
 		}
-		catch(Exception ex)
-		{
+		catch(Exception ex) {
 			throw new DMLRuntimeException(ex);
 		}
 	}
