@@ -20,7 +20,9 @@
 package org.apache.sysml.runtime.controlprogram.context;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
+import java.util.stream.Collectors;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -42,6 +44,7 @@ import org.apache.sysml.runtime.instructions.cp.CPInstruction;
 import org.apache.sysml.runtime.instructions.cp.CPOperand;
 import org.apache.sysml.runtime.instructions.cp.Data;
 import org.apache.sysml.runtime.instructions.cp.FunctionCallCPInstruction;
+import org.apache.sysml.runtime.instructions.cp.ListObject;
 import org.apache.sysml.runtime.instructions.cp.ScalarObject;
 import org.apache.sysml.runtime.instructions.cp.ScalarObjectFactory;
 import org.apache.sysml.runtime.instructions.gpu.context.GPUContext;
@@ -50,10 +53,13 @@ import org.apache.sysml.runtime.matrix.MatrixCharacteristics;
 import org.apache.sysml.runtime.matrix.MetaDataFormat;
 import org.apache.sysml.runtime.matrix.MetaData;
 import org.apache.sysml.runtime.matrix.data.FrameBlock;
+import org.apache.sysml.runtime.matrix.data.InputInfo;
 import org.apache.sysml.runtime.matrix.data.MatrixBlock;
+import org.apache.sysml.runtime.matrix.data.OutputInfo;
 import org.apache.sysml.runtime.matrix.data.Pair;
 import org.apache.sysml.runtime.util.MapReduceTool;
 import org.apache.sysml.utils.GPUStatistics;
+import org.apache.sysml.utils.Statistics;
 
 
 public class ExecutionContext {
@@ -71,7 +77,7 @@ public class ExecutionContext {
 	/**
 	 * List of {@link GPUContext}s owned by this {@link ExecutionContext}
 	 */
-    protected List<GPUContext> _gpuContexts = new ArrayList<>();
+	protected List<GPUContext> _gpuContexts = new ArrayList<>();
 
 	protected ExecutionContext()
 	{
@@ -95,6 +101,10 @@ public class ExecutionContext {
 	public Program getProgram(){
 		return _prog;
 	}
+
+	public void setProgram(Program prog) {
+		_prog = prog;
+	}
 	
 	public LocalVariableMap getVariables() {
 		return _variables;
@@ -150,7 +160,7 @@ public class ExecutionContext {
 		return _variables.get(name);
 	}
 	
-	public Data getVariable(CPOperand operand) throws DMLRuntimeException {
+	public Data getVariable(CPOperand operand) {
 		return operand.getDataType().isScalar() ?
 			getScalarInput(operand) : getVariable(operand.getName());
 	}
@@ -167,15 +177,11 @@ public class ExecutionContext {
 		return _variables.remove(name);
 	}
 
-	public void setMetaData(String fname, MetaData md) 
-		throws DMLRuntimeException 
-	{
+	public void setMetaData(String fname, MetaData md) {
 		_variables.get(fname).setMetaData(md);
 	}
 	
-	public MetaData getMetaData(String varname) 
-		throws DMLRuntimeException 
-	{
+	public MetaData getMetaData(String varname) {
 		return _variables.get(varname).getMetaData();
 	}
 	
@@ -183,10 +189,12 @@ public class ExecutionContext {
 		Data dat = getVariable(varname);
 		return (dat!= null && dat instanceof MatrixObject);
 	}
+	
+	public MatrixObject getMatrixObject(CPOperand input) {
+		return getMatrixObject(input.getName());
+	}
 
-	public MatrixObject getMatrixObject(String varname) 
-		throws DMLRuntimeException
-	{
+	public MatrixObject getMatrixObject(String varname) {
 		Data dat = getVariable(varname);
 		
 		//error handling if non existing or no matrix
@@ -197,45 +205,46 @@ public class ExecutionContext {
 		
 		return (MatrixObject) dat;
 	}
+
+	public boolean isFrameObject(String varname) {
+		Data dat = getVariable(varname);
+		return (dat!= null && dat instanceof FrameObject);
+	}
 	
-	public FrameObject getFrameObject(String varname) 
-		throws DMLRuntimeException
-	{
+	public FrameObject getFrameObject(CPOperand input) {
+		return getFrameObject(input.getName());
+	}
+	
+	public FrameObject getFrameObject(String varname) {
 		Data dat = getVariable(varname);
-		
 		//error handling if non existing or no matrix
 		if( dat == null )
 			throw new DMLRuntimeException("Variable '"+varname+"' does not exist in the symbol table.");
 		if( !(dat instanceof FrameObject) )
 			throw new DMLRuntimeException("Variable '"+varname+"' is not a frame.");
-		
 		return (FrameObject) dat;
 	}
 
-	public CacheableData<?> getCacheableData(String varname) 
-		throws DMLRuntimeException
-	{
+	public CacheableData<?> getCacheableData(CPOperand input) {
+		return getCacheableData(input.getName());
+	}
+	
+	public CacheableData<?> getCacheableData(String varname) {
 		Data dat = getVariable(varname);
-		
 		//error handling if non existing or no matrix
 		if( dat == null )
 			throw new DMLRuntimeException("Variable '"+varname+"' does not exist in the symbol table.");
 		if( !(dat instanceof CacheableData<?>) )
 			throw new DMLRuntimeException("Variable '"+varname+"' is not a matrix or frame.");
-		
 		return (CacheableData<?>) dat;
 	}
 
-	public void releaseCacheableData(String varname) 
-		throws DMLRuntimeException
-	{
+	public void releaseCacheableData(String varname) {
 		CacheableData<?> dat = getCacheableData(varname);
 		dat.release();
 	}
 	
-	public MatrixCharacteristics getMatrixCharacteristics( String varname ) 
-		throws DMLRuntimeException
-	{
+	public MatrixCharacteristics getMatrixCharacteristics( String varname ) {
 		return getMetaData(varname).getMatrixCharacteristics();
 	}
 	
@@ -245,9 +254,8 @@ public class ExecutionContext {
 	 * @param varName variable name
 	 * @param opcode  extended opcode
 	 * @return matrix block
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public MatrixBlock getMatrixInput(String varName, String opcode) throws DMLRuntimeException {
+	public MatrixBlock getMatrixInput(String varName, String opcode) {
 		long t1 = opcode != null && DMLScript.STATISTICS && DMLScript.FINEGRAINED_STATISTICS ? System.nanoTime() : 0;
 		MatrixBlock mb = getMatrixInput(varName);
 		if(opcode != null && DMLScript.STATISTICS && DMLScript.FINEGRAINED_STATISTICS) {
@@ -265,31 +273,24 @@ public class ExecutionContext {
 	 * 
 	 * @param varName variable name
 	 * @return matrix block
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public MatrixBlock getMatrixInput(String varName) 
-		throws DMLRuntimeException 
-	{	
+	public MatrixBlock getMatrixInput(String varName) {
 		MatrixObject mo = getMatrixObject(varName);
 		return mo.acquireRead();
 	}
 	
-	public void setMetaData(String varName, long nrows, long ncols) 
-		throws DMLRuntimeException  
-	{
+	public void setMetaData(String varName, long nrows, long ncols) {
 		MatrixObject mo = getMatrixObject(varName);
 		if(mo.getNumRows() == nrows && mo.getNumColumns() == ncols) 
 			return;
-		
 		MetaData oldMetaData = mo.getMetaData();
 		if( oldMetaData == null || !(oldMetaData instanceof MetaDataFormat) )
 			throw new DMLRuntimeException("Metadata not available");
-			
-		MatrixCharacteristics mc = new MatrixCharacteristics((long)nrows, (long)ncols, 
-				(int) mo.getNumRowsPerBlock(), (int)mo.getNumColumnsPerBlock());
+		MatrixCharacteristics mc = new MatrixCharacteristics(nrows, ncols,
+			(int) mo.getNumRowsPerBlock(), (int)mo.getNumColumnsPerBlock());
 		mo.setMetaData(new MetaDataFormat(mc, 
-				((MetaDataFormat)oldMetaData).getOutputInfo(),
-				((MetaDataFormat)oldMetaData).getInputInfo()));
+			((MetaDataFormat)oldMetaData).getOutputInfo(),
+			((MetaDataFormat)oldMetaData).getInputInfo()));
 	}
 	
 	/**
@@ -299,9 +300,8 @@ public class ExecutionContext {
 	 * @param d1 dimension1
 	 * @param d2 dimension1
 	 * @return valid d1 or d2
-	 * @throws DMLRuntimeException if error occurs
 	 */
-	private static long validateDimensions(long d1, long d2) throws DMLRuntimeException {
+	private static long validateDimensions(long d1, long d2) {
 		if(d1 >= 0 && d2 >= 0 && d1 != d2) {
 			throw new DMLRuntimeException("Incorrect dimensions:" + d1 + " != " + d2);
 		}
@@ -314,11 +314,8 @@ public class ExecutionContext {
 	 * @param numRows number of rows of matrix object
 	 * @param numCols number of columns of matrix object
 	 * @return a pair containing the wrapping {@link MatrixObject} and a boolean indicating whether a cuda memory allocation took place (as opposed to the space already being allocated)
-	 * @throws DMLRuntimeException
 	 */
-	public Pair<MatrixObject, Boolean> getDenseMatrixOutputForGPUInstruction(String varName, long numRows, long numCols)
-		throws DMLRuntimeException 
-	{	
+	public Pair<MatrixObject, Boolean> getDenseMatrixOutputForGPUInstruction(String varName, long numRows, long numCols) {
 		MatrixObject mo = allocateGPUMatrixObject(varName, numRows, numCols);
 		boolean allocated = mo.getGPUObject(getGPUContext(0)).acquireDeviceModifyDense();
 		mo.getMatrixCharacteristics().setNonZeros(-1);
@@ -334,26 +331,22 @@ public class ExecutionContext {
 	 * @param numCols number of columns of matrix object
 	 * @param nnz number of non zeroes
 	 * @return matrix object
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public Pair<MatrixObject, Boolean> getSparseMatrixOutputForGPUInstruction(String varName, long numRows, long numCols, long nnz)
-		throws DMLRuntimeException
-	{
+	public Pair<MatrixObject, Boolean> getSparseMatrixOutputForGPUInstruction(String varName, long numRows, long numCols, long nnz) {
 		MatrixObject mo = allocateGPUMatrixObject(varName, numRows, numCols);
 		mo.getMatrixCharacteristics().setNonZeros(nnz);
 				boolean allocated = mo.getGPUObject(getGPUContext(0)).acquireDeviceModifySparse();
 		return new Pair<>(mo, allocated);
 	}
 
-    /**
+	/**
 	 * Allocates the {@link GPUObject} for a given LOPS Variable (eg. _mVar3)
 	 * @param varName variable name
 	 * @param numRows number of rows of matrix object
 	 * @param numCols number of columns of matrix object
 	 * @return matrix object
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public MatrixObject allocateGPUMatrixObject(String varName, long numRows, long numCols) throws DMLRuntimeException {
+	public MatrixObject allocateGPUMatrixObject(String varName, long numRows, long numCols) {
 		MatrixObject mo = getMatrixObject(varName);
 		long dim1 = -1; long dim2 = -1;
 		DMLRuntimeException e = null;
@@ -385,9 +378,7 @@ public class ExecutionContext {
 		return mo;
 	}
 
-	public MatrixObject getMatrixInputForGPUInstruction(String varName, String opcode)
-			throws DMLRuntimeException 
-	{
+	public MatrixObject getMatrixInputForGPUInstruction(String varName, String opcode) {
 		GPUContext gCtx = getGPUContext(0);
 		MatrixObject mo = getMatrixObject(varName);
 		if(mo == null) {
@@ -407,26 +398,21 @@ public class ExecutionContext {
 	 * Unpins a currently pinned matrix variable and update fine-grained statistics. 
 	 * 
 	 * @param varName variable name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public void releaseMatrixInput(String varName, String opcode) throws DMLRuntimeException {
+	public void releaseMatrixInput(String varName) {
+		getMatrixObject(varName).release();
+	}
+	
+	public void releaseMatrixInput(String varName, String opcode) {
 		long t1 = opcode != null && DMLScript.STATISTICS && DMLScript.FINEGRAINED_STATISTICS ? System.nanoTime() : 0;
-		MatrixObject mo = getMatrixObject(varName);
-		mo.release(opcode);
+		releaseMatrixInput(varName);
 		if(opcode != null && DMLScript.STATISTICS && DMLScript.FINEGRAINED_STATISTICS) {
 			long t2 = System.nanoTime();
 			GPUStatistics.maintainCPMiscTimes(opcode, CPInstruction.MISC_TIMER_RELEASE_INPUT_MB, t2-t1);
 		}
 	}
 	
-	public void releaseMatrixInput(String varName) throws DMLRuntimeException {
-		MatrixObject mo = getMatrixObject(varName);
-		mo.release(null);
-	}
-	
-	public void releaseMatrixInputForGPUInstruction(String varName)
-		throws DMLRuntimeException 
-	{
+	public void releaseMatrixInputForGPUInstruction(String varName) {
 		MatrixObject mo = getMatrixObject(varName);
 		mo.getGPUObject(getGPUContext(0)).releaseInput();
 	}
@@ -436,11 +422,8 @@ public class ExecutionContext {
 	 * 
 	 * @param varName variable name
 	 * @return frame block
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public FrameBlock getFrameInput(String varName) 
-		throws DMLRuntimeException 
-	{	
+	public FrameBlock getFrameInput(String varName) {
 		FrameObject fo = getFrameObject(varName);
 		return fo.acquireRead();
 	}
@@ -449,22 +432,17 @@ public class ExecutionContext {
 	 * Unpins a currently pinned frame variable. 
 	 * 
 	 * @param varName variable name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public void releaseFrameInput(String varName) 
-		throws DMLRuntimeException 
-	{
+	public void releaseFrameInput(String varName) {
 		FrameObject fo = getFrameObject(varName);
 		fo.release();
 	}
 	
-	public ScalarObject getScalarInput(CPOperand input) throws DMLRuntimeException {
+	public ScalarObject getScalarInput(CPOperand input) {
 		return getScalarInput(input.getName(), input.getValueType(), input.isLiteral());
 	}
 	
-	public ScalarObject getScalarInput(String name, ValueType vt, boolean isLiteral)
-		throws DMLRuntimeException 
-	{
+	public ScalarObject getScalarInput(String name, ValueType vt, boolean isLiteral) {
 		if ( isLiteral ) {
 			return ScalarObjectFactory.createScalarObject(vt, name);
 		}
@@ -476,53 +454,75 @@ public class ExecutionContext {
 		}
 	}
 
-	public void setScalarOutput(String varName, ScalarObject so) 
-		throws DMLRuntimeException 
-	{
+	public void setScalarOutput(String varName, ScalarObject so) {
 		setVariable(varName, so);
 	}
-	
-	public void releaseMatrixOutputForGPUInstruction(String varName) throws DMLRuntimeException {
+
+	public ListObject getListObject(String name) {
+		Data dat = getVariable(name);
+		//error handling if non existing or no list
+		if (dat == null)
+			throw new DMLRuntimeException("Variable '" + name + "' does not exist in the symbol table.");
+		if (!(dat instanceof ListObject))
+			throw new DMLRuntimeException("Variable '" + name + "' is not a list.");
+		return (ListObject) dat;
+	}
+
+	public void releaseMatrixOutputForGPUInstruction(String varName) {
 		MatrixObject mo = getMatrixObject(varName);
 		if(mo.getGPUObject(getGPUContext(0)) == null || !mo.getGPUObject(getGPUContext(0)).isAllocated()) {
 			throw new DMLRuntimeException("No output is allocated on GPU");
 		}
+		setMetaData(varName, new MetaDataFormat(mo.getMatrixCharacteristics(), OutputInfo.BinaryBlockOutputInfo, InputInfo.BinaryBlockInputInfo));
 		mo.getGPUObject(getGPUContext(0)).releaseOutput();
 	}
 	
-	public void setMatrixOutput(String varName, MatrixBlock outputData) throws DMLRuntimeException  {
-		setMatrixOutput(varName, outputData, null);
-	}
-
-	public void setMatrixOutput(String varName, MatrixBlock outputData, String opcode) throws DMLRuntimeException {
+	public void setMatrixOutput(String varName, MatrixBlock outputData) {
 		MatrixObject mo = getMatrixObject(varName);
-		mo.acquireModify(outputData, opcode);
-		mo.release(opcode);
+		mo.acquireModify(outputData);
+		mo.release();
 		setVariable(varName, mo);
 	}
+	
+	public void setMatrixOutput(String varName, MatrixBlock outputData, String opcode) {
+		setMatrixOutput(varName, outputData);
+	}
 
-	public void setMatrixOutput(String varName, MatrixBlock outputData, UpdateType flag, String opcode) 
-		throws DMLRuntimeException 
-	{
+	public void setMatrixOutput(String varName, MatrixBlock outputData, UpdateType flag) {
 		if( flag.isInPlace() ) {
 			//modify metadata to carry update status
 			MatrixObject mo = getMatrixObject(varName);
 			mo.setUpdateType( flag );
 		}
-		
-		//default case
-		setMatrixOutput(varName, outputData, opcode);
+		setMatrixOutput(varName, outputData);
+	}
+	
+	public void setMatrixOutput(String varName, MatrixBlock outputData, UpdateType flag, String opcode) {
+		setMatrixOutput(varName, outputData, flag);
 	}
 
-	public void setFrameOutput(String varName, FrameBlock outputData) 
-		throws DMLRuntimeException 
-	{
+	public void setFrameOutput(String varName, FrameBlock outputData) {
 		FrameObject fo = getFrameObject(varName);
 		fo.acquireModify(outputData);
 		fo.release();
 		setVariable(varName, fo);
 	}
 	
+	public List<MatrixBlock> getMatrixInputs(CPOperand[] inputs) {
+		return Arrays.stream(inputs).filter(in -> in.isMatrix())
+			.map(in -> getMatrixInput(in.getName())).collect(Collectors.toList());
+	}
+	
+	public List<ScalarObject> getScalarInputs(CPOperand[] inputs) {
+		return Arrays.stream(inputs).filter(in -> in.isScalar())
+			.map(in -> getScalarInput(in)).collect(Collectors.toList());
+	}
+	
+	public void releaseMatrixInputs(CPOperand[] inputs) {
+		Arrays.stream(inputs).filter(in -> in.isMatrix())
+			.forEach(in -> releaseMatrixInput(in.getName()));
+	}
+	
 	/**
 	 * Pin a given list of variables i.e., set the "clean up" state in 
 	 * corresponding matrix objects, so that the cached data inside these
@@ -537,23 +537,42 @@ public class ExecutionContext {
 	 * @param varList variable list
 	 * @return indicator vector of old cleanup state of matrix objects
 	 */
-	public boolean[] pinVariables(ArrayList<String> varList) 
+	public boolean[] pinVariables(List<String> varList) 
 	{
-		//2-pass approach since multiple vars might refer to same matrix object
-		boolean[] varsState = new boolean[varList.size()];
-		
-		//step 1) get current information
+		//analyze list variables
+		int nlist = 0;
+		int nlistItems = 0;
 		for( int i=0; i<varList.size(); i++ ) {
 			Data dat = _variables.get(varList.get(i));
-			if( dat instanceof MatrixObject )
-				varsState[i] = ((MatrixObject)dat).isCleanupEnabled();
+			if( dat instanceof ListObject ) {
+				nlistItems += ((ListObject)dat).getNumCacheableData();
+				nlist++;
+			}
+		}
+		
+		//2-pass approach since multiple vars might refer to same matrix object
+		boolean[] varsState = new boolean[varList.size()-nlist+nlistItems];
+		
+		//step 1) get current information
+		for( int i=0, pos=0; i<varList.size(); i++ ) {
+			Data dat = _variables.get(varList.get(i));
+			if( dat instanceof CacheableData<?>  )
+				varsState[pos++] = ((CacheableData<?>)dat).isCleanupEnabled();
+			else if( dat instanceof ListObject )
+				for( Data dat2 : ((ListObject)dat).getData() )
+					if( dat2 instanceof CacheableData<?> )
+						varsState[pos++] = ((CacheableData<?>)dat2).isCleanupEnabled();
 		}
 		
 		//step 2) pin variables
 		for( int i=0; i<varList.size(); i++ ) {
 			Data dat = _variables.get(varList.get(i));
-			if( dat instanceof MatrixObject )
-				((MatrixObject)dat).enableCleanup(false); 
+			if( dat instanceof CacheableData<?> )
+				((CacheableData<?>)dat).enableCleanup(false);
+			else if( dat instanceof ListObject )
+				for( Data dat2 : ((ListObject)dat).getData() )
+					if( dat2 instanceof CacheableData<?> )
+						((CacheableData<?>)dat2).enableCleanup(false);
 		}
 		
 		return varsState;
@@ -575,11 +594,15 @@ public class ExecutionContext {
 	 * @param varList variable list
 	 * @param varsState variable state
 	 */
-	public void unpinVariables(ArrayList<String> varList, boolean[] varsState) {
-		for( int i=0; i<varList.size(); i++ ) {
+	public void unpinVariables(List<String> varList, boolean[] varsState) {
+		for( int i=0, pos=0; i<varList.size(); i++ ) {
 			Data dat = _variables.get(varList.get(i));
-			if( dat instanceof MatrixObject )
-				((MatrixObject)dat).enableCleanup(varsState[i]);
+			if( dat instanceof CacheableData<?> )
+				((CacheableData<?>)dat).enableCleanup(varsState[pos++]);
+			else if( dat instanceof ListObject )
+				for( Data dat2 : ((ListObject)dat).getData() )
+					if( dat2 instanceof CacheableData<?> )
+						((CacheableData<?>)dat2).enableCleanup(varsState[pos++]);
 		}
 	}
 	
@@ -608,9 +631,19 @@ public class ExecutionContext {
 		return ret;
 	}
 	
-	public void cleanupCacheableData(CacheableData<?> mo)
-		throws DMLRuntimeException 
-	{
+	public final void cleanupDataObject(Data dat) {
+		if( dat == null ) return;
+		if ( dat instanceof CacheableData )
+			cleanupCacheableData( (CacheableData<?>)dat );
+		else if( dat instanceof ListObject )
+			for( Data dat2 : ((ListObject)dat).getData() )
+				if( dat2 instanceof CacheableData<?> )
+					cleanupCacheableData( (CacheableData<?>)dat2 );
+	}
+	
+	public void cleanupCacheableData(CacheableData<?> mo) {
+		if (DMLScript.JMLC_MEM_STATISTICS)
+			Statistics.removeCPMemObject(System.identityHashCode(mo));
 		//early abort w/o scan of symbol table if no cleanup required
 		boolean fileExists = (mo.isHDFSFileExists() && mo.getFileName() != null);
 		if( !CacheableData.isCachingActive() && !fileExists )
@@ -644,51 +677,45 @@ public class ExecutionContext {
 		}
 	}
 
-	public void updateDebugState( int index ) throws DMLRuntimeException 
-	{
+	public void updateDebugState( int index ) {
 		if(DMLScript.ENABLE_DEBUG_MODE) {
 			_dbState.getPC().setProgramBlockNumber(index);
 		}
 	}
 
-	public void updateDebugState( Instruction currInst ) throws DMLRuntimeException
-	{
+	public void updateDebugState( Instruction currInst ) {
 		if (DMLScript.ENABLE_DEBUG_MODE) {
 			// New change so that shell doesnot seem like it is hanging while running MR job
 			// Since UI cannot accept instructions when user is submitting the program
 			_dbState.nextCommand = false;
 			// Change to stop before first instruction of a given line
 			//update current instruction ID and line number 
-			_dbState.getPC().setInstID(currInst.getInstID()); 				
+			_dbState.getPC().setInstID(currInst.getInstID());
 			_dbState.getPC().setLineNumber(currInst.getLineNum());
 			// Change to stop before first instruction of a given line
-			suspendIfAskedInDebugMode(currInst);	
+			suspendIfAskedInDebugMode(currInst);
 		}
 	}
 
-	public void clearDebugProgramCounters()
-	{
+	public void clearDebugProgramCounters() {
 		if(DMLScript.ENABLE_DEBUG_MODE) {
 			_dbState.pc = null;
 		}
 	}
 	
-	public void handleDebugException( Exception ex )
-	{
+	public void handleDebugException( Exception ex ) {
 		_dbState.getDMLStackTrace(ex);
 		_dbState.suspend = true;
 	}
 
-	public void handleDebugFunctionEntry( FunctionCallCPInstruction funCallInst ) throws DMLRuntimeException
-	{
+	public void handleDebugFunctionEntry( FunctionCallCPInstruction funCallInst ) {
 		//push caller frame into call stack
 		_dbState.pushFrame(getVariables(), _dbState.getPC());
 		//initialize pc for callee's frame
 		_dbState.pc = new DMLProgramCounter(funCallInst.getNamespace(), funCallInst.getFunctionName(), 0, 0);
 	}
 	
-	public void handleDebugFunctionExit( FunctionCallCPInstruction funCallInst )
-	{
+	public void handleDebugFunctionExit( FunctionCallCPInstruction funCallInst ) {
 		//pop caller frame from call stack
 		DMLFrame fr = _dbState.popFrame();
 		//update pc to caller's frame
@@ -706,10 +733,9 @@ public class ExecutionContext {
 	 * then it will wait until user issues a new debugger command.
 	 * 
 	 * @param currInst current instruction
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
 	@SuppressWarnings("deprecation")
-	private void suspendIfAskedInDebugMode(Instruction currInst ) throws DMLRuntimeException {
+	private void suspendIfAskedInDebugMode(Instruction currInst ) {
 		if (!DMLScript.ENABLE_DEBUG_MODE) {
 			System.err.println("ERROR: The function suspendIfAskedInDebugMode should not be called in non-debug mode.");
 		}
