@@ -46,12 +46,11 @@ import org.apache.sysml.runtime.DMLScriptException;
  */
 public class Builtin extends ValueFunction 
 {
-
 	private static final long serialVersionUID = 3836744687789840574L;
 	
 	public enum BuiltinCode { SIN, COS, TAN, SINH, COSH, TANH, ASIN, ACOS, ATAN, LOG, LOG_NZ, MIN,
 		MAX, ABS, SIGN, SQRT, EXP, PLOGP, PRINT, PRINTF, NROW, NCOL, LENGTH, ROUND, MAXINDEX, MININDEX,
-		STOP, CEIL, FLOOR, CUMSUM, CUMPROD, CUMMIN, CUMMAX, INVERSE, SPROP, SIGMOID, EVAL }
+		STOP, CEIL, FLOOR, CUMSUM, CUMPROD, CUMMIN, CUMMAX, CUMSUMPROD, INVERSE, SPROP, SIGMOID, EVAL, LIST }
 	public BuiltinCode bFunc;
 	
 	private static final boolean FASTMATH = true;
@@ -82,6 +81,7 @@ public class Builtin extends ValueFunction
 		String2BuiltinCode.put( "print"  , BuiltinCode.PRINT);
 		String2BuiltinCode.put( "printf"  , BuiltinCode.PRINTF);
 		String2BuiltinCode.put( "eval"  , BuiltinCode.EVAL);
+		String2BuiltinCode.put( "list"  , BuiltinCode.LIST);
 		String2BuiltinCode.put( "nrow"   , BuiltinCode.NROW);
 		String2BuiltinCode.put( "ncol"   , BuiltinCode.NCOL);
 		String2BuiltinCode.put( "length" , BuiltinCode.LENGTH);
@@ -91,6 +91,7 @@ public class Builtin extends ValueFunction
 		String2BuiltinCode.put( "floor"  , BuiltinCode.FLOOR);
 		String2BuiltinCode.put( "ucumk+" , BuiltinCode.CUMSUM);
 		String2BuiltinCode.put( "ucum*"  , BuiltinCode.CUMPROD);
+		String2BuiltinCode.put( "ucumk+*", BuiltinCode.CUMSUMPROD);
 		String2BuiltinCode.put( "ucummin", BuiltinCode.CUMMIN);
 		String2BuiltinCode.put( "ucummax", BuiltinCode.CUMMAX);
 		String2BuiltinCode.put( "inverse", BuiltinCode.INVERSE);
@@ -103,7 +104,7 @@ public class Builtin extends ValueFunction
 	private static Builtin logObj = null, lognzObj = null, minObj = null, maxObj = null, maxindexObj = null, minindexObj=null;
 	private static Builtin absObj = null, signObj = null, sqrtObj = null, expObj = null, plogpObj = null, printObj = null, printfObj;
 	private static Builtin nrowObj = null, ncolObj = null, lengthObj = null, roundObj = null, ceilObj=null, floorObj=null; 
-	private static Builtin inverseObj=null, cumsumObj=null, cumprodObj=null, cumminObj=null, cummaxObj=null;
+	private static Builtin inverseObj=null, cumsumObj=null, cumprodObj=null, cumminObj=null, cummaxObj=null, cumsprodObj=null;
 	private static Builtin stopObj = null, spropObj = null, sigmoidObj = null;
 	
 	private Builtin(BuiltinCode bf) {
@@ -256,7 +257,11 @@ public class Builtin extends ValueFunction
 		case CUMPROD:
 			if ( cumprodObj == null )
 				cumprodObj = new Builtin(BuiltinCode.CUMPROD);
-			return cumprodObj;	
+			return cumprodObj;
+		case CUMSUMPROD:
+			if ( cumsprodObj == null )
+				cumsprodObj = new Builtin(BuiltinCode.CUMSUMPROD);
+			return cumsprodObj;	
 		case CUMMIN:
 			if ( cumminObj == null )
 				cumminObj = new Builtin(BuiltinCode.CUMMIN);
@@ -291,9 +296,7 @@ public class Builtin extends ValueFunction
 	}
 
 	@Override
-	public double execute (double in) 
-		throws DMLRuntimeException 
-	{
+	public double execute (double in) {
 		switch(bFunc) {
 			case SIN:    return FASTMATH ? FastMath.sin(in) : Math.sin(in);
 			case COS:    return FASTMATH ? FastMath.cos(in) : Math.cos(in);
@@ -337,130 +340,94 @@ public class Builtin extends ValueFunction
 	}
 
 	@Override
-	public double execute (long in) throws DMLRuntimeException {
+	public double execute (long in) {
 		return execute((double)in);
 	}
 
 	/*
 	 * Builtin functions with two inputs
-	 */	
-	@Override
-	public double execute (double in1, double in2) throws DMLRuntimeException {
-		switch(bFunc) {
-		
-		/*
-		 * Arithmetic relational operators (==, !=, <=, >=) must be instead of
-		 * <code>Double.compare()</code> due to the inconsistencies in the way
-		 * NaN and -0.0 are handled. The behavior of methods in
-		 * <code>Double</code> class are designed mainly to make Java
-		 * collections work properly. For more details, see the help for
-		 * <code>Double.equals()</code> and <code>Double.comapreTo()</code>.
-		 */
-		case MAX:
-		case CUMMAX:
-			//return (Double.compare(in1, in2) >= 0 ? in1 : in2);
-			return (in1 >= in2 ? in1 : in2);
-		case MIN:
-		case CUMMIN:
-			//return (Double.compare(in1, in2) <= 0 ? in1 : in2);
-			return (in1 <= in2 ? in1 : in2);
-			
-			// *** HACK ALERT *** HACK ALERT *** HACK ALERT ***
-			// rowIndexMax() and its siblings require comparing four values, but
-			// the aggregation API only allows two values. So the execute()
-			// method receives as its argument the two cell values to be
-			// compared and performs just the value part of the comparison. We
-			// return an integer cast down to a double, since the aggregation
-			// API doesn't have any way to return anything but a double. The
-			// integer returned takes on three posssible values: //
-			// .     0 => keep the index associated with in1 //
-			// .     1 => use the index associated with in2 //
-			// .     2 => use whichever index is higher (tie in value) //
-		case MAXINDEX:
-			if (in1 == in2) {
-				return 2;
-			} else if (in1 > in2) {
-				return 1;
-			} else { // in1 < in2
-				return 0;
-			}
-		case MININDEX:
-			if (in1 == in2) {
-				return 2;
-			} else if (in1 < in2) {
-				return 1;
-			} else { // in1 > in2
-				return 0;
-			}
-			// *** END HACK ***
-		case LOG:
-			//faster in Math
-			return (Math.log(in1)/Math.log(in2)); 
-		case LOG_NZ:
-			//faster in Math
-			return (in1==0) ? 0 : (Math.log(in1)/Math.log(in2));
-		default:
-			throw new DMLRuntimeException("Builtin.execute(): Unknown operation: " + bFunc);
-		}
-	}
-	
-	/**
-	 * Simplified version without exception handling
-	 * 
-	 * @param in1 double 1
-	 * @param in2 double 2
-	 * @return result
 	 */
-	public double execute2(double in1, double in2) 
-	{
+	@Override
+	public double execute (double in1, double in2) {
 		switch(bFunc) {
+			/*
+			 * Arithmetic relational operators (==, !=, <=, >=) must be instead of
+			 * <code>Double.compare()</code> due to the inconsistencies in the way
+			 * NaN and -0.0 are handled. The behavior of methods in
+			 * <code>Double</code> class are designed mainly to make Java
+			 * collections work properly. For more details, see the help for
+			 * <code>Double.equals()</code> and <code>Double.comapreTo()</code>.
+			 */
 			case MAX:
 			case CUMMAX:
-				//return (Double.compare(in1, in2) >= 0 ? in1 : in2); 
-				return (in1 >= in2 ? in1 : in2);
+				return Math.max(in1, in2);
 			case MIN:
 			case CUMMIN:
-				//return (Double.compare(in1, in2) <= 0 ? in1 : in2); 
-				return (in1 <= in2 ? in1 : in2);
-			case MAXINDEX: 
-				return (in1 >= in2) ? 1 : 0;
-			case MININDEX: 
-				return (in1 <= in2) ? 1 : 0;
+				return Math.min(in1, in2);
+				
+				// *** HACK ALERT *** HACK ALERT *** HACK ALERT ***
+				// rowIndexMax() and its siblings require comparing four values, but
+				// the aggregation API only allows two values. So the execute()
+				// method receives as its argument the two cell values to be
+				// compared and performs just the value part of the comparison. We
+				// return an integer cast down to a double, since the aggregation
+				// API doesn't have any way to return anything but a double. The
+				// integer returned takes on three posssible values: //
+				// .     0 => keep the index associated with in1 //
+				// .     1 => use the index associated with in2 //
+				// .     2 => use whichever index is higher (tie in value) //
+			case MAXINDEX:
+				if (in1 == in2) {
+					return 2;
+				} else if (in1 > in2) {
+					return 1;
+				} else { // in1 < in2
+					return 0;
+				}
+			case MININDEX:
+				if (in1 == in2) {
+					return 2;
+				} else if (in1 < in2) {
+					return 1;
+				} else { // in1 > in2
+					return 0;
+				}
+				// *** END HACK ***
+			case LOG://faster in Math
+				return (Math.log(in1)/Math.log(in2)); 
+			case LOG_NZ: //faster in Math
+				return (in1==0) ? 0 : (Math.log(in1)/Math.log(in2));
 			default:
-				// For performance reasons, avoid throwing an exception 
-				return -1;
+				throw new DMLRuntimeException("Builtin.execute(): Unknown operation: " + bFunc);
 		}
 	}
 	
 	@Override
-	public double execute (long in1, long in2) throws DMLRuntimeException {
+	public double execute (long in1, long in2) {
 		switch(bFunc) {
-		
-		case MAX:    
-		case CUMMAX:   return (in1 >= in2 ? in1 : in2); 
-		
-		case MIN:    
-		case CUMMIN:   return (in1 <= in2 ? in1 : in2); 
-		
-		case MAXINDEX: return (in1 >= in2) ? 1 : 0;
-		case MININDEX: return (in1 <= in2) ? 1 : 0;
-		
-		case LOG:
-			//faster in Math
-			return Math.log(in1)/Math.log(in2);
-		case LOG_NZ:
-			//faster in Math
-			return (in1==0) ? 0 : Math.log(in1)/Math.log(in2);
-
-		default:
-			throw new DMLRuntimeException("Builtin.execute(): Unknown operation: " + bFunc);
+			case MAX:
+			case CUMMAX:   return Math.max(in1, in2);
+			
+			case MIN:
+			case CUMMIN:   return Math.min(in1, in2);
+			
+			case MAXINDEX: return (in1 >= in2) ? 1 : 0;
+			case MININDEX: return (in1 <= in2) ? 1 : 0;
+			
+			case LOG:
+				//faster in Math
+				return Math.log(in1)/Math.log(in2);
+			case LOG_NZ:
+				//faster in Math
+				return (in1==0) ? 0 : Math.log(in1)/Math.log(in2);
+	
+			default:
+				throw new DMLRuntimeException("Builtin.execute(): Unknown operation: " + bFunc);
 		}
 	}
 
 	@Override
-	public String execute (String in1) 
-		throws DMLRuntimeException 
-	{
+	public String execute (String in1) {
 		switch (bFunc) {
 		case PRINT:
 			if (!DMLScript.suppressPrint2Stdout())
