@@ -39,7 +39,7 @@ import org.apache.sysml.runtime.instructions.cp.CPInstruction;
 import org.apache.sysml.runtime.instructions.cp.CPInstruction.CPType;
 import org.apache.sysml.runtime.instructions.cp.CentralMomentCPInstruction;
 import org.apache.sysml.runtime.instructions.cp.CompressionCPInstruction;
-import org.apache.sysml.runtime.instructions.cp.ConvolutionCPInstruction;
+import org.apache.sysml.runtime.instructions.cp.DnnCPInstruction;
 import org.apache.sysml.runtime.instructions.cp.CovarianceCPInstruction;
 import org.apache.sysml.runtime.instructions.cp.DataGenCPInstruction;
 import org.apache.sysml.runtime.instructions.cp.DataPartitionCPInstruction;
@@ -64,14 +64,11 @@ import org.apache.sysml.runtime.instructions.cp.UaggOuterChainCPInstruction;
 import org.apache.sysml.runtime.instructions.cp.UnaryCPInstruction;
 import org.apache.sysml.runtime.instructions.cp.VariableCPInstruction;
 import org.apache.sysml.runtime.instructions.cpfile.MatrixIndexingCPFileInstruction;
-import org.apache.sysml.runtime.instructions.cpfile.ParameterizedBuiltinCPFileInstruction;
+import org.apache.sysml.runtime.util.UtilFunctions;
 
 public class CPInstructionParser extends InstructionParser 
 {
-	
 	public static final HashMap<String, CPType> String2CPInstructionType;
-	public static final HashMap<String, CPType> String2CPFileInstructionType;
-	
 	static {
 		String2CPInstructionType = new HashMap<>();
 		String2CPInstructionType.put( "ba+*"   	, CPType.AggregateBinary);
@@ -102,11 +99,14 @@ public class CPInstructionParser extends InstructionParser
 		String2CPInstructionType.put( "uar+"    , CPType.AggregateUnary);
 		String2CPInstructionType.put( "uac+"    , CPType.AggregateUnary);
 		String2CPInstructionType.put( "ua*"     , CPType.AggregateUnary);
+		String2CPInstructionType.put( "uar*"    , CPType.AggregateUnary);
+		String2CPInstructionType.put( "uac*"    , CPType.AggregateUnary);
 		String2CPInstructionType.put( "uatrace" , CPType.AggregateUnary);
 		String2CPInstructionType.put( "uaktrace", CPType.AggregateUnary);
 		String2CPInstructionType.put( "nrow"    ,CPType.AggregateUnary);
 		String2CPInstructionType.put( "ncol"    ,CPType.AggregateUnary);
 		String2CPInstructionType.put( "length"  ,CPType.AggregateUnary);
+		String2CPInstructionType.put( "exists"  ,CPType.AggregateUnary);
 
 		String2CPInstructionType.put( "uaggouterchain", CPType.UaggOuterChain);
 		
@@ -146,9 +146,11 @@ public class CPInstructionParser extends InstructionParser
 		String2CPInstructionType.put( "log"  , CPType.Builtin);
 		String2CPInstructionType.put( "log_nz"  , CPType.Builtin);
 
+		String2CPInstructionType.put( "solve"  , CPType.Binary);
 		String2CPInstructionType.put( "max"  , CPType.Binary);
 		String2CPInstructionType.put( "min"  , CPType.Binary);
-		String2CPInstructionType.put( "solve"  , CPType.Binary);
+		String2CPInstructionType.put( "nmax"  , CPType.BuiltinNary);
+		String2CPInstructionType.put( "nmin"  , CPType.BuiltinNary);
 		
 		String2CPInstructionType.put( "exp"   , CPType.Unary);
 		String2CPInstructionType.put( "abs"   , CPType.Unary);
@@ -171,6 +173,7 @@ public class CPInstructionParser extends InstructionParser
 		String2CPInstructionType.put( "floor" , CPType.Unary);
 		String2CPInstructionType.put( "ucumk+", CPType.Unary);
 		String2CPInstructionType.put( "ucum*" , CPType.Unary);
+		String2CPInstructionType.put( "ucumk+*" , CPType.Unary);
 		String2CPInstructionType.put( "ucummin", CPType.Unary);
 		String2CPInstructionType.put( "ucummax", CPType.Unary);
 		String2CPInstructionType.put( "stop"  , CPType.Unary);
@@ -179,23 +182,28 @@ public class CPInstructionParser extends InstructionParser
 		String2CPInstructionType.put( "sprop", CPType.Unary);
 		String2CPInstructionType.put( "sigmoid", CPType.Unary);
 		
-		String2CPInstructionType.put( "printf" , CPType.BuiltinNary);
-		String2CPInstructionType.put( "cbind" , CPType.BuiltinNary);
-		String2CPInstructionType.put( "rbind" , CPType.BuiltinNary);
-		String2CPInstructionType.put( "eval" , CPType.BuiltinNary);
-
+		String2CPInstructionType.put( "printf", CPType.BuiltinNary);
+		String2CPInstructionType.put( "cbind",  CPType.BuiltinNary);
+		String2CPInstructionType.put( "rbind",  CPType.BuiltinNary);
+		String2CPInstructionType.put( "eval",   CPType.BuiltinNary);
+		String2CPInstructionType.put( "list",   CPType.BuiltinNary);
+		
 		// Parameterized Builtin Functions
-		String2CPInstructionType.put( "cdf"	 		, CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "invcdf"	 	, CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "groupedagg"	, CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "rmempty"	    , CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "replace"	    , CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "rexpand"	    , CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "toString"    , CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "transformapply",CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put("paramserv", 		CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "nvlist",  CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "cdf",            CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "invcdf",         CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "groupedagg",     CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "rmempty" ,       CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "replace",        CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "lowertri",       CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "uppertri",       CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "rexpand",        CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "toString",       CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "transformapply", CPType.ParameterizedBuiltin);
 		String2CPInstructionType.put( "transformdecode",CPType.ParameterizedBuiltin);
 		String2CPInstructionType.put( "transformcolmap",CPType.ParameterizedBuiltin);
-		String2CPInstructionType.put( "transformmeta",CPType.ParameterizedBuiltin);
+		String2CPInstructionType.put( "transformmeta",  CPType.ParameterizedBuiltin);
 		String2CPInstructionType.put( "transformencode",CPType.MultiReturnParameterizedBuiltin);
 		
 		// Ternary Instruction Opcodes
@@ -228,20 +236,21 @@ public class CPInstructionParser extends InstructionParser
 		String2CPInstructionType.put( "rsort"      , CPType.Reorg);
 
 		// Opcodes related to convolutions
-		String2CPInstructionType.put( "relu_backward"      , CPType.Convolution);
-		String2CPInstructionType.put( "relu_maxpooling"      , CPType.Convolution);
-		String2CPInstructionType.put( "relu_maxpooling_backward"      , CPType.Convolution);
-		String2CPInstructionType.put( "maxpooling"      , CPType.Convolution);
-		String2CPInstructionType.put( "maxpooling_backward"      , CPType.Convolution);
-		String2CPInstructionType.put( "avgpooling"      , CPType.Convolution);
-		String2CPInstructionType.put( "avgpooling_backward"      , CPType.Convolution);
-		String2CPInstructionType.put( "conv2d"      , CPType.Convolution);
-		String2CPInstructionType.put( "conv2d_bias_add"      , CPType.Convolution);
-		String2CPInstructionType.put( "conv2d_backward_filter"      , CPType.Convolution);
-		String2CPInstructionType.put( "conv2d_backward_data"      , CPType.Convolution);
-		String2CPInstructionType.put( "bias_add"      , CPType.Convolution);
-		String2CPInstructionType.put( "bias_multiply"      , CPType.Convolution);
-		String2CPInstructionType.put( "channel_sums"      , CPType.Convolution);
+		String2CPInstructionType.put( "relu_backward"      , CPType.Dnn);
+		String2CPInstructionType.put( "relu_maxpooling"      , CPType.Dnn);
+		String2CPInstructionType.put( "relu_maxpooling_backward"      , CPType.Dnn);
+		String2CPInstructionType.put( "maxpooling"      , CPType.Dnn);
+		String2CPInstructionType.put( "maxpooling_backward"      , CPType.Dnn);
+		String2CPInstructionType.put( "avgpooling"      , CPType.Dnn);
+		String2CPInstructionType.put( "avgpooling_backward"      , CPType.Dnn);
+		String2CPInstructionType.put( "conv2d"      , CPType.Dnn);
+		String2CPInstructionType.put( "conv2d_bias_add"      , CPType.Dnn);
+		String2CPInstructionType.put( "conv2d_backward_filter"      , CPType.Dnn);
+		String2CPInstructionType.put( "conv2d_backward_data"      , CPType.Dnn);
+		String2CPInstructionType.put( "bias_add"      , CPType.Dnn);
+		String2CPInstructionType.put( "bias_multiply"      , CPType.Dnn);
+		String2CPInstructionType.put( "batch_norm2d",           CPType.Dnn);
+		String2CPInstructionType.put( "batch_norm2d_backward",  CPType.Dnn);
 		
 		// Quaternary instruction opcodes
 		String2CPInstructionType.put( "wsloss"  , CPType.Quaternary);
@@ -286,19 +295,11 @@ public class CPInstructionParser extends InstructionParser
 		String2CPInstructionType.put( "partition", 	CPType.Partition);
 		String2CPInstructionType.put( "compress", 	CPType.Compression);
 		String2CPInstructionType.put( "spoof", 		CPType.SpoofFused);
-
-		
-		//CP FILE instruction
-		String2CPFileInstructionType = new HashMap<>();
-		String2CPFileInstructionType.put( "rmempty"	    , CPType.ParameterizedBuiltin);
 	}
 
-	public static CPInstruction parseSingleInstruction (String str ) 
-		throws DMLRuntimeException 
-	{
+	public static CPInstruction parseSingleInstruction (String str ) {
 		if ( str == null || str.isEmpty() )
 			return null;
-
 		CPType cptype = InstructionUtils.getCPType(str); 
 		if ( cptype == null ) 
 			throw new DMLRuntimeException("Unable derive cptype for instruction: " + str);
@@ -308,16 +309,11 @@ public class CPInstructionParser extends InstructionParser
 		return cpinst;
 	}
 	
-	public static CPInstruction parseSingleInstruction ( CPType cptype, String str ) 
-		throws DMLRuntimeException 
-	{
+	public static CPInstruction parseSingleInstruction ( CPType cptype, String str ) {
 		ExecType execType = null; 
-		
 		if ( str == null || str.isEmpty() ) 
 			return null;
-		
-		switch(cptype) 
-		{
+		switch(cptype) {
 			case AggregateUnary:
 				return AggregateUnaryCPInstruction.parseInstruction(str);
 			
@@ -348,8 +344,8 @@ public class CPInstructionParser extends InstructionParser
 			case Reorg:
 				return ReorgCPInstruction.parseInstruction(str);
 				
-			case Convolution:
-				 return ConvolutionCPInstruction.parseInstruction(str);
+			case Dnn:
+				 return DnnCPInstruction.parseInstruction(str);
 				
 			case UaggOuterChain:
 				return UaggOuterChainCPInstruction.parseInstruction(str);
@@ -371,14 +367,10 @@ public class CPInstructionParser extends InstructionParser
 				
 			case External:
 				return FunctionCallCPInstruction.parseInstruction(str);
-				
-			case ParameterizedBuiltin: 
-				execType = ExecType.valueOf( str.split(Instruction.OPERAND_DELIM)[0] ); 
-				if( execType == ExecType.CP )
-					return ParameterizedBuiltinCPInstruction.parseInstruction(str);
-				else //exectype CP_FILE
-					return ParameterizedBuiltinCPFileInstruction.parseInstruction(str);
-	
+
+			case ParameterizedBuiltin:
+				return ParameterizedBuiltinCPInstruction.parseInstruction(str);
+			
 			case MultiReturnParameterizedBuiltin:
 				return MultiReturnParameterizedBuiltinCPInstruction.parseInstruction(str);
 				
@@ -401,7 +393,8 @@ public class CPInstructionParser extends InstructionParser
 			case Builtin: 
 				String []parts = InstructionUtils.getInstructionPartsWithValueType(str);
 				if ( parts[0].equals("log") || parts[0].equals("log_nz") ) {
-					if ( parts.length == 3 ) {
+					if ( parts.length == 3 || (parts.length == 4 &&
+						UtilFunctions.isIntegerNumber(parts[3])) ) {
 						// B=log(A), y=log(x)
 						return UnaryCPInstruction.parseInstruction(str);
 					} else if ( parts.length == 4 ) {
