@@ -28,6 +28,8 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.LocalFileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.sysml.api.DMLScript;
+import org.apache.sysml.conf.CompilerConfig.ConfigType;
+import org.apache.sysml.conf.ConfigurationManager;
 import org.apache.sysml.lops.Lop;
 import org.apache.sysml.lops.UnaryCP;
 import org.apache.sysml.parser.Expression.DataType;
@@ -38,18 +40,18 @@ import org.apache.sysml.runtime.controlprogram.caching.FrameObject;
 import org.apache.sysml.runtime.controlprogram.caching.MatrixObject;
 import org.apache.sysml.runtime.controlprogram.caching.MatrixObject.UpdateType;
 import org.apache.sysml.runtime.controlprogram.context.ExecutionContext;
-import org.apache.sysml.runtime.controlprogram.parfor.ProgramConverter;
+import org.apache.sysml.runtime.util.ProgramConverter;
 import org.apache.sysml.runtime.controlprogram.parfor.util.IDSequence;
 import org.apache.sysml.runtime.instructions.Instruction;
 import org.apache.sysml.runtime.instructions.InstructionUtils;
+import org.apache.sysml.runtime.io.FileFormatPropertiesCSV;
+import org.apache.sysml.runtime.io.FileFormatProperties;
 import org.apache.sysml.runtime.io.IOUtilFunctions;
 import org.apache.sysml.runtime.io.WriterMatrixMarket;
 import org.apache.sysml.runtime.io.WriterTextCSV;
 import org.apache.sysml.runtime.matrix.MatrixCharacteristics;
-import org.apache.sysml.runtime.matrix.MatrixFormatMetaData;
+import org.apache.sysml.runtime.matrix.MetaDataFormat;
 import org.apache.sysml.runtime.matrix.MetaData;
-import org.apache.sysml.runtime.matrix.data.CSVFileFormatProperties;
-import org.apache.sysml.runtime.matrix.data.FileFormatProperties;
 import org.apache.sysml.runtime.matrix.data.FrameBlock;
 import org.apache.sysml.runtime.matrix.data.InputInfo;
 import org.apache.sysml.runtime.matrix.data.MatrixBlock;
@@ -86,7 +88,7 @@ public class VariableCPInstruction extends CPInstruction {
 		CopyVariable,
 		MoveVariable,
 		RemoveVariable,
-		RemoveVariableAndFile,		 
+		RemoveVariableAndFile,
 		CastAsScalarVariable, 
 		CastAsMatrixVariable, 
 		CastAsFrameVariable,
@@ -98,61 +100,55 @@ public class VariableCPInstruction extends CPInstruction {
 		SetFileName, 
 	}
 	
-	private static IDSequence _uniqueVarID;	
+	private static final IDSequence _uniqueVarID = new IDSequence(true);
 	private static final int CREATEVAR_FILE_NAME_VAR_POS=3;
 	
 	private final VariableOperationCode opcode;
 	private final List<CPOperand> inputs;
 	private final CPOperand output;
-	private MetaData metadata;
-	private UpdateType _updateType;
+	private final MetaData metadata;
+	private final UpdateType _updateType;
 	
 	// Frame related members
-	private String _schema;
+	private final String _schema;
 	
 	// CSV related members (used only in createvar instructions)
-	private FileFormatProperties _formatProperties;
-	
-	static {
-		_uniqueVarID  = new IDSequence(true); 
-	}
+	private final FileFormatProperties _formatProperties;
 
 	private VariableCPInstruction(VariableOperationCode op, CPOperand in1, CPOperand in2, CPOperand in3, CPOperand out,
-			int _arity, String sopcode, String istr) {
-		super(sopcode, istr);
-		_cptype = CPINSTRUCTION_TYPE.Variable;
+			MetaData meta, FileFormatProperties fprops, String schema, UpdateType utype, String sopcode, String istr) {
+		super(CPType.Variable, sopcode, istr);
 		opcode = op;
-		inputs = new ArrayList<CPOperand>();
+		inputs = new ArrayList<>();
 		addInput(in1);
 		addInput(in2);
 		addInput(in3);
 		output = out;
-
-		_formatProperties = null;
-		_schema = null;
-	}
-
-	// This version of the constructor is used only in case of CreateVariable
-	private VariableCPInstruction(VariableOperationCode op, CPOperand in1, CPOperand in2, CPOperand in3, MetaData md,
-			UpdateType updateType, int _arity, String schema, String sopcode, String istr) {
-		this(op, in1, in2, in3, (CPOperand) null, _arity, sopcode, istr);
-		metadata = md;
-		_updateType = updateType;
+		metadata = meta;
+		_formatProperties = fprops;
 		_schema = schema;
+		_updateType = utype;
+	}
+	
+	private VariableCPInstruction(VariableOperationCode op, CPOperand in1, CPOperand in2, CPOperand in3, CPOperand out,
+			String sopcode, String istr) {
+		this(op, in1, in2, in3, out, null, null, null, null, sopcode, istr);
 	}
 
 	// This version of the constructor is used only in case of CreateVariable
 	private VariableCPInstruction(VariableOperationCode op, CPOperand in1, CPOperand in2, CPOperand in3, MetaData md,
-			UpdateType updateType, int _arity, FileFormatProperties formatProperties, String schema, String sopcode,
+			UpdateType updateType, String schema, String sopcode, String istr) {
+		this(op, in1, in2, in3, null, md, null, schema, updateType, sopcode, istr);
+	}
+
+	// This version of the constructor is used only in case of CreateVariable
+	private VariableCPInstruction(VariableOperationCode op, CPOperand in1, CPOperand in2, CPOperand in3, MetaData md,
+			UpdateType updateType, FileFormatProperties formatProperties, String schema, String sopcode,
 			String istr) {
-		this(op, in1, in2, in3, (CPOperand) null, _arity, sopcode, istr);
-		metadata = md;
-		_updateType = updateType;
-		_formatProperties = formatProperties;
-		_schema = schema;
+		this(op, in1, in2, in3, null, md, formatProperties, schema, updateType, sopcode, istr);
 	}
 
-	private static VariableOperationCode getVariableOperationCode ( String str ) throws DMLRuntimeException {
+	private static VariableOperationCode getVariableOperationCode ( String str ) {
 		
 		if ( str.equalsIgnoreCase("createvar"))
 			return VariableOperationCode.CreateVariable;
@@ -235,10 +231,6 @@ public class VariableCPInstruction extends CPInstruction {
 		return _formatProperties;
 	}
 	
-	public void setFormatProperties(FileFormatProperties prop) {
-		_formatProperties = prop;
-	}
-	
 	public List<CPOperand> getInputs() {
 		return inputs;
 	}
@@ -287,13 +279,10 @@ public class VariableCPInstruction extends CPInstruction {
 		}
 	}
 	
-	public static VariableCPInstruction parseInstruction ( String str ) 
-		throws DMLRuntimeException 
-	{
+	public static VariableCPInstruction parseInstruction ( String str ) {
 		String[] parts = InstructionUtils.getInstructionPartsWithValueType ( str );
 		String opcode = parts[0];
 		VariableOperationCode voc = getVariableOperationCode(opcode);
-		int _arity = -1;
 		
 		if ( voc == VariableOperationCode.CreateVariable ){
 			if ( parts.length < 5 )  //&& parts.length != 10 )
@@ -311,9 +300,8 @@ public class VariableCPInstruction extends CPInstruction {
 				throw new DMLRuntimeException("Invalid number of operands in write instruction: " + str);
 		}
 		else {
-			_arity = getArity(voc);
 			if( voc != VariableOperationCode.RemoveVariable )
-				InstructionUtils.checkNumFields ( parts, _arity ); // no output
+				InstructionUtils.checkNumFields ( parts, getArity(voc) ); // no output
 		}
 		
 		CPOperand in1=null, in2=null, in3=null, in4=null, out=null;
@@ -328,7 +316,7 @@ public class VariableCPInstruction extends CPInstruction {
 			in1 = new CPOperand(parts[1], vt, dt);
 			// file name
 			in2 = new CPOperand(parts[2], ValueType.STRING, DataType.SCALAR);
-			// file name override flag
+			// file name override flag (always literal)
 			in3 = new CPOperand(parts[3], ValueType.BOOLEAN, DataType.SCALAR);
 			
 			// format 
@@ -360,7 +348,7 @@ public class VariableCPInstruction extends CPInstruction {
 			else {
 				throw new DMLRuntimeException("Invalid number of operands in createvar instruction: " + str);
 			}
-			MatrixFormatMetaData iimd = new MatrixFormatMetaData(mc, oi, ii);
+			MetaDataFormat iimd = new MetaDataFormat(mc, oi, ii);
 			UpdateType updateType = UpdateType.COPY;
 			if ( parts.length >= 12 )
 				updateType = UpdateType.valueOf(parts[11].toUpperCase());
@@ -377,7 +365,7 @@ public class VariableCPInstruction extends CPInstruction {
 					boolean hasHeader = Boolean.parseBoolean(parts[12]);
 					String delim = parts[13];
 					boolean sparse = Boolean.parseBoolean(parts[14]);
-					fmtProperties = new CSVFileFormatProperties(hasHeader, delim, sparse) ;
+					fmtProperties = new FileFormatPropertiesCSV(hasHeader, delim, sparse) ;
 				}
 				else {
 					boolean hasHeader = Boolean.parseBoolean(parts[12]);
@@ -387,12 +375,12 @@ public class VariableCPInstruction extends CPInstruction {
 					String naStrings = null;
 					if ( parts.length == 17+extSchema )
 						naStrings = parts[16];
-					fmtProperties = new CSVFileFormatProperties(hasHeader, delim, fill, fillValue, naStrings) ;
+					fmtProperties = new FileFormatPropertiesCSV(hasHeader, delim, fill, fillValue, naStrings) ;
 				}
-				return new VariableCPInstruction(VariableOperationCode.CreateVariable, in1, in2, in3, iimd, updateType, parts.length, fmtProperties, schema, opcode, str);
+				return new VariableCPInstruction(VariableOperationCode.CreateVariable, in1, in2, in3, iimd, updateType, fmtProperties, schema, opcode, str);
 			}
 			else {
-				return new VariableCPInstruction(VariableOperationCode.CreateVariable, in1, in2, in3, iimd, updateType, parts.length, schema, opcode, str);
+				return new VariableCPInstruction(VariableOperationCode.CreateVariable, in1, in2, in3, iimd, updateType, schema, opcode, str);
 			}
 		case AssignVariable:
 			in1 = new CPOperand(parts[1]);
@@ -414,7 +402,7 @@ public class VariableCPInstruction extends CPInstruction {
 			
 		case RemoveVariable:
 			VariableCPInstruction rminst = new VariableCPInstruction(
-				getVariableOperationCode(opcode), null, null, null, out, _arity, opcode, str);
+				getVariableOperationCode(opcode), null, null, null, out, opcode, str);
 			for( int i=1; i<parts.length; i++ )
 				rminst.addInput(new CPOperand(parts[i], ValueType.UNKNOWN, DataType.SCALAR));
 			return rminst;
@@ -442,22 +430,21 @@ public class VariableCPInstruction extends CPInstruction {
 			in2 = new CPOperand(parts[2]);
 			in3 = new CPOperand(parts[3]);
 			
-			VariableCPInstruction inst = new VariableCPInstruction(getVariableOperationCode(opcode), in1, in2, in3, out, _arity, opcode, str); 
-			
+			FileFormatProperties fprops = null;
 			if ( in3.getName().equalsIgnoreCase("csv") ) {
 				boolean hasHeader = Boolean.parseBoolean(parts[4]);
 				String delim = parts[5];
 				boolean sparse = Boolean.parseBoolean(parts[6]);
-				FileFormatProperties formatProperties = new CSVFileFormatProperties(hasHeader, delim, sparse);
-				inst.setFormatProperties(formatProperties);
+				fprops = new FileFormatPropertiesCSV(hasHeader, delim, sparse);
 				in4 = new CPOperand(parts[7]); // description
-				inst.addInput(in4);
 			} else {
-				FileFormatProperties ffp = new FileFormatProperties();
-				inst.setFormatProperties(ffp);
+				fprops = new FileFormatProperties();
 				in4 = new CPOperand(parts[4]); // description
-				inst.addInput(in4);
 			}
+			VariableCPInstruction inst = new VariableCPInstruction(
+				getVariableOperationCode(opcode), in1, in2, in3, out, null, fprops, null, null, opcode, str); 
+			inst.addInput(in4);
+			
 			return inst;
 			
 		case Read:
@@ -474,13 +461,11 @@ public class VariableCPInstruction extends CPInstruction {
 			break;
 		
 		}
-		return new VariableCPInstruction(getVariableOperationCode(opcode), in1, in2, in3, out, _arity, opcode, str);
+		return new VariableCPInstruction(getVariableOperationCode(opcode), in1, in2, in3, out, opcode, str);
 	}
 	
 	@Override
-	public void processInstruction(ExecutionContext ec) 
-		throws DMLRuntimeException 
-	{	
+	public void processInstruction(ExecutionContext ec) {
 		switch ( opcode ) 
 		{ 
 		case CreateVariable:
@@ -489,21 +474,19 @@ public class VariableCPInstruction extends CPInstruction {
 				//create new variable for symbol table and cache
 				//(existing objects gets cleared through rmvar instructions)
 				String fname = getInput2().getName();
-				
 				// check if unique filename needs to be generated
-				boolean overrideFileName = ((BooleanObject) ec.getScalarInput(getInput3().getName(), getInput3().getValueType(), true)).getBooleanValue();
-				if ( overrideFileName ) {
-					fname = fname + "_" + _uniqueVarID.getNextID();
+				if( Boolean.parseBoolean(getInput3().getName()) ) {
+					fname = new StringBuilder(fname.length()+16).append(fname)
+						.append('_').append(_uniqueVarID.getNextID()).toString();
 				}
-				
 				MatrixObject mobj = new MatrixObject(getInput1().getValueType(), fname );
-				mobj.setVarName(getInput1().getName());
-				mobj.setDataType(DataType.MATRIX);
 				//clone meta data because it is updated on copy-on-write, otherwise there
 				//is potential for hidden side effects between variables.
 				mobj.setMetaData((MetaData)metadata.clone());
 				mobj.setFileFormatProperties(_formatProperties);
 				mobj.setUpdateType(_updateType);
+				mobj.enableCleanup(!getInput1().getName()
+					.startsWith(org.apache.sysml.lops.Data.PREAD_PREFIX));
 				ec.setVariable(getInput1().getName(), mobj);
 				if(DMLScript.STATISTICS && _updateType.isInPlace())
 					Statistics.incrementTotalUIPVar();
@@ -511,17 +494,17 @@ public class VariableCPInstruction extends CPInstruction {
 			else if( getInput1().getDataType() == DataType.FRAME ) {
 				String fname = getInput2().getName();
 				FrameObject fobj = new FrameObject(fname);
-				fobj.setVarName(getInput1().getName());
-				fobj.setDataType(DataType.FRAME);
 				fobj.setMetaData((MetaData)metadata.clone());
 				fobj.setFileFormatProperties(_formatProperties);
 				if( _schema != null )
 					fobj.setSchema(_schema); //after metadata
+				fobj.enableCleanup(!getInput1().getName()
+					.startsWith(org.apache.sysml.lops.Data.PREAD_PREFIX));
 				ec.setVariable(getInput1().getName(), fobj);
 			}
 			else if ( getInput1().getDataType() == DataType.SCALAR ){
-				ScalarObject sobj = null;
-				ec.setScalarOutput(getInput1().getName(), sobj);
+				//created variable not called for scalars
+				ec.setScalarOutput(getInput1().getName(), null);
 			}
 			else {
 				throw new DMLRuntimeException("Unexpected data type: " + getInput1().getDataType());
@@ -574,7 +557,7 @@ public class VariableCPInstruction extends CPInstruction {
 			break;
 			
 		case CastAsScalarVariable: //castAsScalarVariable
-			if( getInput1().getDataType()==DataType.FRAME ) {
+			if( getInput1().getDataType().isFrame() ) {
 				FrameBlock fBlock = ec.getFrameInput(getInput1().getName());
 				if( fBlock.getNumRows()!=1 || fBlock.getNumColumns()!=1 )
 					throw new DMLRuntimeException("Dimension mismatch - unable to cast frame '"+getInput1().getName()+"' of dimension ("+fBlock.getNumRows()+" x "+fBlock.getNumColumns()+") to scalar.");
@@ -583,7 +566,7 @@ public class VariableCPInstruction extends CPInstruction {
 				ec.setScalarOutput(output.getName(), 
 						ScalarObjectFactory.createScalarObject(fBlock.getSchema()[0], value));
 			}
-			else { //assume DataType.MATRIX otherwise
+			else if( getInput1().getDataType().isMatrix() ) {
 				MatrixBlock mBlock = ec.getMatrixInput(getInput1().getName(), getExtendedOpcode());
 				if( mBlock.getNumRows()!=1 || mBlock.getNumColumns()!=1 )
 					throw new DMLRuntimeException("Dimension mismatch - unable to cast matrix '"+getInput1().getName()+"' of dimension ("+mBlock.getNumRows()+" x "+mBlock.getNumColumns()+") to scalar.");
@@ -591,20 +574,56 @@ public class VariableCPInstruction extends CPInstruction {
 				ec.releaseMatrixInput(getInput1().getName(), getExtendedOpcode());
 				ec.setScalarOutput(output.getName(), new DoubleObject(value));
 			}
+			else if( getInput1().getDataType().isList() ) {
+				//TODO handling of cleanup status, potentially new object
+				ListObject list = (ListObject)ec.getVariable(getInput1().getName());
+				ec.setVariable(output.getName(), list.slice(0));
+			}
+			else {
+				throw new DMLRuntimeException("Unsupported data type "
+					+ "in as.scalar(): "+getInput1().getDataType().name());
+			}
 			break;
 		case CastAsMatrixVariable:{
-			MatrixBlock out = null;
-			if( getInput1().getDataType()==DataType.FRAME ) {
+			if( getInput1().getDataType().isFrame() ) {
 				FrameBlock fin = ec.getFrameInput(getInput1().getName());
-				out = DataConverter.convertToMatrixBlock(fin);
+				MatrixBlock out = DataConverter.convertToMatrixBlock(fin);
 				ec.releaseFrameInput(getInput1().getName());
+				ec.setMatrixOutput(output.getName(), out, getExtendedOpcode());
 			}
-			else { //assume DataType.SCALAR otherwise
-				ScalarObject scalarInput = ec.getScalarInput(getInput1().getName(), getInput1().getValueType(), getInput1().isLiteral());
-				out = new MatrixBlock(1,1,false);
-				out.quickSetValue(0, 0, scalarInput.getDoubleValue());		
+			else if( getInput1().getDataType().isScalar() ) {
+				ScalarObject scalarInput = ec.getScalarInput(
+					getInput1().getName(), getInput1().getValueType(), getInput1().isLiteral());
+				MatrixBlock out = new MatrixBlock(scalarInput.getDoubleValue());
+				ec.setMatrixOutput(output.getName(), out, getExtendedOpcode());
+			}
+			else if( getInput1().getDataType().isList() ) {
+				//TODO handling of cleanup status, potentially new object
+				ListObject list = (ListObject)ec.getVariable(getInput1().getName());
+				if( list.getLength() > 1 ) {
+					if( !list.checkAllDataTypes(DataType.SCALAR) )
+						throw new DMLRuntimeException("as.matrix over multi-entry list only allows scalars.");
+					MatrixBlock out = new MatrixBlock(list.getLength(), 1, false);
+					for( int i=0; i<list.getLength(); i++ )
+						out.quickSetValue(i, 0, ((ScalarObject)list.slice(i)).getDoubleValue());
+					ec.setMatrixOutput(output.getName(), out, getExtendedOpcode());
+				}
+				else {
+					//pass through matrix input or create 1x1 matrix for scalar
+					Data tmp = list.slice(0);
+					if( tmp instanceof ScalarObject && tmp.getValueType()!=ValueType.STRING ) {
+						MatrixBlock out = new MatrixBlock(((ScalarObject)tmp).getDoubleValue());
+						ec.setMatrixOutput(output.getName(), out, getExtendedOpcode());
+					} 
+					else {
+						ec.setVariable(output.getName(), tmp);
+					}
+				}
+			}
+			else {
+				throw new DMLRuntimeException("Unsupported data type "
+					+ "in as.matrix(): "+getInput1().getDataType().name());
 			}
-			ec.setMatrixOutput(output.getName(), out, getExtendedOpcode());
 			break;
 		}
 		case CastAsFrameVariable:{
@@ -613,7 +632,7 @@ public class VariableCPInstruction extends CPInstruction {
 				ScalarObject scalarInput = ec.getScalarInput(getInput1());
 				out = new FrameBlock(1, getInput1().getValueType());
 				out.ensureAllocatedColumns(1);
-				out.set(0, 0, scalarInput.getStringValue());	
+				out.set(0, 0, scalarInput.getStringValue());
 			}
 			else { //DataType.FRAME
 				MatrixBlock min = ec.getMatrixInput(getInput1().getName(), getExtendedOpcode());
@@ -624,13 +643,13 @@ public class VariableCPInstruction extends CPInstruction {
 			break;
 		}
 		case CastAsDoubleVariable:{ 
-			ScalarObject scalarInput = ec.getScalarInput(getInput1());
-			ec.setScalarOutput(output.getName(), new DoubleObject(scalarInput.getDoubleValue()));
+			ScalarObject in = ec.getScalarInput(getInput1());
+			ec.setScalarOutput(output.getName(), ScalarObjectFactory.castToDouble(in));
 			break;
 		}
-		case CastAsIntegerVariable:{ 
-			ScalarObject scalarInput = ec.getScalarInput(getInput1());
-			ec.setScalarOutput(output.getName(), new IntObject(scalarInput.getLongValue()));
+		case CastAsIntegerVariable:{
+			ScalarObject in = ec.getScalarInput(getInput1());
+			ec.setScalarOutput(output.getName(), ScalarObjectFactory.castToLong(in));
 			break;
 		}
 		case CastAsBooleanVariable:{ 
@@ -699,30 +718,28 @@ public class VariableCPInstruction extends CPInstruction {
 	 * Currently, applicable only when format=binaryblock.
 	 *  
 	 * @param ec execution context
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
 	@SuppressWarnings("rawtypes")
-	private void processMoveInstruction(ExecutionContext ec) throws DMLRuntimeException {
+	private void processMoveInstruction(ExecutionContext ec) {
 		
 		if ( getInput3() == null ) {
 			// example: mvvar tempA A
 			
 			// get source variable 
-			Data srcData = ec.getVariable(getInput1().getName());		
-				
+			Data srcData = ec.getVariable(getInput1().getName());
+			
 			if ( srcData == null ) {
 				throw new DMLRuntimeException("Unexpected error: could not find a data object "
 					+ "for variable name:" + getInput1().getName() + ", while processing instruction ");
 			}
 			
-			if( getInput2().getDataType().isMatrix() ) {
+			if( getInput2().getDataType().isMatrix() || getInput2().getDataType().isFrame() ) {
 				// remove existing variable bound to target name
 				Data tgt = ec.removeVariable(getInput2().getName());
 					
 				//cleanup matrix data on fs/hdfs (if necessary)
-				if ( tgt != null && tgt instanceof MatrixObject ) {
-					ec.cleanupMatrixObject((MatrixObject) tgt);
-				}
+				if( tgt != null )
+					ec.cleanupDataObject(tgt);
 			}
 			
 			// do the actual move
@@ -758,11 +775,10 @@ public class VariableCPInstruction extends CPInstruction {
 	 * Example: cpvar &lt;srcvar&gt; &lt;destvar&gt;
 	 * 
 	 * @param ec execution context
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	private void processCopyInstruction(ExecutionContext ec) throws DMLRuntimeException {
+	private void processCopyInstruction(ExecutionContext ec) {
 		// get source variable 
-		Data dd = ec.getVariable(getInput1().getName());		
+		Data dd = ec.getVariable(getInput1().getName());
 			
 		if ( dd == null ) 
 			throw new DMLRuntimeException("Unexpected error: could not find a data object for variable name:" + getInput1().getName() + ", while processing instruction " +this.toString());
@@ -771,9 +787,8 @@ public class VariableCPInstruction extends CPInstruction {
 		Data input2_data = ec.removeVariable(getInput2().getName());
 		
 		//cleanup matrix data on fs/hdfs (if necessary)
-		if ( input2_data != null && input2_data instanceof MatrixObject ) {
-			ec.cleanupMatrixObject((MatrixObject) input2_data);
-		}
+		if( input2_data != null )
+			ec.cleanupDataObject(input2_data);
 		
 		// do the actual copy!
 		ec.setVariable(getInput2().getName(), dd);
@@ -787,11 +802,8 @@ public class VariableCPInstruction extends CPInstruction {
 	 * the format given by the corresponding symbol table entry.
 	 * 
 	 * @param ec execution context
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	private void processWriteInstruction(ExecutionContext ec) 
-		throws DMLRuntimeException 
-	{
+	private void processWriteInstruction(ExecutionContext ec) {
 		//get filename (literal or variable expression)
 		String fname = ec.getScalarInput(getInput2().getName(), ValueType.STRING, getInput2().isLiteral()).getStringValue();
 		String desc = ec.getScalarInput(getInput4().getName(), ValueType.STRING, getInput4().isLiteral()).getStringValue();
@@ -825,21 +837,13 @@ public class VariableCPInstruction extends CPInstruction {
 	 * 
 	 * @param ec execution context
 	 * @param varname variable name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void processRemoveVariableInstruction( ExecutionContext ec, String varname ) 
-		throws DMLRuntimeException
-	{
+	public static void processRemoveVariableInstruction( ExecutionContext ec, String varname ) {
 		// remove variable from symbol table
-		Data input1_data = ec.removeVariable(varname);
-		
-		if ( input1_data == null )
-			throw new DMLRuntimeException("Unexpected error: could not find a data object for variable name:" + varname + ", while processing rmvar instruction.");
-
+		Data dat = ec.removeVariable(varname);
 		//cleanup matrix data on fs/hdfs (if necessary)
-		if ( input1_data instanceof MatrixObject ) {
-			ec.cleanupMatrixObject( (MatrixObject) input1_data );
-		}
+		if( dat != null )
+			ec.cleanupDataObject(dat);
 	}
 	
 	/**
@@ -847,11 +851,8 @@ public class VariableCPInstruction extends CPInstruction {
 	 * 
 	 * @param ec execution context
 	 * @param fname file name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	private void writeCSVFile(ExecutionContext ec, String fname) 
-		throws DMLRuntimeException 
-	{
+	private void writeCSVFile(ExecutionContext ec, String fname) {
 		MatrixObject mo = ec.getMatrixObject(getInput1().getName());
 		String outFmt = "csv";
 		
@@ -862,10 +863,10 @@ public class VariableCPInstruction extends CPInstruction {
 		}
 		else {
 			try {
-				OutputInfo oi = ((MatrixFormatMetaData)mo.getMetaData()).getOutputInfo();
-				MatrixCharacteristics mc = ((MatrixFormatMetaData)mo.getMetaData()).getMatrixCharacteristics();
+				OutputInfo oi = ((MetaDataFormat)mo.getMetaData()).getOutputInfo();
+				MatrixCharacteristics mc = ((MetaDataFormat)mo.getMetaData()).getMatrixCharacteristics();
 				if(oi == OutputInfo.CSVOutputInfo) {
-					WriterTextCSV writer = new WriterTextCSV((CSVFileFormatProperties)_formatProperties);
+					WriterTextCSV writer = new WriterTextCSV((FileFormatPropertiesCSV)_formatProperties);
 					writer.addHeaderToCSV(mo.getFileName(), fname, mc.getRows(), mc.getCols());
 				}
 				else if ( oi == OutputInfo.BinaryBlockOutputInfo || oi == OutputInfo.TextCellOutputInfo ) {
@@ -888,11 +889,8 @@ public class VariableCPInstruction extends CPInstruction {
 	 * 
 	 * @param ec execution context
 	 * @param fname file name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	private void writeMMFile(ExecutionContext ec, String fname) 
-		throws DMLRuntimeException 
-	{
+	private void writeMMFile(ExecutionContext ec, String fname) {
 		MatrixObject mo = ec.getMatrixObject(getInput1().getName());
 		String outFmt = "matrixmarket";
 		if(mo.isDirty()) {
@@ -901,12 +899,12 @@ public class VariableCPInstruction extends CPInstruction {
 			mo.exportData(fname, outFmt);
 		}
 		else {
-			OutputInfo oi = ((MatrixFormatMetaData)mo.getMetaData()).getOutputInfo();
+			OutputInfo oi = ((MetaDataFormat)mo.getMetaData()).getOutputInfo();
 			MatrixCharacteristics mc = mo.getMatrixCharacteristics();
 			if(oi == OutputInfo.TextCellOutputInfo) {
 				try {
-					WriterMatrixMarket writer = new WriterMatrixMarket();
-					writer.mergeTextcellToMatrixMarket(mo.getFileName(), fname, mc.getRows(), mc.getCols(), mc.getNonZeros());
+					WriterMatrixMarket.mergeTextcellToMatrixMarket(mo.getFileName(), 
+						fname, mc.getRows(), mc.getCols(), mc.getNonZeros());
 				} catch (IOException e) {
 					throw new DMLRuntimeException(e);
 				}
@@ -924,11 +922,8 @@ public class VariableCPInstruction extends CPInstruction {
 	 * 
 	 * @param ec execution context
 	 * @param fname file name
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	private void writeScalarToHDFS(ExecutionContext ec, String fname) 
-		throws DMLRuntimeException 
-	{
+	private void writeScalarToHDFS(ExecutionContext ec, String fname) {
 		try {
 			ScalarObject scalar = ec.getScalarInput(getInput1().getName(), 
 				getInput1().getValueType(), getInput1().isLiteral());
@@ -946,9 +941,7 @@ public class VariableCPInstruction extends CPInstruction {
 		}
 	}
 	
-	private static void cleanDataOnHDFS(MatrixObject mo) 
-		throws DMLRuntimeException 
-	{
+	private static void cleanDataOnHDFS(MatrixObject mo) {
 		try {
 			String fpath = mo.getFileName();
 			if (fpath != null) {
@@ -969,17 +962,10 @@ public class VariableCPInstruction extends CPInstruction {
 			sb.append(Lop.OPERAND_DELIMITOR);
 			sb.append(varName);
 		}
-		String str = sb.toString();
-		
-		try {
-			return parseInstruction(str);
-		} 
-		catch (DMLRuntimeException ex) {
-			throw new RuntimeException(ex);
-		}
+		return parseInstruction(sb.toString());
 	}
 	
-	public static Instruction prepareCopyInstruction(String srcVar, String destVar) throws DMLRuntimeException {
+	public static Instruction prepareCopyInstruction(String srcVar, String destVar) {
 		StringBuilder sb = new StringBuilder();
 		sb.append("CP");
 		sb.append(Lop.OPERAND_DELIMITOR);
@@ -988,12 +974,10 @@ public class VariableCPInstruction extends CPInstruction {
 		sb.append(srcVar);
 		sb.append(Lop.OPERAND_DELIMITOR);
 		sb.append(destVar);
-		String str = sb.toString();
-
-		return parseInstruction(str);
+		return parseInstruction(sb.toString());
 	}
 	
-	public static Instruction prepareMoveInstruction(String srcVar, String destFileName, String format) throws DMLRuntimeException {
+	public static Instruction prepareMoveInstruction(String srcVar, String destFileName, String format) {
 		StringBuilder sb = new StringBuilder();
 		sb.append("CP");
 		sb.append(Lop.OPERAND_DELIMITOR);
@@ -1005,11 +989,10 @@ public class VariableCPInstruction extends CPInstruction {
 		sb.append(Lop.OPERAND_DELIMITOR);
 		sb.append(format);
 		String str = sb.toString();
-
 		return parseInstruction(str);
 	}
 	
-	public static Instruction prepareMoveInstruction(String srcVar, String destVar) throws DMLRuntimeException {
+	public static Instruction prepareMoveInstruction(String srcVar, String destVar) {
 		// example: mvvar tempA A 
 		StringBuilder sb = new StringBuilder();
 		sb.append("CP");
@@ -1020,11 +1003,15 @@ public class VariableCPInstruction extends CPInstruction {
 		sb.append(Lop.OPERAND_DELIMITOR);
 		sb.append(destVar);
 		String str = sb.toString();
-
 		return parseInstruction(str);
 	}
 	
 	private static String getBasicCreateVarString(String varName, String fileName, boolean fNameOverride, DataType dt, String format) {
+		//note: the filename override property leads to concatenation of unique ids in order to 
+		//ensure conflicting filenames for objects that originate from the same instruction
+		boolean lfNameOverride = fNameOverride && !ConfigurationManager
+			.getCompilerConfigFlag(ConfigType.IGNORE_TEMPORARY_FILENAMES);
+		
 		StringBuilder sb = new StringBuilder();
 		sb.append("CP");
 		sb.append(Lop.OPERAND_DELIMITOR);
@@ -1035,7 +1022,7 @@ public class VariableCPInstruction extends CPInstruction {
 		sb.append(fileName);		// Constant CREATEVAR_FILE_NAME_VAR_POS is used to find a position of filename within a string generated through this function.
 									// If this position of filename within this string changes then constant CREATEVAR_FILE_NAME_VAR_POS to be updated.
 		sb.append(Lop.OPERAND_DELIMITOR);
-		sb.append(fNameOverride);
+		sb.append(lfNameOverride);
 		sb.append(Lop.OPERAND_DELIMITOR);
 		sb.append(dt.toString());
 		sb.append(Lop.OPERAND_DELIMITOR);
@@ -1043,11 +1030,11 @@ public class VariableCPInstruction extends CPInstruction {
 		return sb.toString();
 	}
 	
-	public static Instruction prepareCreateMatrixVariableInstruction(String varName, String fileName, boolean fNameOverride, String format) throws DMLRuntimeException {
+	public static Instruction prepareCreateMatrixVariableInstruction(String varName, String fileName, boolean fNameOverride, String format) {
 		return parseInstruction(getBasicCreateVarString(varName, fileName, fNameOverride, DataType.MATRIX, format));
 	}
 
-	public static Instruction prepareCreateVariableInstruction(String varName, String fileName, boolean fNameOverride, DataType dt, String format, MatrixCharacteristics mc, UpdateType update) throws DMLRuntimeException {
+	public static Instruction prepareCreateVariableInstruction(String varName, String fileName, boolean fNameOverride, DataType dt, String format, MatrixCharacteristics mc, UpdateType update) {
 		StringBuilder sb = new StringBuilder();
 		sb.append(getBasicCreateVarString(varName, fileName, fNameOverride, dt, format));
 		
@@ -1069,7 +1056,7 @@ public class VariableCPInstruction extends CPInstruction {
 		return parseInstruction(str);
 	}	
 	
-	public static Instruction prepareCreateVariableInstruction(String varName, String fileName, boolean fNameOverride, DataType dt, String format, MatrixCharacteristics mc, UpdateType update, boolean hasHeader, String delim, boolean sparse) throws DMLRuntimeException {
+	public static Instruction prepareCreateVariableInstruction(String varName, String fileName, boolean fNameOverride, DataType dt, String format, MatrixCharacteristics mc, UpdateType update, boolean hasHeader, String delim, boolean sparse) {
 		StringBuilder sb = new StringBuilder();
 		sb.append(getBasicCreateVarString(varName, fileName, fNameOverride, dt, format));
 		
@@ -1099,9 +1086,7 @@ public class VariableCPInstruction extends CPInstruction {
 	}	
 	
 	@Override
-	public void updateInstructionThreadID(String pattern, String replace)
-		throws DMLRuntimeException
-	{
+	public void updateInstructionThreadID(String pattern, String replace) {
 		if(    opcode == VariableOperationCode.CreateVariable
 			|| opcode == VariableOperationCode.SetFileName )
 		{
