@@ -19,6 +19,7 @@
 package org.apache.sysml.runtime.matrix.data;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutorService;
@@ -30,8 +31,10 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.sysml.api.DMLScript;
 import org.apache.sysml.hops.OptimizerUtils;
 import org.apache.sysml.runtime.DMLRuntimeException;
+import org.apache.sysml.runtime.functionobjects.KahanPlus;
+import org.apache.sysml.runtime.instructions.cp.KahanObject;
 import org.apache.sysml.runtime.util.CommonThreadPool;
-import org.apache.sysml.runtime.util.ConvolutionUtils;
+import org.apache.sysml.runtime.util.DnnUtils;
 
 /*
  * This class allows users to invoke deep learning related operations 
@@ -140,9 +143,8 @@ public class LibMatrixDNN {
 	 * @param filter filter
 	 * @param outputBlock output of convolution
 	 * @param params convolution parameters
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void conv2d(MatrixBlock input, MatrixBlock filter, MatrixBlock outputBlock, ConvolutionParameters params) throws DMLRuntimeException {
+	public static void conv2d(MatrixBlock input, MatrixBlock filter, MatrixBlock outputBlock, DnnParameters params) {
 		LibMatrixDNN.checkInputsConv2d(input, filter, outputBlock, params);
 		if(params.bias != null && params.bias.isInSparseFormat())
 			params.bias.sparseToDense(); // Since bias is extremely small array
@@ -161,9 +163,9 @@ public class LibMatrixDNN {
 	 * @param dout errors from next layer
 	 * @param outputBlock  output errors
 	 * @param params convolution parameters
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
-	 */
-	public static void conv2dBackwardData(MatrixBlock filter, MatrixBlock dout, MatrixBlock outputBlock, ConvolutionParameters params) throws DMLRuntimeException {
+	 
+*/
+	public static void conv2dBackwardData(MatrixBlock filter, MatrixBlock dout, MatrixBlock outputBlock, DnnParameters params) {
 		checkInputsConv2dBackwardData(filter, dout, outputBlock, params);
 		
 		long nnz = execute(LibMatrixDNNConv2d.getConv2dBackwardDataWorkers(params), params);
@@ -180,9 +182,8 @@ public class LibMatrixDNN {
 	 * @param dout errors from next layer
 	 * @param outputBlock  output errors
 	 * @param params convolution parameters
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void conv2dBackwardFilter(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, ConvolutionParameters params) throws DMLRuntimeException {
+	public static void conv2dBackwardFilter(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, DnnParameters params) {
 		checkInputsConv2dBackwardFilter(input, dout, outputBlock, params);
 		
 		execute(LibMatrixDNNConv2d.getConv2dBackwardFilterWorkers(params), params);
@@ -192,7 +193,7 @@ public class LibMatrixDNN {
 		outputBlock.examSparsity();
 	}
 	
-	public static void pooling(MatrixBlock input, MatrixBlock output, ConvolutionParameters params, PoolingType poolType) throws DMLRuntimeException {
+	public static void pooling(MatrixBlock input, MatrixBlock output, DnnParameters params, PoolingType poolType) {
 		params.input1 = input;
 		params.output = output;
 		
@@ -222,10 +223,9 @@ public class LibMatrixDNN {
 	 * @param params convolution parameters
 	 * @param performReluBackward perform ReLU backward
 	 * @param poolType type of pooling
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
 	public static void poolingBackward(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, 
-			ConvolutionParameters params, boolean performReluBackward, PoolingType poolType) throws DMLRuntimeException {
+			DnnParameters params, boolean performReluBackward, PoolingType poolType) {
 		params.input1 = input;
 		params.input2 = dout;
 		params.output = outputBlock;
@@ -269,11 +269,10 @@ public class LibMatrixDNN {
 	 * @param dout errors from next layer
 	 * @param outputBlock output matrix
 	 * @param numThreads number of threads
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void reluBackward(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, int numThreads) throws DMLRuntimeException {
+	public static void reluBackward(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, int numThreads) {
 		int N = input.getNumRows();
-		ConvolutionParameters params = new ConvolutionParameters(N, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, numThreads);
+		DnnParameters params = new DnnParameters(N, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, numThreads);
 		params.input1 = input;
 		params.input2 = dout;
 		params.output = outputBlock;
@@ -299,9 +298,8 @@ public class LibMatrixDNN {
 	 * @param bias bias matrix
 	 * @param outputBlock output matrix
 	 * @param numThreads number of threads
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void biasAdd(MatrixBlock input, MatrixBlock bias, MatrixBlock outputBlock, int numThreads) throws DMLRuntimeException {
+	public static void biasAdd(MatrixBlock input, MatrixBlock bias, MatrixBlock outputBlock, int numThreads) {
 		int N = input.getNumRows();
 		int K = bias.getNumRows();
 		int PQ = input.getNumColumns() / K;
@@ -313,23 +311,15 @@ public class LibMatrixDNN {
 		double [] outputArray = outputBlock.getDenseBlockValues();
 		if(input.isEmptyBlock()) {
 			for(int n = 0;  n < N; n++) 
-				ConvolutionUtils.fillBias(bias, outputArray, n, n+1, N, K, PQ);
+				DnnUtils.fillBias(bias, outputArray, n, n+1, N, K, PQ);
 		}
 		else {
 			// Handles both dense and sparse inputs and copies it to dense output
-			outputBlock.copy(input); 
-			int index = 0;
+			outputBlock.copy(input, false);
 			if(bias.isInSparseFormat())
 				bias.sparseToDense(); // Since bias is extremely small array
 			double [] biasArr = bias.getDenseBlockValues();
-			for(int n = 0; n < N; n++) {
-				for(int k = 0; k < K; k++) {
-					double biasVal = biasArr[k];
-					for(int pq = 0; pq < PQ; pq++, index++) {
-						outputArray[index] += biasVal;
-					}
-				}
-			}
+			addBias(outputArray, biasArr, 1, N, K, PQ);
 		}
 		
 		//post-processing: maintain nnz
@@ -337,6 +327,324 @@ public class LibMatrixDNN {
 		outputBlock.examSparsity();
 	}
 	
+	/**
+	 * Perform channel sum operation
+	 * 
+	 * @param input input matrix block
+	 * @param outputBlock output matrix block
+	 * @param C number of channels
+	 * @param HW height X width
+	 */
+	public static void channelSums(MatrixBlock input, MatrixBlock outputBlock, int C, int HW) {
+		double [] output = outputBlock.getDenseBlockValues();
+		if(input.isInSparseFormat()) {
+			SparseBlock sblock = input.getSparseBlock();
+			for(int n = 0; n < input.getNumRows(); n++) {
+				if( sblock.isEmpty(n) )
+					continue;
+				int apos = sblock.pos(n);
+				int alen = sblock.size(n);
+				int[] aix = sblock.indexes(n);
+				double[] avals = sblock.values(n);
+				
+				// Iterate over the sparse block
+				for(int j=apos; j<apos+alen; j++) {
+					// Note: the input is of shape [N, CHW]
+					int chw = aix[j];
+					
+					// Get individual zero-based c,h,w indexes from zero-based 'chw'
+					int c = chw / HW;
+					output[c] += avals[j];
+				}
+			}
+		}
+		else {
+			double [] inArr = input.getDenseBlockValues();
+			if(inArr != null) {
+				KahanPlus kplus = KahanPlus.getKahanPlusFnObject();
+				for(int c = 0; c < C; c++) {
+					KahanObject sum = new KahanObject(0.0, 0.0);
+					for(int n = 0; n < input.getNumRows(); n++) {
+						int index =  n*C*HW + c*HW;
+						for(int hw = 0; hw < HW; hw++, index++) {
+							kplus.execute2(sum, inArr[index]);
+						}
+					}
+					output[c] = sum._sum; 
+				}
+			}
+		}
+		outputBlock.recomputeNonZeros();
+	}
+	
+	public static void batchNorm2DBackward(MatrixBlock image, MatrixBlock dout, MatrixBlock scale, double epsilon,  
+			MatrixBlock resultSaveMean, MatrixBlock resultSaveInvVariance,
+			MatrixBlock dX, MatrixBlock dScale, MatrixBlock dBias) {
+		int N = image.getNumRows();
+		int K = scale.getNumRows();
+		int PQ = image.getNumColumns() / K;
+		channelSums(image, dBias, K, PQ);
+		// Since output
+		if(dBias.isInSparseFormat())
+			dBias.sparseToDense();
+		if(dScale.isInSparseFormat())
+			dScale.sparseToDense();
+		if(dX.isInSparseFormat())
+			dX.sparseToDense();
+		// Very small matrices
+		if(resultSaveMean.isInSparseFormat())
+			resultSaveMean.sparseToDense();
+		if(resultSaveInvVariance.isInSparseFormat())
+			resultSaveInvVariance.sparseToDense();
+		if(scale.isInSparseFormat())
+			scale.sparseToDense();
+		double [] dBiasArr = dBias.getDenseBlockValues();
+		double [] dScaleArr = dScale.getDenseBlockValues();
+		double [] dXArr = dX.getDenseBlockValues();
+		double [] mean = resultSaveMean.getDenseBlockValues();
+		double [] invVar = resultSaveInvVariance.getDenseBlockValues();
+		double [] scaleArr = scale.getDenseBlockValues();
+		// since K is relatively small, it reduces code complexity. We can avoid this in subsequent commits.
+		mean = (mean==null) ? new double[K] : mean; 
+		invVar = (invVar==null) ? new double[K] : invVar;
+		scaleArr = (scaleArr == null) ? new double[K] : scaleArr;
+		
+		// TODO: Handle sparse image and dout cases:
+		if(image.isInSparseFormat())
+			image.sparseToDense();
+		if(dout.isInSparseFormat())
+			dout.sparseToDense();
+		
+		if(!image.isInSparseFormat() && !dout.isInSparseFormat()) {
+			double [] imageArr = image.getDenseBlockValues();
+			double [] doutArr = dout.getDenseBlockValues();
+			double constant1 = Math.pow(N*PQ, -1);
+			int KPQ = K*PQ;
+			for(int k = 0; k < K; k++) {
+				double dvar = 0; 
+				double dmean_norm_branch = 0; double dmean_var_branch  = 0;
+				double sumDout = 0; double sum = 0;
+				for(int n = 0; n < N; n++) {
+					int index = n*KPQ + k*PQ; 
+					for(int pq = 0; pq < PQ; pq++, index++) {
+						double doutVal = doutArr != null ? doutArr[index] : 0;
+						double centered = imageArr != null ? imageArr[index] : 0;
+						centered -= mean[k];
+						double dnorm = doutVal*scaleArr[k];
+						dvar -= 0.5*centered*Math.pow(invVar[k], 3)*dnorm;
+						dmean_norm_branch -= dnorm*invVar[k];
+						sum += centered * invVar[k] * doutVal;
+						sumDout += doutVal;
+						dmean_var_branch -= 2*constant1*centered;
+					}
+				}
+				dBiasArr[k] = sumDout;
+				dScaleArr[k] = sum;
+				dmean_var_branch *= dvar;
+				double dmean = dmean_norm_branch + dmean_var_branch;
+				double dX_mean_branch = constant1*dmean;
+				
+				for(int n = 0; n < N; n++) {
+					int index = n*KPQ + k*PQ; 
+					for(int pq = 0; pq < PQ; pq++, index++) {
+						double doutVal = doutArr != null ? doutArr[index] : 0;
+						double centered = imageArr != null ? imageArr[index] : 0;
+						centered -= mean[k];
+						double dnorm = doutVal*scaleArr[k];
+						double dX_norm_branch = dnorm*invVar[k];
+						double dX_var_branch = 2*constant1*centered*dvar;
+						dXArr[index] = dX_norm_branch + dX_mean_branch + dX_var_branch;
+					}
+				}
+			}
+		}
+		else {
+			throw new DMLRuntimeException("Sparse format is not yet supported for batch norm backward");
+		}
+		dBias.recomputeNonZeros();
+		dScale.recomputeNonZeros();
+		dX.recomputeNonZeros();
+	}
+	
+	public static void batchNorm2D(MatrixBlock image, MatrixBlock scale, MatrixBlock bias, MatrixBlock runningMean, 
+			MatrixBlock runningVar, String phase, double epsilon, double mu,
+			MatrixBlock ret, MatrixBlock retRunningMean, MatrixBlock retRunningVar, 
+			MatrixBlock resultSaveMean, MatrixBlock resultSaveInvVariance) {
+		// Since bias, scale, runningMean, runningVar are extremely small array
+		if(bias.isInSparseFormat())
+			bias.sparseToDense();
+		double [] biasArr = bias.getDenseBlockValues();
+		if(scale.isInSparseFormat())
+			scale.sparseToDense();
+		double [] scaleArr = scale.getDenseBlockValues();
+		if(runningMean.isInSparseFormat())
+			runningMean.sparseToDense();
+		double [] runningMeanArr = runningMean.getDenseBlockValues(); // ema_mean
+		if(runningVar.isInSparseFormat())
+			runningVar.sparseToDense(); 
+		double [] runningVarArr = runningVar.getDenseBlockValues(); // ema_var
+		
+		double [] retRunningMeanArr = retRunningMean.getDenseBlockValues(); // ema_mean_upd
+		double [] retRunningVarArr = retRunningVar.getDenseBlockValues(); // ema_var_upd
+		double [] resultSaveMeanArr = resultSaveMean.getDenseBlockValues(); // cache_mean
+		double [] resultSaveInvVarianceArr = resultSaveInvVariance.getDenseBlockValues(); // cache_inv_var
+		
+		int N = image.getNumRows();
+		int K = bias.getNumRows(); // number of output channels
+		int PQ = image.getNumColumns() / K; // output height X output width
+		
+		if(phase.equalsIgnoreCase("train")) { 
+			computeBiasSumAndSumSquares(image, resultSaveMeanArr, resultSaveInvVarianceArr, K, PQ);
+			int NPQ = N*PQ;
+			for(int k = 0; k < K; k++) {
+				double mean = resultSaveMeanArr[k] / NPQ;
+				double var = resultSaveInvVarianceArr[k]/NPQ - Math.pow(mean, 2.0);
+				resultSaveMeanArr[k] = mean;
+				resultSaveInvVarianceArr[k] = Math.pow(Math.sqrt(var + epsilon), -1.0);
+				retRunningMeanArr[k] = mu*((runningMeanArr!=null)?runningMeanArr[k]:0) + (1-mu)*mean;
+				retRunningVarArr[k] = mu*((runningVarArr!=null)?runningVarArr[k]:0) + (1-mu)*mean;
+			}
+		}
+		else if(phase.equalsIgnoreCase("test")) {
+			copy(runningMean, retRunningMeanArr); // ema_mean_upd = ema_mean
+			copy(runningVar, retRunningVarArr); // ema_var_upd = ema_var
+			copy(runningMean, resultSaveMeanArr); // cache_mean = ema_mean
+			double invSqrtEps = Math.pow(Math.sqrt(epsilon), -1.0);
+			double [] inArr = runningVar.getDenseBlockValues();
+			if(inArr != null) {
+				for(int i = 0; i < inArr.length; i++) {
+					resultSaveInvVarianceArr[i] = Math.pow(Math.sqrt(inArr[i] + epsilon), -1.0);
+				}
+			}
+			else {
+				Arrays.fill(resultSaveInvVarianceArr, invSqrtEps);
+			}
+		}
+		else {
+			throw new DMLRuntimeException("Incorrect mode: Expected either train or test, but found " + phase);
+		}
+		
+		// Normalize, shift, and scale
+		double [] retArr = ret.getDenseBlockValues();
+		copy(image, retArr);
+		if(resultSaveMean != null && resultSaveInvVariance != null && biasArr != null && scaleArr != null) {
+			// Common scenario:
+			int index = 0;
+			for(int n = 0; n < N; n++) {
+				for(int k = 0; k < K; k++) {
+					for(int pq = 0; pq < PQ; pq++, index++) {
+						retArr[index] = (retArr[index]-resultSaveMeanArr[k])*resultSaveInvVarianceArr[k]*scaleArr[k] + biasArr[k];
+					}
+				}
+			}
+		}
+		else {
+			addBias(retArr, resultSaveMeanArr, -1, N, K, PQ);
+			multBias(retArr, resultSaveInvVarianceArr, N, K, PQ);
+			multBias(retArr, scaleArr, N, K, PQ);
+			addBias(retArr, biasArr, 1, N, K, PQ);
+		}
+		ret.recomputeNonZeros();
+		retRunningMean.recomputeNonZeros();
+		retRunningVar.recomputeNonZeros();
+		resultSaveMean.recomputeNonZeros();
+		resultSaveInvVariance.recomputeNonZeros();
+	}
+	
+	private static void copy(MatrixBlock input, double [] output) {
+		if(input.isInSparseFormat()) {
+			SparseBlock sblock = input.getSparseBlock();
+			int numCols = input.getNumColumns();
+			for(int n = 0; n < input.getNumRows(); n++) {
+				if( sblock.isEmpty(n) )
+					continue;
+				int apos = sblock.pos(n);
+				int alen = sblock.size(n);
+				int[] aix = sblock.indexes(n);
+				double[] avals = sblock.values(n);
+				
+				// Iterate over the sparse block
+				for(int j=apos; j<apos+alen; j++) {
+					output[n*numCols + aix[j]] = avals[j];
+				}
+			}
+		}
+		else {
+			double [] inputArr = input.getDenseBlockValues();
+			if(inputArr != null) {
+				System.arraycopy(inputArr, 0, output, 0, inputArr.length);
+			}
+		}
+	}
+	
+	public static void addBias(double[] a, double[] bias, double biasMultiplier, int N, int K, int PQ) {
+		if( bias == null )
+			return;
+		int index = 0;
+		for(int n = 0; n < N; n++) {
+			for(int k = 0; k < K; k++) {
+				double biasVal = biasMultiplier*bias[k];
+				for(int pq = 0; pq < PQ; pq++, index++)
+					a[index] += biasVal;
+			}
+		}
+	}
+	
+	public static void multBias(double[] a, double[] bias, int N, int K, int PQ) {
+		if( bias == null ) {
+			Arrays.fill(a, 0);
+			return;
+		}
+		int index = 0;
+		for(int n = 0; n < N; n++) {
+			for(int k = 0; k < K; k++) {
+				double biasVal = bias[k];
+				for(int pq = 0; pq < PQ; pq++, index++)
+					a[index] *= biasVal;
+			}
+		}
+	}
+	
+	private static void computeBiasSumAndSumSquares(MatrixBlock image, double [] sumArr, double [] sumSquaresArr, int K, int PQ) {
+		if(sumArr.length != K) {
+			throw new DMLRuntimeException("Expected the length of array to be " + K + ", but instead is " + sumArr.length);
+		}
+		if(sumSquaresArr.length != K) {
+			throw new DMLRuntimeException("Expected the length of array to be " + K + ", but instead is " + sumSquaresArr.length);
+		}
+		if(image.isInSparseFormat()) {
+			SparseBlock sblock = image.getSparseBlock();
+			for(int r = 0; r < image.getNumRows(); r++) {
+				if( sblock.isEmpty(r) )
+					continue;
+				int apos = sblock.pos(r);
+				int alen = sblock.size(r);
+				int[] aix = sblock.indexes(r);
+				double[] avals = sblock.values(r);
+				for(int j=apos; j<apos+alen; j++) {
+					int k = aix[j] / PQ;
+					sumArr[k] += avals[j];
+					sumSquaresArr[k] += Math.pow(avals[j], 2.0);
+				}
+			}
+		}
+		else {
+			double [] X = image.getDenseBlockValues();
+			int N = image.getNumRows();
+			if(X != null) {
+				int index = 0;
+				for(int n = 0; n < N; n++) {
+					for(int k = 0; k < K; k++) {
+						for(int pq = 0; pq < PQ; pq++, index++) {
+							sumArr[k] += X[index];
+							sumSquaresArr[k] += Math.pow(X[index], 2.0);
+						}
+					}
+				}
+			}
+		}
+	}
+	
 	
 	/**
 	 * Performs the operation corresponding to the DML script:
@@ -348,14 +656,13 @@ public class LibMatrixDNN {
 	 * @param bias bias matrix
 	 * @param outputBlock output matrix
 	 * @param numThreads number of threads
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void biasMultiply(MatrixBlock input, MatrixBlock bias, MatrixBlock outputBlock, int numThreads) throws DMLRuntimeException {
+	public static void biasMultiply(MatrixBlock input, MatrixBlock bias, MatrixBlock outputBlock, int numThreads) {
 		int N = input.getNumRows();
 		int K = bias.getNumRows();
 		int PQ = input.getNumColumns() / K;
 		
-		ConvolutionParameters params = new ConvolutionParameters(N, PQ, -1, -1, K, -1, -1, -1, -1, -1, -1, numThreads);
+		DnnParameters params = new DnnParameters(N, PQ, -1, -1, K, -1, -1, -1, -1, -1, -1, numThreads);
 		params.input1 = input;
 		params.input2 = bias;
 		params.output = outputBlock;
@@ -383,28 +690,27 @@ public class LibMatrixDNN {
 				}
 			}
 			else {
+				SparseBlock sblock = outputBlock.sparseBlock;
 				// First delete those elements which will become zero 
 				for(int k = 0; k < K; k++) {
 					if(biasArr[k] == 0) {
 						for(int n = 0; n < N; n++) {
-							outputBlock.sparseBlock.deleteIndexRange(n, k*PQ, (k+1)*PQ);
+							if( sblock.isEmpty(n) ) continue;
+							sblock.deleteIndexRange(n, k*PQ, (k+1)*PQ);
 						}
 					}
 				}
 				// Then perform bias_multiply for non-zero bias entries
 				for(int n = 0; n < N; n++) {
-					if( !outputBlock.sparseBlock.isEmpty(n) ) {
-						int apos = outputBlock.sparseBlock.pos(n);
-						int alen = outputBlock.sparseBlock.size(n);
-						int[] aix = outputBlock.sparseBlock.indexes(n);
-						double[] avals = outputBlock.sparseBlock.values(n);
-						
-						for(int j=apos; j<apos+alen; j++) {
-							// Since aix[j] => KPQ
-							int k = aix[j] % PQ;
-							if(biasArr[k] != 0)
-								avals[j] *= biasArr[k];
-						}
+					if( sblock.isEmpty(n) ) continue;
+					int apos = sblock.pos(n);
+					int alen = sblock.size(n);
+					int[] aix = sblock.indexes(n);
+					double[] avals = sblock.values(n);
+					for(int j=apos; j<apos+alen; j++) {
+						int k = aix[j] / PQ; //aix[j] KPQ
+						if(biasArr[k] != 0)
+							avals[j] *= biasArr[k];
 					}
 				}
 			}
@@ -423,9 +729,8 @@ public class LibMatrixDNN {
 	 *  
 	 * @param tasks deep learning related tasks
 	 * @param params convolution parameters
-	 * @throws DMLRuntimeException if the error occurs
 	 */
-	private static long execute(ArrayList<Callable<Long>> tasks, ConvolutionParameters params) throws DMLRuntimeException {
+	private static long execute(ArrayList<Callable<Long>> tasks, DnnParameters params) {
 		int k = OptimizerUtils.getConstrainedNumThreads(params.numThreads);
 		long lnnz = 0;
 		try {
@@ -451,16 +756,16 @@ public class LibMatrixDNN {
 		return lnnz;
 	}
 	
-	private static void checkOrThrowException(String msg, long lhs, long rhs) throws DMLRuntimeException {
+	private static void checkOrThrowException(String msg, long lhs, long rhs) {
 		if(lhs != rhs)
 			throw new DMLRuntimeException(msg + ":" + lhs + " != " + rhs);
 	}
-	private static void checkOrThrowException(String msg, long lhs, long rhs1, long rhs2, long rhs3) throws DMLRuntimeException {
+	private static void checkOrThrowException(String msg, long lhs, long rhs1, long rhs2, long rhs3) {
 		if(lhs != (rhs1*rhs2*rhs3))
 			throw new DMLRuntimeException(msg + ":" + lhs + " != (" + rhs1 + " * " + rhs2 + " * " + rhs3);
 	}
 	
-	static void checkInputsConv2dBackwardData(MatrixBlock filter, MatrixBlock dout, MatrixBlock outputBlock, ConvolutionParameters params)  throws DMLRuntimeException {
+	static void checkInputsConv2dBackwardData(MatrixBlock filter, MatrixBlock dout, MatrixBlock outputBlock, DnnParameters params) {
 		params.input1 = filter;
 		params.input2 = dout;
 		params.output = outputBlock;
@@ -485,7 +790,7 @@ public class LibMatrixDNN {
 		}
 	}
 	
-	static void checkInputsConv2dBackwardFilter(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, ConvolutionParameters params)  throws DMLRuntimeException {
+	static void checkInputsConv2dBackwardFilter(MatrixBlock input, MatrixBlock dout, MatrixBlock outputBlock, DnnParameters params) {
 		params.input1 = input;
 		params.input2 = dout;
 		params.output = outputBlock;
@@ -510,7 +815,7 @@ public class LibMatrixDNN {
 		}
 	}
 	
-	static void checkInputsConv2d(MatrixBlock input, MatrixBlock filter, MatrixBlock outputBlock, ConvolutionParameters params) throws DMLRuntimeException {
+	static void checkInputsConv2d(MatrixBlock input, MatrixBlock filter, MatrixBlock outputBlock, DnnParameters params) {
 		params.input1 = input;
 		params.input2 = filter;
 		params.output = outputBlock;
@@ -542,7 +847,7 @@ public class LibMatrixDNN {
 	 * 
 	 * @param params parameters required for max_pool and max_pool_backward operations
 	 */
-	private static void fillIndexesArray(ConvolutionParameters params) {
+	private static void fillIndexesArray(DnnParameters params) {
 		params.start_indexes_h = new int[params.P];
 		params.end_indexes_h = new int[params.P];
 		params.start_indexes_w = new int[params.Q];
