@@ -22,6 +22,7 @@ package org.apache.sysml.runtime.matrix.data;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.List;
 
 import org.apache.sysml.parser.Expression.ValueType;
 import org.apache.sysml.runtime.DMLRuntimeException;
@@ -45,9 +46,7 @@ import org.apache.sysml.runtime.util.UtilFunctions;
 public class OperationsOnMatrixValues 
 {
 	public static void performReorg(MatrixIndexes indexesIn, MatrixValue valueIn, MatrixIndexes indexesOut, 
-			         MatrixValue valueOut, ReorgOperator op, int startRow, int startColumn, int length) 
-		throws DMLRuntimeException
-	{
+			         MatrixValue valueOut, ReorgOperator op, int startRow, int startColumn, int length) {
 		//operate on the value indexes first
 		op.fn.execute(indexesIn, indexesOut);
 		
@@ -56,57 +55,43 @@ public class OperationsOnMatrixValues
 	}
 
 	public static void performAppend(MatrixValue valueIn1, MatrixValue valueIn2,
-			ArrayList<IndexedMatrixValue> outlist, int blockRowFactor, int blockColFactor,  boolean cbind, boolean m2IsLast, int nextNCol) 
-	throws DMLRuntimeException
-	{
+			ArrayList<IndexedMatrixValue> outlist, int blockRowFactor, int blockColFactor,  boolean cbind, boolean m2IsLast, int nextNCol) {
 		valueIn1.append(valueIn2, outlist, blockRowFactor, blockColFactor, cbind, m2IsLast, nextNCol);
 	}
 	
 	public static void performZeroOut(MatrixIndexes indexesIn, MatrixValue valueIn, 
-			MatrixIndexes indexesOut, MatrixValue valueOut, IndexRange range, boolean complementary) 
-	throws DMLRuntimeException
-	{
+			MatrixIndexes indexesOut, MatrixValue valueOut, IndexRange range, boolean complementary) {
 		valueIn.zeroOutOperations(valueOut, range, complementary);
 		indexesOut.setIndexes(indexesIn);
 	}
 	
 	// ------------- Ternary Operations -------------
 	public static void performCtable(MatrixIndexes indexesIn1, MatrixValue valueIn1, MatrixIndexes indexesIn2, MatrixValue valueIn2, 
-			MatrixIndexes indexesIn3, MatrixValue valueIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) 
-		throws DMLRuntimeException
-	{
+			MatrixIndexes indexesIn3, MatrixValue valueIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) {
 		//operation on the cells inside the value
 		valueIn1.ctableOperations(op, valueIn2, valueIn3, resultMap, resultBlock);
 	}
 	
 	public static void performCtable(MatrixIndexes indexesIn1, MatrixValue valueIn1, MatrixIndexes indexesIn2, MatrixValue valueIn2, 
-			double scalarIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op) 
-		throws DMLRuntimeException
-	{
+			double scalarIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op) {
 		//operation on the cells inside the value
 		valueIn1.ctableOperations(op, valueIn2, scalarIn3, false, resultMap, resultBlock);
 	}
 	
 	public static void performCtable(MatrixIndexes indexesIn1, MatrixValue valueIn1, double scalarIn2, 
-			double scalarIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) 
-		throws DMLRuntimeException
-	{
+			double scalarIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) {
 		//operation on the cells inside the value
 		valueIn1.ctableOperations(op, scalarIn2, scalarIn3, resultMap, resultBlock);
 	}
 	
 	public static void performCtable(MatrixIndexes indexesIn1, MatrixValue valueIn1, double scalarIn2, boolean left,
-			int brlen, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) 
-		throws DMLRuntimeException
-	{
+			int brlen, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) {
 		//operation on the cells inside the value
 		valueIn1.ctableOperations(op, indexesIn1, scalarIn2, left, brlen, resultMap, resultBlock);
 	}
 	
 	public static void performCtable(MatrixIndexes indexesIn1, MatrixValue valueIn1, double scalarIn2, 
-			MatrixIndexes indexesIn3, MatrixValue valueIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) 
-		throws DMLRuntimeException
-	{
+			MatrixIndexes indexesIn3, MatrixValue valueIn3, CTableMap resultMap, MatrixBlock resultBlock, Operator op ) {
 		//operation on the cells inside the value
 		valueIn1.ctableOperations(op, scalarIn2, valueIn3, resultMap, resultBlock);
 	}
@@ -114,16 +99,12 @@ public class OperationsOnMatrixValues
 	
 	//binary operations are those that the indexes of both cells have to be matched
 	public static void performBinaryIgnoreIndexes(MatrixValue value1, MatrixValue value2, 
-			MatrixValue valueOut, BinaryOperator op) 
-	throws DMLRuntimeException
-	{
+			MatrixValue valueOut, BinaryOperator op) {
 		value1.binaryOperations(op, value2, valueOut);
 	}
 
 	public static void startAggregation(MatrixValue valueOut, MatrixValue correction, AggregateOperator op, 
-			int rlen, int clen, boolean sparseHint, boolean imbededCorrection)
-		throws DMLRuntimeException
-	{
+			int rlen, int clen, boolean sparseHint, boolean imbededCorrection) {
 		int outRow=0, outCol=0, corRow=0, corCol=0;
 		if(op.correctionExists)
 		{
@@ -215,12 +196,18 @@ public class OperationsOnMatrixValues
 	}
 	
 	public static void incrementalAggregation(MatrixValue valueAgg, MatrixValue correction, MatrixValue valueAdd, 
-			AggregateOperator op, boolean imbededCorrection) throws DMLRuntimeException
+			AggregateOperator op, boolean imbededCorrection) {
+		incrementalAggregation(valueAgg, correction, valueAdd, op, imbededCorrection, true);
+	}
+	
+	
+	public static void incrementalAggregation(MatrixValue valueAgg, MatrixValue correction, MatrixValue valueAdd, 
+			AggregateOperator op, boolean imbededCorrection, boolean deep)
 	{
 		if(op.correctionExists)
 		{
 			if(!imbededCorrection || op.correctionLocation==CorrectionLocationType.NONE)
-				valueAgg.incrementalAggregate(op, correction, valueAdd);
+				valueAgg.incrementalAggregate(op, correction, valueAdd, deep);
 			else
 				valueAgg.incrementalAggregate(op, valueAdd);
 		}
@@ -229,9 +216,7 @@ public class OperationsOnMatrixValues
 	}
 	
 	public static void performAggregateUnary(MatrixIndexes indexesIn, MatrixValue valueIn, MatrixIndexes indexesOut, 
-			MatrixValue valueOut, AggregateUnaryOperator op,int brlen, int bclen)
-		throws DMLRuntimeException
-	{
+			MatrixValue valueOut, AggregateUnaryOperator op,int brlen, int bclen) {
 		//operate on the value indexes first
 		op.indexFn.execute(indexesIn, indexesOut);
 		
@@ -239,34 +224,28 @@ public class OperationsOnMatrixValues
 		valueIn.aggregateUnaryOperations(op, valueOut, brlen, bclen, indexesIn);
 	}
 	
-	public static void performAggregateBinary(MatrixIndexes indexes1, MatrixBlock value1, MatrixIndexes indexes2, MatrixBlock value2, 
-			MatrixIndexes indexesOut, MatrixBlock valueOut, AggregateBinaryOperator op)
-	throws DMLRuntimeException
-	{
+	public static MatrixBlock matMult(MatrixIndexes indexes1, MatrixBlock value1, MatrixIndexes indexes2, MatrixBlock value2, 
+			MatrixIndexes indexesOut, MatrixBlock valueOut, AggregateBinaryOperator op) {
 		//compute output index
 		indexesOut.setIndexes(indexes1.getRowIndex(), indexes2.getColumnIndex());
 		//perform on the value
 		if( value2 instanceof CompressedMatrixBlock )
-			value2.aggregateBinaryOperations(value1, value2, valueOut, op);
+			return value2.aggregateBinaryOperations(value1, value2, valueOut, op);
 		else //default
-			value1.aggregateBinaryOperations(indexes1, value1, indexes2, value2, valueOut, op);
+			return value1.aggregateBinaryOperations(indexes1, value1, indexes2, value2, valueOut, op);
 	}
 
-	public static MatrixValue performAggregateBinaryIgnoreIndexes(MatrixBlock value1, MatrixBlock value2,
-			MatrixBlock valueOut, AggregateBinaryOperator op) 
-	throws DMLRuntimeException {
+	public static MatrixBlock matMult(MatrixBlock value1, MatrixBlock value2,
+			MatrixBlock valueOut, AggregateBinaryOperator op) {
 		//perform on the value
 		if( value2 instanceof CompressedMatrixBlock )
-			value2.aggregateBinaryOperations(value1, value2, valueOut, op);
+			return value2.aggregateBinaryOperations(value1, value2, valueOut, op);
 		else
-			value1.aggregateBinaryOperations(value1, value2, valueOut, op);
-		return valueOut;
+			return value1.aggregateBinaryOperations(value1, value2, valueOut, op);
 	}
 
 	@SuppressWarnings("rawtypes")
-	public static ArrayList performSlice(IndexRange ixrange, int brlen, int bclen, int iix, int jix, CacheBlock in) 
-		throws DMLRuntimeException
-	{
+	public static List performSlice(IndexRange ixrange, int brlen, int bclen, int iix, int jix, CacheBlock in) {
 		if( in instanceof MatrixBlock )
 			return performSlice(ixrange, brlen, bclen, iix, jix, (MatrixBlock)in);
 		else if( in instanceof FrameBlock )
@@ -274,21 +253,15 @@ public class OperationsOnMatrixValues
 		throw new DMLRuntimeException("Unsupported cache block type: "+in.getClass().getName());
 	}
 	
-	
 	@SuppressWarnings("rawtypes")
-	public static ArrayList performSlice(IndexRange ixrange, int brlen, int bclen, int iix, int jix, MatrixBlock in) 
-			throws DMLRuntimeException
-	{
+	public static List performSlice(IndexRange ixrange, int brlen, int bclen, int iix, int jix, MatrixBlock in) {
 		IndexedMatrixValue imv = new IndexedMatrixValue(new MatrixIndexes(iix, jix), (MatrixBlock)in);
 		ArrayList<IndexedMatrixValue> outlist = new ArrayList<>();
 		performSlice(imv, ixrange, brlen, bclen, outlist);
-	
 		return SparkUtils.fromIndexedMatrixBlockToPair(outlist);
 	}
 
-	public static void performSlice(IndexedMatrixValue in, IndexRange ixrange, int brlen, int bclen, ArrayList<IndexedMatrixValue> outlist) 
-		throws DMLRuntimeException
-	{
+	public static void performSlice(IndexedMatrixValue in, IndexRange ixrange, int brlen, int bclen, ArrayList<IndexedMatrixValue> outlist) {
 		long cellIndexTopRow = UtilFunctions.computeCellIndex(in.getIndexes().getRowIndex(), brlen, 0);
 		long cellIndexBottomRow = UtilFunctions.computeCellIndex(in.getIndexes().getRowIndex(), brlen, in.getValue().getNumRows()-1);
 		long cellIndexLeftCol = UtilFunctions.computeCellIndex(in.getIndexes().getColumnIndex(), bclen, 0);
@@ -347,9 +320,7 @@ public class OperationsOnMatrixValues
 		in.getValue().slice(outlist, tmpRange, rowCut, colCut, brlen, bclen, boundaryRlen, boundaryClen);
 	}
 
-	public static void performShift(IndexedMatrixValue in, IndexRange ixrange, int brlen, int bclen, long rlen, long clen, ArrayList<IndexedMatrixValue> outlist) 
-		throws DMLRuntimeException
-	{
+	public static void performShift(IndexedMatrixValue in, IndexRange ixrange, int brlen, int bclen, long rlen, long clen, ArrayList<IndexedMatrixValue> outlist) {
 		MatrixIndexes ix = in.getIndexes();
 		MatrixBlock mb = (MatrixBlock)in.getValue();
 		
@@ -398,7 +369,7 @@ public class OperationsOnMatrixValues
 		}
 	}
 
-	public static void performMapGroupedAggregate( Operator op, IndexedMatrixValue inTarget, MatrixBlock groups, int ngroups, int brlen, int bclen, ArrayList<IndexedMatrixValue> outlist ) throws DMLRuntimeException
+	public static void performMapGroupedAggregate( Operator op, IndexedMatrixValue inTarget, MatrixBlock groups, int ngroups, int brlen, int bclen, ArrayList<IndexedMatrixValue> outlist )
 	{
 		MatrixIndexes ix = inTarget.getIndexes();
 		MatrixBlock target = (MatrixBlock)inTarget.getValue();
@@ -409,30 +380,27 @@ public class OperationsOnMatrixValues
 		if( out.getNumRows()<=brlen && out.getNumColumns()<=bclen )
 		{
 			//single output block
-			outlist.add( new IndexedMatrixValue(new MatrixIndexes(1,ix.getColumnIndex()), out) );	
+			outlist.add( new IndexedMatrixValue(new MatrixIndexes(1,ix.getColumnIndex()), out) );
 		}
 		else
 		{
-			//multiple output blocks (by op def, single column block )				
+			//multiple output blocks (by op def, single column block )
 			for(int blockRow = 0; blockRow < (int)Math.ceil(out.getNumRows()/(double)brlen); blockRow++)
 			{
-				int maxRow = (blockRow*brlen + brlen < out.getNumRows()) ? brlen : out.getNumRows() - blockRow*brlen;			
+				int maxRow = (blockRow*brlen + brlen < out.getNumRows()) ? brlen : out.getNumRows() - blockRow*brlen;
 				int row_offset = blockRow*brlen;
 
 				//copy submatrix to block
-				MatrixBlock tmp = out.slice( row_offset, row_offset+maxRow-1, 
-						             0, out.getNumColumns()-1, new MatrixBlock() );
+				MatrixBlock tmp = out.slice(row_offset, row_offset+maxRow-1);
 				
 				//append block to result cache
-				outlist.add(new IndexedMatrixValue(new MatrixIndexes(blockRow+1,ix.getColumnIndex()), tmp));			
+				outlist.add(new IndexedMatrixValue(new MatrixIndexes(blockRow+1,ix.getColumnIndex()), tmp));
 			}
 		}
 	}
 	
 	@SuppressWarnings("rawtypes")
-	public static ArrayList performSlice(IndexRange ixrange, int brlen, int bclen, int iix, int jix, FrameBlock in) 
-			throws DMLRuntimeException
-	{
+	public static ArrayList performSlice(IndexRange ixrange, int brlen, int bclen, int iix, int jix, FrameBlock in) {
 		Pair<Long, FrameBlock> lfp = new Pair<>(new Long(((iix-1)*brlen)+1), in);
 		ArrayList<Pair<Long, FrameBlock>> outlist = new ArrayList<>();
 		performSlice(lfp, ixrange, brlen, bclen, outlist);
@@ -449,11 +417,8 @@ public class OperationsOnMatrixValues
 	 * @param brlen number of rows in a block
 	 * @param bclen number of columns in a block
 	 * @param outlist list of pairs of frame blocks
-	 * @throws DMLRuntimeException if DMLRuntimeException occurs
 	 */
-	public static void performSlice(Pair<Long,FrameBlock> in, IndexRange ixrange, int brlen, int bclen, ArrayList<Pair<Long,FrameBlock>> outlist) 
-		throws DMLRuntimeException
-	{
+	public static void performSlice(Pair<Long,FrameBlock> in, IndexRange ixrange, int brlen, int bclen, ArrayList<Pair<Long,FrameBlock>> outlist) {
 		long index = in.getKey();
 		FrameBlock block = in.getValue();
 		
@@ -501,9 +466,7 @@ public class OperationsOnMatrixValues
 		block.slice(outlist, tmpRange, rowCut);
 	}
 
-	public static void performShift(Pair<Long,FrameBlock> in, IndexRange ixrange, int brlenLeft, int clenLeft/*, int bclen*/, long rlen, long clen, ArrayList<Pair<Long,FrameBlock>> outlist) 
-		throws DMLRuntimeException
-	{
+	public static void performShift(Pair<Long,FrameBlock> in, IndexRange ixrange, int brlenLeft, int clenLeft/*, int bclen*/, long rlen, long clen, ArrayList<Pair<Long,FrameBlock>> outlist) {
 		Long ix = in.getKey();
 		FrameBlock fb = in.getValue();
 		long start_lhs_globalRowIndex = ixrange.rowStart + (ix-1);
@@ -555,5 +518,4 @@ public class OperationsOnMatrixValues
 			outlist.add(new Pair<>((leftRowIndex-1)*brlenLeft+1, resultBlock));
 		}
 	}
-
 }
