@@ -47,12 +47,13 @@ import org.apache.sysml.parser.Expression.DataType;
 import org.apache.sysml.parser.Expression.ValueType;
 import org.apache.sysml.runtime.DMLRuntimeException;
 import org.apache.sysml.runtime.controlprogram.context.SparkExecutionContext;
+import org.apache.sysml.runtime.io.FileFormatPropertiesCSV;
 import org.apache.sysml.runtime.io.FrameReader;
 import org.apache.sysml.runtime.io.FrameReaderFactory;
 import org.apache.sysml.runtime.matrix.MatrixCharacteristics;
-import org.apache.sysml.runtime.matrix.data.CSVFileFormatProperties;
 import org.apache.sysml.runtime.matrix.data.FrameBlock;
 import org.apache.sysml.runtime.matrix.data.InputInfo;
+import org.apache.sysml.runtime.matrix.data.MatrixBlock;
 import org.apache.sysml.runtime.matrix.data.MatrixValue.CellIndex;
 import org.apache.sysml.runtime.matrix.data.OutputInfo;
 import org.apache.sysml.runtime.util.DataConverter;
@@ -504,6 +505,11 @@ public abstract class AutomatedTestBase
 		return matrix;
 	}
 
+	protected double[][] writeInputMatrixWithMTD(String name, MatrixBlock matrix, boolean bIncludeR) {
+		double[][] data = DataConverter.convertToDoubleMatrix(matrix);
+		return writeInputMatrixWithMTD(name, data, bIncludeR);
+	}
+	
 	protected double[][] writeInputMatrixWithMTD(String name, double[][] matrix, boolean bIncludeR) {
 		MatrixCharacteristics mc = new MatrixCharacteristics(matrix.length, matrix[0].length, OptimizerUtils.DEFAULT_BLOCKSIZE, OptimizerUtils.DEFAULT_BLOCKSIZE, -1);
 		return writeInputMatrixWithMTD(name, matrix, bIncludeR, mc);
@@ -713,9 +719,7 @@ public abstract class AutomatedTestBase
 	}
 
 
-	protected static FrameBlock readDMLFrameFromHDFS(String fileName, InputInfo iinfo)
-			throws DMLRuntimeException, IOException
-	{
+	protected static FrameBlock readDMLFrameFromHDFS(String fileName, InputInfo iinfo) throws IOException {
 		//read frame data from hdfs
 		String strFrameFileName = baseDirectory + OUTPUT_DIR + fileName;
 		FrameReader reader = FrameReaderFactory.createFrameReader(iinfo);
@@ -725,9 +729,7 @@ public abstract class AutomatedTestBase
 	}
 
 
-	protected static FrameBlock readDMLFrameFromHDFS(String fileName, InputInfo iinfo, MatrixCharacteristics md)
-			throws DMLRuntimeException, IOException
-	{
+	protected static FrameBlock readDMLFrameFromHDFS(String fileName, InputInfo iinfo, MatrixCharacteristics md) throws IOException {
 		//read frame data from hdfs
 		String strFrameFileName = baseDirectory + OUTPUT_DIR + fileName;
 		FrameReader reader = FrameReaderFactory.createFrameReader(iinfo);
@@ -735,13 +737,11 @@ public abstract class AutomatedTestBase
 		return reader.readFrameFromHDFS(strFrameFileName, md.getRows(), md.getCols());
 	}
 
-	protected static FrameBlock readRFrameFromHDFS(String fileName, InputInfo iinfo, MatrixCharacteristics md)
-			throws DMLRuntimeException, IOException
-	{
+	protected static FrameBlock readRFrameFromHDFS(String fileName, InputInfo iinfo, MatrixCharacteristics md) throws IOException {
 		//read frame data from hdfs
 		String strFrameFileName = baseDirectory + EXPECTED_DIR + fileName;
 
-		CSVFileFormatProperties fprop = new CSVFileFormatProperties();
+		FileFormatPropertiesCSV fprop = new FileFormatPropertiesCSV();
 		fprop.setHeader(true);
 		FrameReader reader = FrameReaderFactory.createFrameReader(iinfo, fprop);
 
@@ -1105,7 +1105,7 @@ public abstract class AutomatedTestBase
 	 *            -1 there is no limit.
 	 */
 	protected void runTest(boolean exceptionExpected, Class<?> expectedException, int maxMRJobs) {
-		runTest(false, exceptionExpected, expectedException, maxMRJobs);
+		runTest(false, exceptionExpected, expectedException, null, maxMRJobs);
 	}
 
 	/**
@@ -1125,6 +1125,29 @@ public abstract class AutomatedTestBase
 	 *            -1 there is no limit.
 	 */
 	protected void runTest(boolean newWay, boolean exceptionExpected, Class<?> expectedException, int maxMRJobs) {
+		runTest(newWay, exceptionExpected, expectedException, null, maxMRJobs);
+	}
+
+	/**
+	 * <p>
+	 * Runs a test for which the exception expectation and the error message
+	 * can be specified as well as the specific expectation which is expected.
+	 * If SystemML executes more MR jobs than specified in maxMRJobs this test
+	 * will fail.
+	 * </p>
+	 * @param newWay
+	 * 			  in the new way if it is set to true
+	 * @param exceptionExpected
+	 *            exception expected
+	 * @param expectedException
+	 *            expected exception
+	 * @param errMessage
+	 * 		      expected error message
+	 * @param maxMRJobs
+	 *            specifies a maximum limit for the number of MR jobs. If set to
+	 *            -1 there is no limit.
+	 */
+	protected void runTest(boolean newWay, boolean exceptionExpected, Class<?> expectedException, String errMessage, int maxMRJobs) {
 
 		String executionFile = sourceDirectory + selectedTest + ".dml";
 
@@ -1227,6 +1250,12 @@ public abstract class AutomatedTestBase
 			if (exceptionExpected)
 				fail("expected exception which has not been raised: " + expectedException);
 		} catch (Exception e) {
+			if (errMessage != null && !errMessage.equals("")) {
+				boolean result = rCompareException(exceptionExpected, errMessage, e, false);
+				if (exceptionExpected && !result) {
+					fail(String.format("expected exception message '%s' has not been raised.", errMessage));
+				}
+			}
 			if (!exceptionExpected || (expectedException != null && !(e.getClass().equals(expectedException)))) {
 				e.printStackTrace();
 				StringBuilder errorMessage = new StringBuilder();
@@ -1242,6 +1271,16 @@ public abstract class AutomatedTestBase
 		}
 	}
 
+	private boolean rCompareException(boolean exceptionExpected, String errMessage, Throwable e, boolean result) {
+		if (e.getCause() != null) {
+			result |= rCompareException(exceptionExpected, errMessage, e.getCause(), result);
+		}
+		if (exceptionExpected && errMessage != null && e.getMessage().contains(errMessage)) {
+			result = true;
+		}
+		return result;
+	}
+
 	public void cleanupScratchSpace()
 	{
 		try
@@ -1732,11 +1771,8 @@ public abstract class AutomatedTestBase
 	 * @param bIncludeR
 	 *            generates also the corresponding R frame data
 	 * @throws IOException
-	 * @throws DMLRuntimeException
 	 */
-	protected double[][] writeInputFrame(String name, double[][] data, boolean bIncludeR, ValueType[] schema, OutputInfo oi)
-			throws DMLRuntimeException, IOException
-	{
+	protected double[][] writeInputFrame(String name, double[][] data, boolean bIncludeR, ValueType[] schema, OutputInfo oi) throws IOException {
 		String completePath = baseDirectory + INPUT_DIR + name;
 		String completeRPath = baseDirectory + INPUT_DIR + name + ".csv";
 
@@ -1759,16 +1795,12 @@ public abstract class AutomatedTestBase
 		return data;
 	}
 
-	protected double[][] writeInputFrameWithMTD(String name, double[][] data, boolean bIncludeR, ValueType[] schema, OutputInfo oi)
-			throws DMLRuntimeException, IOException
-	{
+	protected double[][] writeInputFrameWithMTD(String name, double[][] data, boolean bIncludeR, ValueType[] schema, OutputInfo oi) throws IOException {
 		MatrixCharacteristics mc = new MatrixCharacteristics(data.length, data[0].length, OptimizerUtils.DEFAULT_BLOCKSIZE, data[0].length, -1);
 		return writeInputFrameWithMTD(name, data, bIncludeR, mc, schema, oi);
 	}
 
-	protected double[][] writeInputFrameWithMTD(String name, double[][] data, boolean bIncludeR, MatrixCharacteristics mc, ValueType[] schema, OutputInfo oi)
-			throws DMLRuntimeException, IOException
-	{
+	protected double[][] writeInputFrameWithMTD(String name, double[][] data, boolean bIncludeR, MatrixCharacteristics mc, ValueType[] schema, OutputInfo oi) throws IOException {
 		writeInputFrame(name, data, bIncludeR, schema, oi);
 
 		// write metadata file
@@ -1798,10 +1830,9 @@ public abstract class AutomatedTestBase
 	 * @param schema
 	 * @param oi
 	 * @throws IOException
-	 * @throws DMLRuntimeException
 	 */
 	protected double[][] writeInputFrame(String name, double[][] data, ValueType[] schema, OutputInfo oi)
-			throws DMLRuntimeException, IOException
+			throws IOException
 	{
 		return writeInputFrame(name, data, false, schema, oi);
 	}
